@@ -18,6 +18,15 @@
     (format #t "~%~%Command ~A~%Tooltip ~A~%Label ~A~%Menu Path ~A~%" name help (d-GetLabel name) (d-GetMenuPath name))))
 ;;;;;;;;;;;;;;; 
 
+;;;;;;;;;;;;Replace a part of a string
+
+(define (replace-nth list n elem)
+  (cond 
+    ((null? list) ())
+    ((eq? n 0) (cons elem (cdr list)))
+    (#t (cons(car list) (replace-nth (cdr list) (- n 1) elem)))))
+
+
 ;;;;;;;;;;;;;; Get highest and lowest note as lilypond syntax. Works on Chords and Single Notes.
 ;;;;;;;;;;;;;; GetNotes returns a string of lily-notes from low to high. Make a list out of them and refer to the first (0) element or last (length -1) one.
 ;;;;;;;;;;;;;; Returns #f if not a chord
@@ -47,6 +56,50 @@
          (d-CursorDown)
          (d-CursorGoDown (+ x dx) x-max dx))))
 
+;Next Selected Object for all Staffs by Nils Gey Feb/2010
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;Moves the cursor to the right. If there is no selection, If that returns #f it will move down one staff and rewind to the start of the selection in this staff..
+;FIXME: After the last selected Item the cursor will move down one staff even outside the selection and will stay there. But a script with SingleAndSelectionSwitcher will NOT be applied to this outside-object and the user will not see this because the cursor is returned to the starting position afterwards.
+
+(define (NextSelectedObjectAllStaffs)
+	(if  (not (d-NextSelectedObject)) 
+		(if  (and (d-MoveToStaffDown) (d-NextSelectedObject) (d-PrevSelectedObject))
+		(begin 
+		 (let loop ()
+			(if (d-PrevSelectedObject)
+				(loop)))
+		#t); block end. 
+		#f ; if !StaffDown
+		); fi StaffDown
+	#t ;NextSelecetedObject was succesful
+	)
+)
+
+
+;SingleAndSelectionSwitcher by Nils Gey Jan/2010
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;Automatically applies a script to a whole selection. You can give different commands or command blocks with (begin) for single items or whole selections. You can enter a complete scheme script with (),  arguments and everything you would want to run standalone. Don't forget to escape chars like  \" . You can even use a complete (begin ) block.
+;But attention! SingleAndSelectionSwitcher will still try to apply the given script to each of the single items alone. If you need a script which differs completly in beaviour for single/selection you have to write your own. You have to take out the (let loop () section for this and write your own selection part there.
+;The applied script itself has to take care if the command can be applied to each potential item. If you want only notes/chords/rests you have to make sure the script does not abort on other objects. Its the same as giving proper return values for a single item, just return #f if a command is not possible for an item. While a single item just returns an error if you don't do it correctly, but does no harm otherwise, a script applied to a selection will stop on that item and leaves you on half on the way.
+;Return values are the return values the script itself gives.
+;Example: (SingleAndSelectionSwitcher  "(d-ChangeDurationByFactorTwo *)" "(d-ChangeDurationByFactorTwo *)")
+
+(define (SingleAndSelectionSwitcher commandsingle commandselection)
+(d-PushPosition)
+(if (and DenemoPref_applytoselection (d-GoToSelectionStart))
+(begin
+	(eval-string  commandselection)
+	(let loop ()
+	(if (NextSelectedObjectAllStaffs)
+	 	(begin (eval-string  commandselection) (loop))
+	))
+	(d-GoToSelectionStart)
+	(d-PopPosition)
+	)
+(begin	
+	(eval-string commandsingle)	)
+))
+;; End of SingleAndSelectionSwitcher
 
 (define Chord? (lambda ()
 		  (string=? (d-GetType) "CHORD")))
@@ -322,7 +375,7 @@
 ;;;; d-DirectivePut-standalone a convenience function for standalone directives
 (define (d-DirectivePut-standalone tag)
   (d-DirectivePut-standalone-minpixels tag 0)
-  (d-CursorLeft))
+  (d-MoveCursorLeft))
 
 (define (d-Directive-standalone tag)
   (if (not (d-Directive-standalone? tag))
@@ -706,4 +759,125 @@
 	(d-DirectivePut-standalone-postfix "Anacrusis" (string-append "\\partial 128*" (number->string duration) " " ))))))
 
 
-;;;;;;;;;;;;;;;;;Multi-Measure Rests
+;;;;;;;;;;;;;;;;;
+(define (DenemoFirst)
+  (begin
+    (display "DenemoFirst")))
+
+
+;;;;;;;;;;;;
+(define (DenemoGoBack)
+  (begin
+    (d-AdjustPlaybackStart -1.0)
+    (d-RefreshDisplay)))
+
+
+(define (DenemoPrevious)
+  (begin
+    (d-AdjustPlaybackEnd -1.0)
+    (d-RefreshDisplay)))
+
+
+(define (DenemoRewind)
+  (begin
+    (display "DenemoRewind")))
+
+(define (DenemoStop)
+  (begin
+    (set! Playback::Loop #f)
+    (d-Stop)))
+
+(define (DenemoPlay)
+  (begin
+    (d-Play "(display \"Here endeth a scripted playback\")")))
+
+(define (DenemoPause)
+  (begin
+    (display "DenemoPause")))
+
+(define (DenemoGoForward)
+  (begin
+    (d-AdjustPlaybackEnd 1.0)
+    (d-RefreshDisplay)))
+
+
+(define (DenemoNext)
+  (begin
+    (d-AdjustPlaybackStart 1.0)
+    (d-RefreshDisplay)))
+
+
+
+(define (DenemoForward)
+  (begin
+    (display "DenemoForward")))
+
+(define (DenemoLast)
+  (begin
+    (display "DenemoLast")))
+
+
+(define Playback::Loop #f)
+(define (DenemoLoop)
+  (begin
+    (display "DenemoLoop")
+    (set! Playback::Loop #t)
+    (d-Play "(if Playback::Loop (DenemoLoop))")
+))
+
+(define DenemoTempo::Value 1.0)
+(define (DenemoTempo)
+  (begin
+    (d-MasterTempo DenemoTempo::Value)))
+
+(define DenemoVolume::Value 1.0)
+(define (DenemoVolume)
+  (begin
+    (d-MasterVolume DenemoVolume::Value)))
+
+    
+(define (DenemoSetPlaybackStart)
+  (begin
+    (if (boolean? (d-GetMidiOnTime))
+	(d-RecreateTimebase))
+    (if (number? (d-GetMidiOnTime))
+	(d-SetPlaybackInterval (d-GetMidiOnTime) #t)
+	(d-RefreshDisplay))))
+
+(define (DenemoSetPlaybackEnd)
+  (begin
+    (if (boolean? (d-GetMidiOffTime))
+	(d-RecreateTimebase))
+    (if (number? (d-GetMidiOffTime))
+	(d-SetPlaybackInterval #t (d-GetMidiOffTime))
+	(d-RefreshDisplay))))
+
+(define (DenemoSetPlaybackIntervalToSelection)
+  (begin
+    
+    (let ((start #f)(end #f))
+      (set! end (d-GetMidiOffTime))
+      (if (boolean? end)
+	  (d-RecreateTimebase))
+      (set! end (d-GetMidiOffTime))
+      (if (boolean? end)
+	  (d-WarningDialog "End the selection at a note")
+	  (begin
+	    (d-GoToMark)
+	    (set! start (d-GetMidiOnTime))
+	    (if (boolean? start)
+		(d-WarningDialog "Start the selection at a note")
+		(begin
+		  (if (< end start)
+		      (d-SetPlaybackInterval end start)
+		      (d-SetPlaybackInterval start end))
+		  (d-RefreshDisplay))))))))
+
+
+(define (DenemoPrintAllHeaders)
+  (let ((lily "printallheaders"))
+    (if (d-CheckLilyVersion "2.11.59")
+	(set! lily "print-all-headers"))
+     (d-DirectivePut-paper-postfix "PrintAllHeaders" (string-append lily " = ##t\n"))))
+
+
