@@ -184,12 +184,13 @@ tupopen;
  */
 typedef enum clefs
 {
-  DENEMO_TREBLE_CLEF,
+  DENEMO_TREBLE_CLEF=0,
   DENEMO_BASS_CLEF,
   DENEMO_ALTO_CLEF,
   DENEMO_G_8_CLEF,
   DENEMO_TENOR_CLEF,
   DENEMO_SOPRANO_CLEF,
+  DENEMO_F_8_CLEF,
   DENEMO_INVALID_CLEF
 }clefs;
 
