@@ -125,12 +125,13 @@ typedef enum input_mode {
   INPUTCLASSIC = 1<<0, /* classic mode */
   INPUTEDIT = 1<<1, /* edit mode */
   INPUTINSERT = 1<<2, /* insert mode */
-#define ENTRY_TYPE_MASK (~(INPUTNORMAL|INPUTREST|INPUTBLANK|INPUTRHYTHM))
+#define ENTRY_TYPE_MASK (~(INPUTNORMAL|INPUTREST|INPUTBLANK))
   INPUTNORMAL = 1<<3, /* entry type notes */
   INPUTREST = 1<<4, /* entry type rests */
   INPUTBLANK = 1<<5,/* entry type non-printing rests */
   TRAVERSE = 1<<6, /* read-only */
-  INPUTRHYTHM = 1<<7, /*Input rhythms without pitches*/
+#define ENTRY_FEEDBACK_MASK (~(INPUTRHYTHM))
+  INPUTRHYTHM = 1<<7, /*Input rhythms gives feedback*/
 }input_mode;
 
 /**
@@ -335,15 +336,23 @@ typedef struct DenemoPrefs
 			     wav file after rendering or not */
   gboolean immediateplayback; /**< This options sends audio directly to synth as notes 
 				are being entered */
+  gboolean startmidiin; /**< try to start midi in on startup */
+  gboolean applytoselection; /**< apply commands to selection */
   gboolean overlays; /*< whether overlays or insert should be used with pitch entry */
   gboolean continuous; /*< whether pitch entry overlays should cross barlines */
   gboolean lilyentrystyle;  
   gboolean createclones;
+  gint mode; /**< mode to start in */
 
-  gboolean articulation_palette; /**< This switch makes the articulation pallete visable */
-  gboolean notation_palette; /**< This switch makes the Note/Rest entry toolbar visable */
-  gboolean rhythm_palette; /**< This option makes the rhythm toolbar visable */
-  gboolean object_palette;  /**< This option makes the object menu toolbar visable */
+  gint zoom; /**< percent zoom  1-100 */
+  gint system_height; /**< percent of display height per system  1-100 */
+
+  gboolean playback_controls; /**< makes the playback controls visible */
+  gboolean articulation_palette; /**< makes the articulation pallete visible */
+  gboolean notation_palette; /**< makes the Note/Rest entry toolbar visible */
+  gboolean rhythm_palette; /**< makes the rhythm toolbar visable */
+  gboolean object_palette;  /**< makes the object menu toolbar visible */
+  gboolean console_pane;  /**< makes the console pane visible */
 
   gboolean visible_directive_buttons; /**< This option makes the hbox containing score/movement directives visible */
 
@@ -366,12 +375,15 @@ typedef struct DenemoPrefs
   GArray *midi_device_array;/**< holds the midi_device array for re-sizing purposes */
   DeviceManagerDevice *midi_device; /**< contains device name and output ports */ 
   GString *fluidsynth_audio_driver; /**< Audio driver used by fluidsynth */
+  GString *fluidsynth_midi_driver; /**< MIDI driver used by fluidsynth */
   GString *fluidsynth_soundfont; /**< Default soundfont for fluidsynth */
   gboolean fluidsynth_reverb; /**< Toggle if reverb is applied to fluidsynth */
   gboolean fluidsynth_chorus; /**< Toggle if chorus is applied to fluidsynth */
   gint fluidsynth_sample_rate;/**< sample rate in Hz > */
   gint fluidsynth_period_size;/**< The size of the audio buffers (in frames).> */
 
+  gdouble display_refresh;/**< time in ms between refresh of display during playback */
+
   GString *pdfviewer; /**< PDF viewer */
   GString *imageviewer; /**< Image Viewer */
   GString *username; /**< Username for use on denemo.org website */
@@ -639,6 +651,17 @@ typedef struct DenemoScore
   gint widthtoworkwith;
   gint staffspace;
 
+
+  gdouble start_time; /**< time in seconds to start playing at */
+  gdouble end_time; /**< time to end playing at */
+  DenemoObject *playingnow; /**< the last object played via MIDI; it must not be dereferenced as it may no longer exist */
+  gdouble start_player;/**< system time when MIDI player started */
+  gdouble master_volume;/**< the volume (velocity) used is this times the nominal vol */
+  gdouble master_tempo;/**< the tempo used is this times the nominal tempo */
+  gdouble tempo_change_time;/**< system time from which the master_tempo is to be used */
+  gdouble rightmost_time; /**< MIDI time of rightmost visible object */
+  gdouble zoom;
+  gdouble system_height;/**< fraction of scorearea height allotted to one system (line) in the score, the same for all lines */
   /* Fields that have more to do with the data model and its manipulation,
    * though they may be modified by side-effects of the drawing routines */
   // score thescore;
@@ -742,6 +765,9 @@ typedef struct DenemoGUI
   GtkWidget *vscrollbar;
   GtkObject *hadjustment;
   GtkWidget *hscrollbar;
+#define DENEMO_MAX_SYSTEMS (100) /**< Number of lines of music that can be displayed */
+  gint lefts[DENEMO_MAX_SYSTEMS];/**< an array to hold the leftmeasurenum of each system in the last-drawn score, used for determining the mouse position on the music */
+  gint rights[DENEMO_MAX_SYSTEMS];/**< an array to hold the rightmeasurenum of each system in the last-drawn score, used for determining the mouse position on the music */
 
   GtkWidget *printarea;/**< area holding a print preview */
   GtkWidget *printvscrollbar;/**< scrollbar widget for printarea */
@@ -750,6 +776,7 @@ typedef struct DenemoGUI
   GtkWidget *buttonboxes;/**< box for boxes showing directives */
   GtkWidget *buttonbox;/**< box for buttons accessing DenemoDirectives attached to the whole score */
 
+
   gchar *xbm; /**< xbm representation of graphic bitmap from selected rectangle in print preview area*/
   gint xbm_width, xbm_height;/**< width and height of the xbm data */
   GtkWidget *textwindow; /**< LilyPond output window */
@@ -764,8 +791,6 @@ typedef struct DenemoGUI
   input_mode mode; /**< Input mode for Score */
   GtkWidget *progressbar;
 
-
-
   GList *movements;   /**< a list of DenemoScore, NULL if just one movement */
   DenemoScore *si;  /**< the (current)  movement in the musical score controlled by this gui */
   DenemoLilyControl lilycontrol; /**< Directives for the start of the score and before every movement */
@@ -831,7 +856,7 @@ struct cs_callback
 	
 };
 
-static gchar* ext_pidfiles[] = {"midiplayer.pid", "csoundplayer.pid", NULL};
+//static gchar* ext_pidfiles[] = {"midiplayer.pid", "csoundplayer.pid", NULL};
 
 /** 
  * The (singleton) root object for the program
@@ -858,6 +883,7 @@ struct DenemoRoot
   DenemoGUI *gui; /**< The current gui */
   GtkWidget *notebook;/**< contains the gui.page widgets */
   GtkWidget *statusbar;
+  GtkWidget *playback_control;/**< frame containing controls for playback */
   gint status_context_id;
   GtkWidget *input_source; /**< A label widget advising of source of external input */
   GString *input_filters; /**< Description of any filters operating on external input */
