@@ -182,16 +182,18 @@ void play_pitch (double pitch, double duration, double volume, int channel) {
 		      SAMPLE_RATE,
 		      1024,            /* frames per buffer */
 		      0,               /* number of buffers, if zero then use default minimum */
+		      paClipOff,       /* we won't output out of range samples so don't bother clipping them */
+		      playCallback,
+		      &out_data );
 #else
-
 		      NULL,		/* input parameters */
 		      &outputParameters,
 		      SAMPLE_RATE,
 		      1024,            /* frames per buffer */
-#endif
 		      paClipOff,       /* we won't output out of range samples so don't bother clipping them */
-		      playCallback,
+		      (PaStreamCallback *)playCallback,
 		      &out_data );
+#endif
   if( err != paNoError ) {
     g_print("Error opening stream\n");
     return;
@@ -217,7 +219,7 @@ int init_audio_out(void)
 	err = Pa_Initialize();
 	if( err != paNoError ) {
 	  g_print("Error initializing portaudio library\n");
-	  return;
+	  return 0;
 	}
 	out_data.frameIndex = 0;
 	out_data.recordedSamples = (SAMPLE *) malloc( TABLE_SIZE*sizeof(SAMPLE) );
@@ -299,6 +301,7 @@ static int recordCallback(const	void *inputBuffer,const void *outputBuffer,
 
 int collect_data_for_tuning(int ok) {
   tuning = ok;
+  return 1;
 }
 
 /******************************************************************
@@ -373,15 +376,18 @@ int pa_main(AubioCallback *fn)
 				SAMPLE_RATE,
 				1024,            /* frames per buffer */
 				0,               /* number of buffers, if zero then use default minimum */
+				paClipOff,  /* we won't output out of range samples so don't bother clipping them */
+				recordCallback,
+				NULL );
 #else
 				&inputParameters,
 				NULL,		/* output parameters */
 				SAMPLE_RATE,
 				1024,            /* frames per buffer */
-#endif
 				paClipOff,  /* we won't output out of range samples so don't bother clipping them */
-				recordCallback,
+				(PaStreamCallback *)recordCallback,
 				NULL );
+#endif
 	if( err != paNoError ) goto error;
 
 	err = Pa_StartStream( stream );
