@@ -6,6 +6,7 @@
  */
 
 #include <string.h>
+#include <stdint.h>
 #include "commandfuncs.h"
 #include "calculatepositions.h"
 #include "chordops.h"
@@ -27,6 +28,12 @@
 #include "prefops.h"
 #include "keyresponses.h"
 #include "view.h"
+#include "pitchentry.h"
+
+#ifdef _HAVE_FLUIDSYNTH_
+#include "fluid.h"
+#endif
+
 /**
  * Macro to get the current DenemoObject
  */
@@ -121,7 +128,7 @@ beamandstemdirhelper (DenemoScore * si)
 /**
  * Set si->current* variables from currentmeasurenum
  * the current object is set to the first in the measure
- * the selection is updated
+ * the selection is not updated
  *
  */
 void
@@ -155,7 +162,7 @@ setcurrents (DenemoScore * si)
     si->cursor_appending = FALSE;
   else
     si->cursor_appending = TRUE;
-  calcmarkboundaries (si);
+  // calcmarkboundaries (si);
 }
 
 /**
@@ -199,10 +206,28 @@ void
 set_width_to_work_with (DenemoGUI * gui)
 {
   GList *g; 
-  for(g = gui->movements;g;g=g->next)
- ((DenemoScore*) g->data)->widthtoworkwith
-    = (gui->scorearea->allocation.width
-       - (RIGHT_MARGIN + KEY_MARGIN + gui->si->maxkeywidth + SPACE_FOR_TIME));
+  
+    for(g = gui->movements;g;g=g->next) {
+      DenemoScore *si =  ((DenemoScore*) g->data);
+
+#if 0
+      si->widthtoworkwith
+	= (double)(gui->scorearea->allocation.width*((int)(1/si->system_height ))/si->zoom
+		   - (RIGHT_MARGIN + KEY_MARGIN + si->maxkeywidth + SPACE_FOR_TIME));
+#else
+      //the total line length for drawing DenemoObjects onto the screen
+      // this length will be divided amongst the systems (line).
+      // This length is in "pixels", the Denemo unit of display, which corresponds to a screen pixel when zoom ==1.0
+      si->widthtoworkwith
+	= (gint)((gui->scorearea->allocation.width/si->zoom
+	   - (RIGHT_MARGIN + KEY_MARGIN + si->maxkeywidth + SPACE_FOR_TIME))*((int)(1/si->system_height )));
+
+#endif
+
+
+
+      //g_print("Width %d from num systems%d\n", si->widthtoworkwith, ((int)(1/si->system_height )));
+    }
 }
 
 /**
@@ -338,8 +363,8 @@ adjuststaffheight (DenemoScore * si, gint amount)
  * Move an entire measure to the left
  *
  */
-void
-measureleft (DenemoScriptParam *param)
+static void
+gomeasureleft (DenemoScriptParam *param, gboolean extend_selection)
 {
   DenemoGUI *gui = Denemo.gui;
   DenemoScore *si = gui->si;
@@ -347,26 +372,33 @@ measureleft (DenemoScriptParam *param)
   if(param==NULL)
     param = &dummy;
   param->status = FALSE;
-
-  if (
-#if 0 //this prevents deletion going past empty measures
-!gui->si->cursor_x && 
-#endif
-gui->si->currentmeasure->prev)
+  if(extend_selection && !si->markstaffnum) set_mark(gui);
+  if (gui->si->currentmeasure->prev)
     {
       gui->si->currentmeasurenum--;
       isoffleftside (gui);
       param->status = TRUE;
     }
   setcurrents (gui->si);
+  if(extend_selection)
+    calcmarkboundaries (gui->si);
+}
+/**
+ * Move an entire measure to the left
+ *
+ */
+void
+measureleft (DenemoScriptParam *param)
+{
+  gomeasureleft(param, TRUE);
 }
 
 /**
  * Move an entire measure to the right
  *
  */
-void
-measureright (DenemoScriptParam *param)
+static void
+gomeasureright (DenemoScriptParam *param, gboolean extend_selection)
 {
   DenemoGUI *gui = Denemo.gui;
   DenemoScore *si = gui->si;
@@ -374,16 +406,35 @@ measureright (DenemoScriptParam *param)
   if(param==NULL)
     param = &dummy;
   param->status = FALSE;
-
+  if(extend_selection && !si->markstaffnum) set_mark(gui);
   if (gui->si->currentmeasure->next)
     {
       gui->si->currentmeasurenum++;
       isoffrightside (gui);
       setcurrents (gui->si);
       param->status = TRUE;
+      if(extend_selection)
+	calcmarkboundaries (gui->si);
     }
 }
 
+void
+measureright (DenemoScriptParam *param)
+{
+  gomeasureright(param, TRUE);
+}
+
+void
+movetomeasureright (DenemoScriptParam *param)
+{
+  gomeasureright(param, FALSE);
+}
+void
+movetomeasureleft (DenemoScriptParam *param)
+{
+  gomeasureleft(param, FALSE);
+}
+
 
 /**
  * swap the current movement for the previous one in the list of movements
@@ -513,9 +564,10 @@ joinstaffs (GtkAction *action, gpointer param)
 /**
  * Move si->currentstaff up an one voice, return TRUE if successful
  * param is NULL for interactive calls, otherwise status is returned in param->status;
+ * alter selection if extend_selection
  */
-gboolean
-voiceup (DenemoScriptParam *param)
+static gboolean
+govoiceup (DenemoScriptParam *param, gboolean extend_selection)
 {
   DenemoGUI *gui = Denemo.gui;
   DenemoScore *si = gui->si;
@@ -525,6 +577,7 @@ voiceup (DenemoScriptParam *param)
   param->status = FALSE;
   if(!gui->si->currentstaff)
     return param->status = FALSE;
+  if(extend_selection && !si->markstaffnum) set_mark(gui);
   if (gui->si->currentstaff && (((DenemoStaff *)(gui->si->currentstaff->data))->voicenumber==2))  {
     hide_lyrics();
     gui->si->currentstaffnum--;
@@ -541,11 +594,11 @@ voiceup (DenemoScriptParam *param)
 }
 
 /**
- * Move si->currentstaff up an one staff/voice, return TRUE if successful
+ * Move si->currentstaff up an one staff, return TRUE if successful
  *
  */
-gboolean
-staffup (DenemoScriptParam *param)
+static gboolean
+gostaffup (DenemoScriptParam *param, gboolean extend_selection)
 {
   DenemoGUI *gui = Denemo.gui;
   DenemoScore *si = gui->si;
@@ -555,8 +608,9 @@ staffup (DenemoScriptParam *param)
   param->status = FALSE;
   if(!gui->si->currentstaff)
     return param->status = FALSE;
+  if(extend_selection && !si->markstaffnum) set_mark(gui);
   while (((DenemoStaff *)(gui->si->currentstaff->data))->voicenumber!=1)
-    voiceup(param);//FIXME check param->status
+    govoiceup(param, extend_selection);//FIXME check param->status
   if (gui->si->currentstaff->prev)
     {
       hide_lyrics();
@@ -564,6 +618,8 @@ staffup (DenemoScriptParam *param)
       gui->si->currentstaff = gui->si->currentstaff->prev;
       setcurrentprimarystaff (gui->si);
       setcurrents (gui->si);
+      if(extend_selection)
+	calcmarkboundaries (gui->si);
       show_lyrics();
       find_leftmost_allcontexts (si);
       gtk_widget_draw (gui->scorearea, NULL);//KLUDGE FIXME gets cursorclef set I think it just needs si->cursorclef = si->curmeasureclef (? after beamandstemdirhelper(si)??)
@@ -577,11 +633,11 @@ staffup (DenemoScriptParam *param)
 
 
 /**
- * Move si->currentstaff down an one staff/voice, return TRUE if successful
- *
+ * Move si->currentstaff down one voice, return TRUE if successful
+ * alter selection if extend_selection
  */
-gboolean
-voicedown (DenemoScriptParam *param)
+static gboolean
+govoicedown (DenemoScriptParam *param, gboolean extend_selection)
 {
   DenemoGUI *gui = Denemo.gui;
   DenemoScore *si = gui->si;
@@ -591,12 +647,15 @@ voicedown (DenemoScriptParam *param)
   param->status = FALSE;
   if(!gui->si->currentstaff)
     return param->status = FALSE;
+  if(extend_selection && !si->markstaffnum) set_mark(gui);
   if (gui->si->currentstaff->next && ((DenemoStaff *)(gui->si->currentstaff->next->data))->voicenumber==2) {
       hide_lyrics();
       gui->si->currentstaffnum++;
       gui->si->currentstaff = gui->si->currentstaff->next;
       setcurrentprimarystaff (gui->si);
       setcurrents (gui->si);
+      if(extend_selection)
+	calcmarkboundaries (gui->si);
       show_lyrics();
       move_viewport_down (gui);
       return param->status = TRUE;
@@ -605,12 +664,43 @@ voicedown (DenemoScriptParam *param)
       warningmessage("This is the last voice");
   return param->status = FALSE;
 }
+
+gboolean
+movetovoicedown (DenemoScriptParam *param)
+{
+  
+  return govoicedown(param, FALSE);
+}
+
+gboolean
+voicedown (DenemoScriptParam *param)
+{
+  
+  return govoicedown(param, TRUE);
+}
+
+gboolean
+movetovoiceup (DenemoScriptParam *param)
+{
+  
+  return govoiceup(param, FALSE);
+}
+
+gboolean
+voiceup (DenemoScriptParam *param)
+{
+  
+  return govoiceup(param, TRUE);
+}
+
+
+
 /**
- * Move si->currentstaff down an one staff/voice, return TRUE if successful
+ * Move si->currentstaff down one staff/voice, return TRUE if successful
  *
  */
-gboolean
-staffdown (DenemoScriptParam *param)
+static gboolean
+gostaffdown (DenemoScriptParam *param, gboolean extend_selection)
 {
   DenemoGUI *gui = Denemo.gui;
   DenemoScore *si = gui->si;
@@ -621,8 +711,9 @@ staffdown (DenemoScriptParam *param)
 
   if(!gui->si->currentstaff)
     return param->status = FALSE;
+  if(extend_selection && !si->markstaffnum) set_mark(gui);
   while (gui->si->currentstaff->next && ((DenemoStaff *)(gui->si->currentstaff->next->data))->voicenumber==2)
-    voicedown(param);//FIXME
+    govoicedown(param, extend_selection);//FIXME
   if (gui->si->currentstaff->next)
     {
       hide_lyrics();
@@ -630,6 +721,8 @@ staffdown (DenemoScriptParam *param)
       gui->si->currentstaff = gui->si->currentstaff->next;
       setcurrentprimarystaff (gui->si);
       setcurrents (gui->si);
+      if(extend_selection)
+	calcmarkboundaries (gui->si);
       show_lyrics();
       find_leftmost_allcontexts (si);
       gtk_widget_draw (gui->scorearea, NULL);//KLUDGE FIXME gets cursorclef set
@@ -641,14 +734,42 @@ staffdown (DenemoScriptParam *param)
   return param->status = FALSE;
 }
 
+gboolean
+movetostaffdown (DenemoScriptParam *param)
+{
+  
+  return gostaffdown(param, FALSE);
+}
+
+gboolean
+staffdown (DenemoScriptParam *param)
+{
+  
+  return gostaffdown(param, TRUE);
+}
+
+gboolean
+movetostaffup (DenemoScriptParam *param)
+{
+  
+  return gostaffup(param, FALSE);
+}
+
+gboolean
+staffup (DenemoScriptParam *param)
+{
+  
+  return gostaffup(param, TRUE);
+}
+
 
 
 /**
  * move the cursor one position to the left
  *
  */
-void
-cursorleft (DenemoScriptParam *param)
+gboolean
+move_left (DenemoScriptParam *param, gboolean extend_selection)
 {
   DenemoGUI *gui = Denemo.gui;
   DenemoScore *si = gui->si;
@@ -656,6 +777,7 @@ cursorleft (DenemoScriptParam *param)
   if(param==NULL)
     param = &dummy;
   param->status = FALSE;
+  if(extend_selection && !si->markstaffnum) set_mark(gui);
   g_debug("cursorleft: cursorpos %d\n", si->cursor_x);
   if (!si->cursor_x)
     {
@@ -695,16 +817,19 @@ cursorleft (DenemoScriptParam *param)
 	  param->status = TRUE;
 	}
     }
-  calcmarkboundaries (si);
+  if(extend_selection)
+    calcmarkboundaries (si);
   write_status(gui);
+  return param->status;
 }
 
 /**
  * move the cursor one position to the right
- * sets param->status TRUE if cursor could still move right.
+ * selection (if any) is extended if extend_selection is TRUE
+ * sets param->status TRUE if cursor could still move right, and returns that value.
  */
 gboolean
-cursorright (DenemoScriptParam *param)
+move_right (DenemoScriptParam *param, gboolean extend_selection)
 {
   DenemoGUI *gui = Denemo.gui;
   DenemoScore *si = gui->si;
@@ -712,6 +837,7 @@ cursorright (DenemoScriptParam *param)
   if(param==NULL)
     param = &dummy;
   param->status = FALSE;
+  if(extend_selection && !si->markstaffnum) set_mark(gui);
   if (si->cursor_appending && si->currentmeasure->next)
     {
       /* Go to the next measure */
@@ -740,13 +866,45 @@ cursorright (DenemoScriptParam *param)
 	}
       else return param->status = FALSE;
     }
-  calcmarkboundaries (si);
+  if(extend_selection)
+    calcmarkboundaries (si);
   write_status(gui);
   return (param->status = (si->currentobject || (!si->cursor_appending) || si->currentmeasure->next));
 }
 
 /**
- * Move the cursor up one position 
+ * move the cursor one position to the right extending the selection if any
+ * sets param->status TRUE if cursor could still move right, and returns that value.
+ */
+gboolean
+cursorright (DenemoScriptParam *param)
+{
+return  move_right(param, TRUE);
+}
+/**
+ * move the cursor one position to the left extending the selection if any
+ * sets param->status TRUE if cursor could still move left, and returns that value.
+ */
+gboolean
+cursorleft (DenemoScriptParam *param)
+{
+return  move_left(param, TRUE);
+}
+// cursor move without altering selection
+gboolean
+movecursorright (DenemoScriptParam *param)
+{
+return  move_right(param, FALSE);
+}
+gboolean
+movecursorleft (DenemoScriptParam *param)
+{
+return  move_left(param, FALSE);
+}
+
+
+/**
+ * Move the cursor up one diatonic step 
  */
 void
 cursorup (DenemoScriptParam *param)
@@ -764,7 +922,7 @@ cursorup (DenemoScriptParam *param)
 }
 
 /**
- * Move the cursor down one position 
+ * Move the cursor down one diatonic step 
  */
 void
 cursordown (DenemoScriptParam *param)
@@ -1096,7 +1254,7 @@ notechange (DenemoScore * si, gboolean remove)
       if (remove == TRUE)
 	ret = removetone (curmudelaobj, si->cursor_y/*mid_c_offset*/, si->cursorclef/*dclef*/);
       else
-	ret = (gboolean)addtone (curmudelaobj, si->cursor_y/* mid_c_offset*/,
+	ret = (gboolean)(intptr_t)addtone (curmudelaobj, si->cursor_y/* mid_c_offset*/,
 		 si->cursoraccs[si->staffletter_y]/* enshift */, si->cursorclef /*dclef*/);
 
       if (curmudelaobj->user_string)
@@ -1500,6 +1658,7 @@ void deletemeasureallstaffs(DenemoGUI * gui)
   si->currentmeasure =
     removemeasures (si, si->currentmeasurenum - 1, 1, TRUE);
   setcurrents (si);
+calcmarkboundaries (si);
   score_status(gui, TRUE);
   si->markstaffnum = 0;
   isoffleftside (gui);
@@ -1526,6 +1685,7 @@ dnm_deletemeasure (DenemoScore * si)
    * the current measure to be the left of what's displayed */
 
   setcurrents (si);
+calcmarkboundaries (si);
   si->markstaffnum = 0;
 
 
@@ -1690,6 +1850,11 @@ dnm_deleteobject (DenemoScore * si)
 	  //    case COMMENT:
 	case MEASUREBREAK:
 	  break;
+	// XXX: unhandled... 
+	case STAFFBREAK:
+	case FAKECHORD:
+	case PARTIAL:
+	  break;
 	}
       si->markstaffnum = 0;
     }
@@ -1741,39 +1906,45 @@ toggle_tie (GtkAction *action, gpointer param)
 
 
 /**
- * Move cursor to the end of the score 
- * @param action - Gtk Action event 
- * @param gui - pointer to the DenemoGUI structure
+ * Move cursor to the end of the score  extending the selection if extend_selection is TRUE
+ * @param param - pointer to a script parameter structure
  * @return none
  */
-void
-toend (GtkAction *action, gpointer param)
+static void
+gotoend (gpointer param, gboolean extend_selection)
 {
   DenemoGUI *gui = Denemo.gui;
-  gui->si->currentmeasurenum = gui->si->leftmeasurenum =
-    gui->si->rightmeasurenum =
+  if(extend_selection && !gui->si->markstaffnum) set_mark(gui);
+  gint dest = gui->si->currentmeasurenum = 
     g_list_length (((DenemoStaff *) gui->si->currentstaff->data)->measures);
   setcurrents (gui->si);
+  if(extend_selection)
+    calcmarkboundaries (gui->si);
   tolastobject(gui);
-  cursorright(param);
+  if(extend_selection)
+    cursorright(param);
+  else
+    movecursorright(param);
   find_leftmost_allcontexts (gui->si);
   update_hscrollbar (gui);
   displayhelper (gui);
 }
 
 /**
- * Move the cursor to the beginning of the score 
- * @param action - Gtk Action event
- * @param gui - pointer to the DenemoGUI structure
+ * Move the cursor to the beginning of the score extending the selection if extend_selection is TRUE
+ * @param param - pointer to a script parameter structure
  * @return none
 */
-void
-tohome (GtkAction *action, gpointer param)
+static void
+gotohome (gpointer param, gboolean extend_selection)
 {
   DenemoGUI *gui = Denemo.gui;
+  if(extend_selection && !gui->si->markstaffnum) set_mark(gui);
   gui->si->currentmeasurenum = gui->si->leftmeasurenum = 1;
   set_rightmeasurenum (gui->si);
   setcurrents (gui->si);
+  if(extend_selection)
+    calcmarkboundaries (gui->si);
   find_leftmost_allcontexts (gui->si);
   update_hscrollbar (gui);
   /*gtk_widget_draw (gui->scorearea, NULL);*/
@@ -1781,6 +1952,60 @@ tohome (GtkAction *action, gpointer param)
 }
 
 
+/**
+ * Move the cursor to the beginning of the score, extending the selection if any. 
+ * @param action - Gtk Action event
+ * @param 
+ * @return none
+*/
+void
+tohome (GtkAction *action, gpointer param)
+{
+  gotohome(param, TRUE);
+}
+
+
+/**
+ * Move the cursor to the end of the score, extending the selection if any. 
+ * @param action - Gtk Action event
+ * @param 
+ * @return none
+*/
+void
+toend (GtkAction *action, gpointer param)
+{
+  gotoend(param, TRUE);
+}
+
+
+
+
+/**
+ * Move the cursor to the beginning of the staff, without extending the selection if any. 
+ * @param action - Gtk Action event
+ * @param 
+ * @return none
+*/
+void
+movetostart (GtkAction *action, gpointer param)
+{
+  gotohome(param, FALSE);
+}
+
+/**
+ * Move the cursor to the end of the staff, without extending the selection if any. 
+ * @param action - Gtk Action event
+ * @param 
+ * @return none
+*/
+void
+movetoend (GtkAction *action, gpointer param)
+{
+  gotoend(param, FALSE);
+}
+
+
+
 /**
  * Insert stem directive,  absolute stemdirection for the 
  * entire staff or until a new stem directive in added
