@@ -8,6 +8,7 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <math.h>
 #include "calculatepositions.h"
 #include "commandfuncs.h"
 #include "contexts.h"
@@ -18,10 +19,12 @@
 #include "hairpin.h"
 #include "staffops.h"
 #include "utils.h"
-#include "../pixmaps/toomany.xbm"
 #include "exportlilypond.h"	/* to generate lily text for display */
 #include "lyparserfuncs.h"	/* to generate lily text for display */
 #include "pitchentry.h"
+#include "lyric.h"
+#include "midi.h"
+
 /**
  * defines function to choose the correct 
  * depending upon the GTK version
@@ -30,6 +33,7 @@
 #define EXCL_WIDTH 3
 #define EXCL_HEIGHT 13
 
+#if 0
 static GdkGC *  bluegc;
 static GdkGC *  redgc;
 static GdkGC *  graygc;
@@ -37,7 +41,7 @@ static GdkGC *  slategraygc;
 static GdkGC *  lightbluegc;
 static GdkGC *  blackgc;
 static GdkGC *greengc = NULL;
-
+#endif
 GdkPixbuf *StaffPixbuf, *StaffPixbufSmall;
 
 static void      
@@ -59,15 +63,14 @@ gint
 scorearea_configure_event (GtkWidget * widget, GdkEventConfigure * event)
 {
   DenemoGUI *gui = Denemo.gui;
-  bluegc = gcs_bluegc();
-  redgc = gcs_redgc();
-  graygc = gcs_graygc();
-  slategraygc = gcs_slategraygc();
-  lightbluegc = gcs_lightbluegc();
-  blackgc = gcs_blackgc();
-  greengc = gcs_greengc();
-  create_tool_pixbuf();
-
+  if(gui==NULL)
+    return TRUE;
+  if(gui->si==NULL)
+    return TRUE;
+  static gboolean init = FALSE;
+  if(!init) {
+    create_tool_pixbuf();
+  }
   /* Create a new backing pixmap of the appropriate size */
   if (gui->pixmap)
     gdk_pixmap_unref (gui->pixmap);
@@ -101,11 +104,11 @@ struct infotopass
   gint measurenum;//would need measurenum_adj to allow control of numbering after pickup etc...
   gint staffnum;
   gint top_y;
-  gint y;
+  //gint y;
   gint markx1, markx2;
   gint marky1, marky2;
-  /*GString *dynamic; */
-  GtkWidget *widget;
+  gboolean line_end;//set true when an object is drawn off the right hand edge
+
   measurenode *curmeasure;
   GList *mwidthiterator;
   GSList *slur_stack;
@@ -118,7 +121,13 @@ struct infotopass
   gint in_highy; // FIXME these are passed in so that highy, lowy do not need to be passed back
   gint in_lowy;
   gboolean mark;//whether the region is selected
-  gboolean *wronglengths;// array of indicators for each measure drawn whether it is correct length 
+  gint *left, *right;//location of right and left measurenum for current system(line)
+  GList *last_midi;//last list of midi events for object at right of window
+  DenemoObject *startobj;//pointer values - if drawing such an object mark as playback start
+  DenemoObject *endobj;//pointer values - if drawing such an object mark as playback end
+  gint startposition;//x coordinate where to start playing
+  gint endposition;//x coordinate where to end playing
+  gint playposition;//x coordinate of currently played music
 };
 
 /* count the number of syllables up to staff->leftmeasurenum */
@@ -163,7 +172,7 @@ static gint count_syllables(DenemoStaff *staff, gint from) {
  * @return excess ticks in the measure at this object. (Negative means still space).
  */
 static gint
-draw_object (objnode * curobj, gint x, gint y,
+draw_object (cairo_t *cr, objnode * curobj, gint x, gint y,
 	     DenemoGUI * gui, struct infotopass *itp)
 {
 
@@ -186,12 +195,41 @@ draw_object (objnode * curobj, gint x, gint y,
   itp->highy = itp->lowy = 0;
   DenemoScore *si = gui->si;
   DenemoObject *mudelaitem = (DenemoObject *) curobj->data;
+
+  //FIXME too much save and restore for trifling reasons...
+  if(itp->mark) {
+    cairo_save(cr);
+    cairo_set_source_rgb( cr, 0.5, 0.5, 1.0 );
+    cairo_rectangle (cr, x+mudelaitem->x, y, 20, 80 );
+    cairo_fill(cr);
+    cairo_restore(cr);
+  }
+
+
+
+
+
+
+
+
+
+
+
+
+
   /* The current note, rest, etc. being painted */
   gint extra;
+  if(mudelaitem == Denemo.gui->si->playingnow)
+    itp->playposition = x + mudelaitem->x;
+  
+  if(mudelaitem == itp->startobj)
+    itp->startposition = x + mudelaitem->x +  mudelaitem->minpixelsalloted;
+  if(mudelaitem == itp->endobj)
+    itp->endposition = x + mudelaitem->x;
 
-
-  if (!greengc)
-    greengc = gcs_greengc ();
+  //g_print("item %p draw at %d\n", mudelaitem, itp->playposition);
+  // if (!greengc)
+  //   greengc = gcs_greengc ();
   /* Should we set cursor-context info before drawing? */
 
   /************ FIXME the drawing is side-effecting the DenemoScore si here *******************/
@@ -202,17 +240,14 @@ draw_object (objnode * curobj, gint x, gint y,
 	memcpy (si->cursoraccs, itp->curaccs, SEVENGINTS);
     }
 
-  {
-    
-    GdkColor *thecolor;
-    if(mudelaitem->type==CHORD && ((chord *) mudelaitem->object)->tone_node)
-      thecolor = &yellow;
-    else
-      thecolor =/* (mudelaitem->isinvisible) ? &white :*/ itp->mark?&blue:&black;
-    gdk_gc_set_foreground (blackgc, thecolor);
+  
+  if(mudelaitem->type==CHORD && ((chord *) mudelaitem->object)->tone_node)
+    cairo_set_source_rgb( cr, 231.0/255, 215.0/255, 39.0/255 );//thecolor = &yellow;
+  
+  if(mudelaitem->midi_events) {
+    itp->last_midi = mudelaitem->midi_events;
   }
 
-
   switch (mudelaitem->type)
     {
     case CHORD:
@@ -223,15 +258,14 @@ draw_object (objnode * curobj, gint x, gint y,
 	//	     gtk_style_get_font (itp->widget->style),
 	//	     x + mudelaitem->x, y, mudelaitem);
   
-	draw_figure (gui->pixmap, itp->gc,
-		     gtk_style_get_font (itp->widget->style),
+	draw_figure ( cr,
 		     x + mudelaitem->x,
 		     y + (thechord->lowesty / 2),
 		     mudelaitem);
   
       else
 	{
-	  draw_chord (gui->pixmap, itp->gc, curobj, x + mudelaitem->x, y,
+	  draw_chord ( cr, curobj, x + mudelaitem->x, y,
 		      GPOINTER_TO_INT (itp->mwidthiterator->data),
 		      itp->curaccs, itp->mark);
 	  if((thechord->highesty) < itp->highy)
@@ -245,15 +279,14 @@ draw_object (objnode * curobj, gint x, gint y,
 
 	}
        if (thechord->is_fakechord)
-	draw_fakechord (gui->pixmap, itp->gc,
-		     gtk_style_get_font (itp->widget->style),
+	draw_fakechord (cr,
 		     x + mudelaitem->x, 
 		     y - 45,
 		     //y - (thechord->highesty ), 
 		     mudelaitem);
        else
 	{
-	  draw_chord (gui->pixmap, itp->gc, curobj, x + mudelaitem->x, y,
+	  draw_chord ( cr, curobj, x + mudelaitem->x, y,
 		      GPOINTER_TO_INT (itp->mwidthiterator->data),
 		      itp->curaccs, itp->mark);
 	}
@@ -263,22 +296,20 @@ draw_object (objnode * curobj, gint x, gint y,
 	   && !itp->slur_stack
 	   && !thechord->is_tied)
 	{
-	  gchar *syllable = (gchar *) next_syllable();
+	  gchar *syllable = (gchar *) next_syllable(0);
 	  if(syllable)
-	    draw_lyric (gui->pixmap, itp->gc,
-			gtk_style_get_font (itp->widget->style),
+	    draw_lyric (cr,
 			x + mudelaitem->x,
-			y + thechord->lowesty,
+			y + itp->in_lowy,
 			syllable);
 	}
 
       if (thechord->dynamics)
-	draw_dynamic (gui->pixmap, itp->gc,
-		      gtk_style_get_font (itp->widget->style),
+	draw_dynamic (cr,
 		      x + mudelaitem->x, y, mudelaitem);
 
       if (thechord->slur_end_p)
-	draw_slur (gui->pixmap, itp->gc, &(itp->slur_stack),
+	draw_slur (cr, &(itp->slur_stack),
 		   x + mudelaitem->x, y);
       if (thechord->slur_begin_p)
 	itp->slur_stack =
@@ -302,7 +333,7 @@ draw_object (objnode * curobj, gint x, gint y,
 		 "removing the crescendo end");
 #endif
 	    }
-	  draw_hairpin (gui->pixmap, itp->gc, &(itp->hairpin_stack),
+	  draw_hairpin (cr, &(itp->hairpin_stack),
 			x + mudelaitem->x, y, 1);
 	}
       else if (thechord->diminuendo_end_p)
@@ -317,20 +348,18 @@ draw_object (objnode * curobj, gint x, gint y,
 		 "removing the diminuendo end");
 #endif
 	    }
-	  draw_hairpin (gui->pixmap, itp->gc, &(itp->hairpin_stack),
+	  draw_hairpin (cr, &(itp->hairpin_stack),
 			x + mudelaitem->x, y, 0);
 	}
 	/* notice the following does not check is_figure but checks if figure is not VOID) */      
       if (!thechord->is_figure && thechord->figure)
       //if (thechord->figure)
-        draw_figure (gui->pixmap, itp->gc,
-		     gtk_style_get_font (itp->widget->style),
+        draw_figure (cr,
 		     x + mudelaitem->x,
 		     y + (thechord->lowesty / 2),
 		     mudelaitem);
       if (!thechord->is_fakechord && thechord->fakechord) 
-	draw_fakechord (gui->pixmap, itp->gc,
-		     gtk_style_get_font (itp->widget->style),
+	draw_fakechord (cr,
 		     x + mudelaitem->x,
 		     y - 45,
 		     //y + (thechord->highesty / 4),
@@ -339,24 +368,22 @@ draw_object (objnode * curobj, gint x, gint y,
       break;
     case TUPOPEN:
     case TUPCLOSE:
-      draw_tupbracket (gui->pixmap, itp->gc,
-		       gtk_style_get_font (itp->widget->style),
+      draw_tupbracket (cr,
 		       x + mudelaitem->x, y, mudelaitem);
       break;
     case LILYDIRECTIVE:
       // if(si->markstaffnum) not available
-      draw_lily_dir(gui->pixmap, itp->gc,
-		       gtk_style_get_font (itp->widget->style),
+      draw_lily_dir(cr,
 		       x + mudelaitem->x, y, itp->in_highy, itp->in_lowy, mudelaitem, itp->mark);  
       break;
     case CLEF:
-      draw_clef (gui->pixmap, itp->gc, x + mudelaitem->x, y,
+      draw_clef (cr, x + mudelaitem->x, y,
 		 itp->clef = ((clef *) mudelaitem->object));
       if (si->currentobject == curobj && si->cursor_appending)
 	si->cursorclef = itp->clef->type;//FIXME drawing is side-effecting the data, presumably to economize on searching for the prevailing clef at the cursor.
       break;
     case KEYSIG:
-      draw_key (gui->pixmap, itp->gc, x + mudelaitem->x, y,
+      draw_key (cr, x + mudelaitem->x, y,
 		((keysig *) mudelaitem->object)->number, itp->key,
 		itp->clef->type, TRUE);
       itp->key = ((keysig *) mudelaitem->object)->number;
@@ -368,8 +395,7 @@ draw_object (objnode * curobj, gint x, gint y,
 	memcpy (si->nextmeasureaccs, itp->keyaccs, SEVENGINTS);
       break;
     case TIMESIG:
-      draw_timesig (gui->pixmap, itp->gc,
-		    gtk_style_get_font (itp->widget->style),
+      draw_timesig (cr,
 		    x + mudelaitem->x, y, itp->time1 =
 		    ((timesig *) mudelaitem->object)->time1, itp->time2 =
 		    ((timesig *) mudelaitem->object)->time2);
@@ -383,30 +409,16 @@ draw_object (objnode * curobj, gint x, gint y,
       itp->tickspermeasure = WHOLE_NUMTICKS * itp->time1 / itp->time2;
       break;
     case STEMDIRECTIVE:
-      draw_stem_directive (gui->pixmap, itp->gc,
-			   gtk_style_get_font (itp->widget->style),
+      draw_stem_directive (cr,
 			   x + mudelaitem->x, y, mudelaitem);
       itp->stem_directive = ((stemdirective *) mudelaitem->object)->type;
       break;
     case GRACE_START:
     case GRACE_END:
-      draw_gracebracket (gui->pixmap, itp->gc,
-			 gtk_style_get_font (itp->widget->style),
+      draw_gracebracket (cr,
 			 x + mudelaitem->x, y, mudelaitem);
       break;
-    case BARLINE:
-      {
-	gint top_y = (si->staffspace / 4) + itp->space_above;
-	x += GPOINTER_TO_INT (itp->mwidthiterator->data) + SPACE_FOR_BARLINE;
-	g_print ("possible y values top_y %d, y %d itp->y %d\n", itp->top_y,
-		 y, itp->y);
-	drawbarline (gui->pixmap, itp->gc, x, itp->y, itp->y + STAFF_HEIGHT,
-		     ((barline *) mudelaitem->object)->type);
-#ifdef DEBUG
-	g_print ("Draw barline\n");
-#endif
-      }
-      break;
+
     default:
       /* Nothing */
       break;
@@ -425,13 +437,13 @@ draw_object (objnode * curobj, gint x, gint y,
 	  extra = MAX (mudelaitem->minpixelsalloted,
 		       space_after (mudelaitem->durinticks,
 				    itp->wholenotewidth));
-	  draw_cursor (gui->pixmap, si, x + mudelaitem->x + extra, y,
+	  draw_cursor (cr, si, x + mudelaitem->x + extra, y,
 		       gui->mode, si->cursorclef);
 	  memcpy (si->cursoraccs, itp->curaccs, SEVENGINTS);
 	}
       else
 	{
-	  draw_cursor (gui->pixmap, si, x + mudelaitem->x, y, gui->mode,
+	  draw_cursor (cr, si, x + mudelaitem->x, y, gui->mode,
 		       si->cursorclef);
 	}
     }
@@ -444,15 +456,20 @@ draw_object (objnode * curobj, gint x, gint y,
       && si->firstmeasuremarked == itp->measurenum
       && si->firstobjmarked == itp->objnum)
     itp->markx1 = x + mudelaitem->x - EXTRAFORSELECTRECT;
+
   if (si->laststaffmarked == itp->staffnum
       && si->lastmeasuremarked == itp->measurenum
       && si->lastobjmarked == itp->objnum)
     itp->markx2 = x + mudelaitem->x + mudelaitem->minpixelsalloted
       + EXTRAFORSELECTRECT;
 
+  //gdk_gc_set_foreground (blackgc, &black);
+
+  //  g_print("obj at %d %d\n",  x + mudelaitem->x + mudelaitem->minpixelsalloted, (int)(gui->scorearea->allocation.width/gui->si->zoom - (RIGHT_MARGIN + KEY_MARGIN + si->maxkeywidth + SPACE_FOR_TIME)));
+  //  itp->line_end = itp->markx2 > (int)(gui->scorearea->allocation.width/gui->si->zoom - (RIGHT_MARGIN + KEY_MARGIN + si->maxkeywidth + SPACE_FOR_TIME));
+
 
 
-  gdk_gc_set_foreground (blackgc, &black);
   //g_print("returning with %d\n", itp->highy);
   /* And give a return value and we're done */
   return (mudelaitem->starttickofnextnote - itp->tickspermeasure);
@@ -468,31 +485,16 @@ draw_object (objnode * curobj, gint x, gint y,
  * return TRUE if measure has correct number of beats
  */
 static gboolean
-draw_measure (measurenode * curmeasure, gint x, gint y,
+draw_measure (cairo_t *cr, measurenode * curmeasure, gint x, gint y,
 	      DenemoGUI * gui, struct infotopass *itp)
 {
-  static GdkPixmap *exclam = NULL;
   static GString *mstring;
-  static PangoContext *context;
-  static PangoLayout *layout;
-  PangoFontDescription *desc;
   gint extra_ticks = 0;//number of ticks by which measure is over-full
   DenemoScore *si = gui->si;
   objnode *curobj;
-  if (!context)
-    {
-      context =
-	gdk_pango_context_get_for_screen (gdk_drawable_get_screen
-					  (gui->pixmap));
-      layout = pango_layout_new (context);
-    }
   /* initialization */
-  if (!exclam)
-    {
-      exclam = bitmaphelper (itp->widget, toomany);
-      mstring = g_string_new (NULL);
-    }
-
+  if (!mstring)
+    mstring = g_string_new (NULL);
   /* Set information about the state at the current measure,
      if necessary */
 
@@ -514,19 +516,16 @@ draw_measure (measurenode * curmeasure, gint x, gint y,
 
   if(itp->measurenum>1) {
     g_string_sprintf (mstring, "%d", itp->measurenum);
-    pango_layout_set_text (layout, mstring->str, -1);
-    desc = pango_font_description_from_string (FONT);
-    pango_layout_set_font_description (layout, desc);
-    pango_font_description_free (desc);
-    gdk_draw_layout (gui->pixmap, itp->gc, x - SPACE_FOR_BARLINE,
-		     y - 12, layout);
+    drawnormaltext_cr (cr, mstring->str, x - SPACE_FOR_BARLINE, y - 12);
   }
+
+
   // draw the cursor and set the side effects up if this didn't happen when drawing the currentobject
   if (!si->currentobject && (si->currentstaffnum == itp->staffnum && si->currentmeasurenum == itp->measurenum))
     {
       /* That is, the cursor's at the beginning of this blank measure */
       si->cursoroffend = FALSE; 
-      draw_cursor (gui->pixmap, si, x, y, gui->mode, itp->clef->type);
+      draw_cursor (cr, si, x, y, gui->mode, itp->clef->type);
       memcpy (si->cursoraccs, itp->curaccs, SEVENGINTS);
       si->cursorclef = itp->clef->type;     
     }
@@ -579,15 +578,52 @@ draw_measure (measurenode * curmeasure, gint x, gint y,
 			  (si->firstobjmarked <= itp->objnum) && 
 			  (si->lastobjmarked >= itp->objnum)))
        );
-    extra_ticks = draw_object (curobj, x, y, gui, itp);
+
+
+    
+    if(itp->measurenum >= si->rightmeasurenum+1)
+      cairo_set_source_rgb( cr, 0.5,0.5,0.5 );//This draws light gray anything that may be only partially visible.
+    else 
+      //  if(itp->mark)
+      //	cairo_set_source_rgb( cr, 0, 0, 1.0 );//blue
+      //  else
+	cairo_set_source_rgb( cr, 0, 0, 0 );//black;
+    extra_ticks = draw_object (cr, curobj, x, y, gui, itp);    
   }
   /* Paint the exclamation point, if necessary */
-  GdkGC *exclamgc = (extra_ticks>0)?redgc:(extra_ticks<0)?bluegc:NULL;
-  if(exclamgc)
-    drawbitmapinverse (gui->pixmap, exclamgc, exclam,
-		       x, y - 8 - EXCL_HEIGHT,
-		       EXCL_WIDTH, EXCL_HEIGHT);
-  return exclamgc!=NULL;
+  cairo_save(cr);
+  if( extra_ticks > 0 )
+    cairo_set_source_rgb( cr, 1.0, 0, 0 );
+  else
+    cairo_set_source_rgb( cr, 0, 0, 1 );
+  if(extra_ticks != 0) {
+    drawlargetext_cr( cr, "!", x, y - 8 );
+    //cairo_set_source_rgb( cr, 0.5, 0.5, 0.5 );
+  } else
+    cairo_set_source_rgb( cr, 0, 0, 0 );
+  //draw the barline
+#if 0
+  //This barline changes appearance depending on where the cursor is
+  cairo_move_to (cr, x + GPOINTER_TO_INT (itp->mwidthiterator->data), y+STAFF_HEIGHT);
+  cairo_line_to (cr, x + GPOINTER_TO_INT (itp->mwidthiterator->data), y);
+  cairo_stroke (cr);
+#else
+  cairo_rectangle (cr, x + GPOINTER_TO_INT (itp->mwidthiterator->data), y-0.5, 1.5, STAFF_HEIGHT+1);
+  cairo_fill(cr);
+#endif
+
+  if (!curmeasure->next)
+    {
+      /* we've reached the end of the score and should
+       * draw the heavy part of double-barline at regular position */
+      x += 3;
+      cairo_rectangle (cr, x + GPOINTER_TO_INT (itp->mwidthiterator->data), y-0.5, 4, STAFF_HEIGHT+1);
+      cairo_fill(cr);
+    }	
+  cairo_restore(cr);
+
+
+  return extra_ticks!=0;
 }
 
 /**
@@ -601,104 +637,164 @@ draw_measure (measurenode * curmeasure, gint x, gint y,
  * return TRUE if the staff has had to made taller
  */
 static gboolean
-draw_staff (DenemoStaff * curstaffstruct, gint y,
+draw_staff (cairo_t *cr, staffnode * curstaff, gint y,
 	    DenemoGUI * gui, struct infotopass *itp)
 {
-  PangoContext *context;
-  PangoLayout *layout;
-  PangoFontDescription *desc;
+  DenemoStaff *thestaff = (DenemoStaff*)curstaff->data;
   gboolean repeat = FALSE;
   DenemoScore *si = gui->si;
-  gint x, i;
-  GdkGC *gc;
-  context =
-    gdk_pango_context_get_for_screen (gdk_drawable_get_screen (gui->pixmap));
-  layout = pango_layout_new (context);
+  gint x  = KEY_MARGIN, i;
+  //g_print("drawing staff %d at %d\n", itp->staffnum, y);
+  cairo_save(cr);
+  //Draw vertical lines to bind the staffs of the system together
+      if(curstaff->prev)
+	{
+	  DenemoStaff *prev = (DenemoStaff *)(curstaff->prev->data);	  
+	  cairo_set_source_rgb( cr, 0, 0, 0);
+	  cairo_rectangle (cr, LEFT_MARGIN, y - STAFF_HEIGHT - prev->space_below - thestaff->space_above, 2, 2*STAFF_HEIGHT + prev->space_below + thestaff->space_above);
+	  cairo_fill(cr);	 
+	}
+      if(curstaff->next)
+	{
+	  DenemoStaff *next = (DenemoStaff *)(curstaff->next->data);
+	  cairo_save(cr);
+	  cairo_set_source_rgb( cr, 0, 0, 0);
+	  cairo_rectangle (cr, LEFT_MARGIN, y, 2, 2*STAFF_HEIGHT + next->space_above + thestaff->space_below);
+	  cairo_fill(cr);	 
+	}
 
-  if ((DenemoStaff *) si->currentstaff->data == curstaffstruct)
-    gc = blackgc;
+  if ((DenemoStaff *) si->currentstaff->data == thestaff)
+    cairo_set_source_rgb( cr, 0,0,0 );
   else
-    gc = slategraygc;
+    cairo_set_source_rgb( cr, 0.3,0.3,0.3 );
 
 
+  if(!itp->line_end) {//not a continuation
+    draw_clef (cr, LEFT_MARGIN, y,
+	       itp->clef = thestaff->leftmost_clefcontext);
+    draw_key (cr, x, y,
+	      itp->key = thestaff->leftmost_keysig->number,
+	      0, itp->clef->type, TRUE);
+    x += si->maxkeywidth;
+    draw_timesig (cr, x,
+		  y, itp->time1 =
+		  thestaff->leftmost_timesig->time1, itp->time2 =
+		  thestaff->leftmost_timesig->time2);
+    x += SPACE_FOR_TIME;
 
-  draw_clef (gui->pixmap, gc, LEFT_MARGIN, y,
-	     itp->clef = curstaffstruct->leftmost_clefcontext);
-  x = KEY_MARGIN;
-  draw_key (gui->pixmap, gc, x, y,
-	    itp->key = curstaffstruct->leftmost_keysig->number,
-	    0, itp->clef->type, TRUE);
-  memcpy (itp->keyaccs, curstaffstruct->leftmost_keysig->accs, SEVENGINTS);
-  x += si->maxkeywidth;
+  } else {
+    draw_clef (cr, LEFT_MARGIN, y, itp->clef);
+    draw_key (cr, x, y,
+	      itp->key, 0, itp->clef->type, TRUE);
+    x += si->maxkeywidth;
+    x += SPACE_FOR_TIME;// to allow the same margin ??
+  }
 
-  draw_timesig (gui->pixmap, gc, gtk_style_get_font (itp->widget->style), x,
-		y, itp->time1 =
-		curstaffstruct->leftmost_timesig->time1, itp->time2 =
-		curstaffstruct->leftmost_timesig->time2);
-  x += SPACE_FOR_TIME;
-  itp->stem_directive = curstaffstruct->leftmost_stem_directive;
+  *itp->left = itp->measurenum;
+  memcpy (itp->keyaccs, thestaff->leftmost_keysig->accs, SEVENGINTS);
+
+
+  itp->stem_directive = thestaff->leftmost_stem_directive;
   itp->tickspermeasure = WHOLE_NUMTICKS * itp->time1 / itp->time2;
-  if (si->firststaffmarked == itp->staffnum)
-    itp->marky1 = y - EXTRAFORSELECTRECT;
-  if (si->laststaffmarked == itp->staffnum)
-    itp->marky2 = y + STAFF_HEIGHT + EXTRAFORSELECTRECT;
-
-  gint staffname_offset = (curstaffstruct->voicenumber == 1) ? 24 :
-    (curstaffstruct->voicenumber == 2
-     || curstaffstruct->voicenumber == 3) ? 12 : 0;
-
-  /* Draw staff name */
-  pango_layout_set_text (layout, curstaffstruct->denemo_name->str, -1);
-  desc = pango_font_description_from_string (FONT);
-  pango_layout_set_font_description (layout, desc);
-  gdk_draw_layout (gui->pixmap, gc, KEY_MARGIN, y - staffname_offset, layout);
 
 
-  // gint title_highy = 0;
-
-  pango_font_description_free (desc);
+  /* Draw staff name on first system */
+  if(!itp->line_end) {
+    gint staffname_offset = (thestaff->voicenumber == 1) ? 24 :
+      (thestaff->voicenumber == 2
+       || thestaff->voicenumber == 3) ? 12 : 0;
+    drawnormaltext_cr( cr, thestaff->denemo_name->str, KEY_MARGIN, y - staffname_offset+10 );
+  } else {
+ cairo_save(cr);
+   gint staffname_offset = (thestaff->voicenumber == 1) ? 24 :
+      (thestaff->voicenumber == 2
+       || thestaff->voicenumber == 3) ? 12 : 0;
+   cairo_translate(cr, 2, (y - staffname_offset+30) );
+   cairo_rotate( cr,- M_PI/5.0 );
+    drawnormaltext_cr( cr, thestaff->denemo_name->str, 0,0 );
+ cairo_restore(cr);
+  }
 
+  cairo_save(cr);
   /* Loop that will draw each measure. Basically a for loop, but was uglier
    * when written that way.  */
-  itp->measurenum = si->leftmeasurenum;
-  itp->curmeasure =
-    g_list_nth (curstaffstruct->measures, itp->measurenum - 1);
 
+  itp->curmeasure =
+    g_list_nth (thestaff->measures, itp->measurenum - 1);
+  // g_print("measurenum %d\nx=%d\n", itp->measurenum, x);
 
   //FIX in measureops.c for case where si->measurewidths is too short
   itp->mwidthiterator = g_list_nth (si->measurewidths, itp->measurenum - 1);
 
+  // g_print("Width is %d\n", itp->mwidthiterator->data);
 
-
-  itp->gc = gc;
-  while (itp->measurenum <= si->rightmeasurenum+1
-	 && itp->measurenum <= g_list_length (curstaffstruct->measures))
+  //itp->gc = gc;
+  itp->line_end = FALSE;
+  while ( (!itp->line_end)  //       itp->measurenum <= si->rightmeasurenum+1
+	 && itp->measurenum <= g_list_length (thestaff->measures))
     {
-      if(itp->measurenum == si->rightmeasurenum+1)
-	itp->gc = slategraygc;
-      itp->wronglengths[itp->measurenum-si->leftmeasurenum] = draw_measure (itp->curmeasure, x, y, gui, itp);
+      
+
+
+      if( x + GPOINTER_TO_INT (itp->mwidthiterator->data) + SPACE_FOR_BARLINE >
+	  (int) (gui->scorearea->allocation.width/gui->si->zoom - (RIGHT_MARGIN + KEY_MARGIN + si->maxkeywidth + SPACE_FOR_TIME)))
+	if(itp->curmeasure->next) {
+	  itp->line_end=TRUE;
+	  //continue; continue is not good here, you cannot see things that Denemo thinks you can on the right.
+	}
+      
+      draw_measure (cr, itp->curmeasure, x, y, gui, itp);
+
       x += GPOINTER_TO_INT (itp->mwidthiterator->data) + SPACE_FOR_BARLINE;
       itp->curmeasure = itp->curmeasure->next;
       itp->mwidthiterator = itp->mwidthiterator->next;
       itp->measurenum++;
-
-      if(-itp->highy>itp->in_highy && -itp->highy<MAXEXTRASPACE){
-	curstaffstruct->space_above = -itp->highy;
-	repeat = TRUE;
-      }
-      if(itp->lowy>itp->in_lowy && itp->lowy<MAXEXTRASPACE){
-	curstaffstruct->space_below = itp->lowy;
-	repeat=TRUE;
-      }				      
+      //g_print("line_end is %d, while itp->measurenum=%d and si->rightmeasurenum=%d\n",  itp->line_end, itp->measurenum, si->rightmeasurenum);
+      if(!itp->line_end) {
+	if(-itp->highy>itp->in_highy && -itp->highy<MAXEXTRASPACE){
+	  thestaff->space_above = -itp->highy;
+	  repeat = TRUE;
+	}
+	if(itp->lowy>itp->in_lowy && itp->lowy<MAXEXTRASPACE){
+	  thestaff->space_below = itp->lowy;
+	  repeat=TRUE;
+	}
+      }			      
     }
 
+  *itp->right = itp->measurenum-1;
+  
+      if(curstaff->prev)
+	{
+	  DenemoStaff *prev = (DenemoStaff *)(curstaff->prev->data);	  
+	  cairo_set_source_rgb( cr, 0, 0, 0);
+	  cairo_rectangle (cr, x - SPACE_FOR_BARLINE, y - STAFF_HEIGHT - prev->space_below - thestaff->space_above, 2, 2*STAFF_HEIGHT + prev->space_below + thestaff->space_above);
+	  cairo_fill(cr);	 
+	}
+      if(curstaff->next)
+	{
+	  DenemoStaff *next = (DenemoStaff *)(curstaff->next->data);
+	  cairo_save(cr);
+	  cairo_set_source_rgb( cr, 0, 0, 0);
+	  cairo_rectangle (cr, x - SPACE_FOR_BARLINE, y, 2, 2*STAFF_HEIGHT + next->space_above + thestaff->space_below);
+	  cairo_fill(cr);	 
+	}
+
+
+  cairo_restore(cr);
   // if(itp->highy > title_highy)
   //  itp->highy = title_highy;
 
   /* now draw the staff lines, reset itp->slur_stack, and we're done */
-  for (i = 0; i < curstaffstruct->no_of_lines; i++, y += LINE_SPACE)
-    gdk_draw_line (gui->pixmap, blackgc, LEFT_MARGIN, y,
-		   x - HALF_BARLINE_SPACE, y);
+  for (i = 0; i < thestaff->no_of_lines; i++, y += LINE_SPACE) {
+    cairo_set_line_width( cr, 1.0 );
+    cairo_move_to( cr, LEFT_MARGIN, y );
+    cairo_line_to( cr, x - HALF_BARLINE_SPACE, y );
+    cairo_stroke( cr );
+  }
+
+ 
+
   /* Initialize the slur_stack for this staff. For the time being,
      slurs that begin and/or end after the portion of the music
      that is shown are not drawn. */
@@ -707,8 +803,60 @@ draw_staff (DenemoStaff * curstaffstruct, gint y,
       g_slist_free (itp->slur_stack);
       itp->slur_stack = NULL;
     }
+  cairo_restore(cr);
   return repeat;
 }
+static void
+print_system_separator (cairo_t *cr, gdouble position){
+  //g_print("At %f for %d\n", position, Denemo.gui->scorearea->allocation.height);
+#define SYSTEM_SEP (6)
+  cairo_save(cr);
+  cairo_set_source_rgb( cr, 0.5, 0.0, 0.0 );
+  cairo_rectangle (cr, 0, position-SYSTEM_SEP/2, Denemo.gui->scorearea->allocation.width/Denemo.gui->si->zoom, SYSTEM_SEP);
+  cairo_set_source_rgb( cr, 0.7, 0.0, 0.0 );
+  cairo_fill(cr);
+#undef SYSTEM_SEP
+  cairo_restore(cr);
+}
+
+typedef enum colors {BLACK, RED, GREEN} colors;
+static void draw_playback_marker(cairo_t *cr, gint color, gint pos, gint yy, gint line_height) {
+  //g_print("drawing marker %x at %d %d %d\n", color, pos, yy, line_height);
+  cairo_save(cr);
+  cairo_set_line_width( cr, 4.0 );
+  switch(color) {
+  case BLACK:
+    cairo_set_source_rgb( cr, 0.0, 0.0, 0.0 );
+    break;
+  case GREEN:
+    cairo_set_source_rgb( cr, 0.0, 1.0, 0.0 );
+    break;
+  case RED:
+    cairo_set_source_rgb( cr, 1.0, 0.0, 0.0 );
+    break;
+  }
+  cairo_move_to( cr, pos, yy-STAFF_HEIGHT );
+  cairo_line_to( cr, pos, yy-STAFF_HEIGHT+line_height);
+  cairo_stroke( cr );
+  cairo_restore(cr);
+
+}
+
+static void draw_playback_markers(cairo_t *cr, struct infotopass *itp, gint yy, gint line_height) {
+
+  if(itp->playposition>-1)
+    draw_playback_marker(cr, BLACK, itp->playposition, yy, line_height);
+  itp->playposition = -1;
+  
+  if(itp->startposition>0)
+    draw_playback_marker(cr, GREEN, itp->startposition, yy, line_height);
+  itp->startposition = -1;
+  
+  if(itp->endposition>0)
+    draw_playback_marker(cr, RED, itp->endposition, yy, line_height);
+  itp->endposition = -1;
+}
+
 
 /**
  * This actually draws the score, staff-by-staff 
@@ -720,29 +868,46 @@ static gboolean
 draw_score (GtkWidget * widget, DenemoGUI * gui)
 {
   staffnode *curstaff;
-  gint x, y=0, i;
+  gint y=0;
   struct infotopass itp;
-
-  GList *mwidthiterator;
   gboolean repeat = FALSE;
   DenemoScore *si = gui->si;
-
+  gint line_height = gui->scorearea->allocation.height*gui->si->system_height/gui->si->zoom;
 
   /* Initialize some fields in itp */
 
-  itp.widget = widget;// this is used for widget->style, it is spuriously passed to the macro bitmaphelper
+
   itp.slur_stack = NULL;
   itp.hairpin_stack = NULL;
 
   itp.highy = 0;//in case there are no objects...
+  itp.lowy = 0;
+  itp.last_midi = NULL;
+  itp.playposition = -1;
+  itp.startposition = -1;
+  itp.endposition = -1;
+  itp.startobj =  itp.endobj = NULL;
   y = 0;
 
+  if(gui->si->smf) {
+    itp.startobj =
+      get_obj_for_start_time(gui->si->smf, gui->si->start_time);
+    itp.endobj =
+      get_obj_for_end_time(gui->si->smf, gui->si->end_time);
+    //g_print("Start time %p %f end time %p %f\n", itp.startobj, si->start_time, itp.endobj, si->end_time);
+  }
+  cairo_t *cr = gdk_cairo_create( gui->pixmap );
+
+  cairo_scale( cr, gui->si->zoom, gui->si->zoom );
+  cairo_translate( cr, 0.5, 0.5 );
+  //cairo_rotate( cr, M_PI/6.0 );
 
   /* Draw each staff */
-  for ((itp.staffnum = si->top_staff,
-	curstaff = g_list_nth (si->thescore, si->top_staff - 1),
-	y += si->staffspace / 4);
-       curstaff && itp.staffnum <= si->bottom_staff; itp.staffnum++) {
+  for (itp.staffnum = si->top_staff,
+	 curstaff = g_list_nth (si->thescore, si->top_staff - 1),
+	 (y += si->staffspace / 4);
+       curstaff && itp.staffnum <= si->bottom_staff; 
+       itp.staffnum++) {
     DenemoStaff *staff = (DenemoStaff *) curstaff->data;
     itp.verse = staff->currentverse?staff->currentverse->data:NULL;
     GdkPixbuf *StaffDirectivesPixbuf = (si->currentstaffnum==itp.staffnum)?StaffPixbuf:StaffPixbufSmall;
@@ -750,29 +915,46 @@ draw_score (GtkWidget * widget, DenemoGUI * gui)
 
     if (curstaff && staff->voicenumber == 1)
       y += staff->space_above;
+
+    //g_print("Incrementing vertically %d\n", y);
     itp.space_above = staff->space_above;
     gint top_y = (si->staffspace / 4) + itp.space_above;
     
     itp.top_y = top_y;
-    itp.y = y;
+    //itp.y = y;
     gint highy = staff->space_above;
     gint lowy =  staff->space_below;
 
     itp.in_highy = highy, itp.in_lowy = lowy;
     itp.highy = 0;//do not pass on extra_space from one staff to the next
-	gdk_draw_rectangle (gui->pixmap, gcs_lightbluegc(), TRUE, 0, y, LEFT_MARGIN, STAFF_HEIGHT/*staff edit*/);
+
+    cairo_save(cr);
+    cairo_set_source_rgb( cr, 0.5, 0.5, 1.0 );
+    cairo_rectangle (cr, 0, y, LEFT_MARGIN, STAFF_HEIGHT/*staff edit*/);
+    cairo_fill(cr);
+    cairo_restore(cr);
 
       if(staff->staff_directives){
 
 	guint width = gdk_pixbuf_get_width( GDK_PIXBUF(StaffDirectivesPixbuf));
 	guint height = gdk_pixbuf_get_height( GDK_PIXBUF(StaffDirectivesPixbuf));
-	gdk_draw_pixbuf(gui->pixmap, NULL, StaffDirectivesPixbuf,  0,0, 0,y, width, height, GDK_RGB_DITHER_NONE,0,0/*staff edit*/);
+	cairo_save( cr );
+	gdk_cairo_set_source_pixbuf( cr, GDK_PIXBUF(StaffDirectivesPixbuf), 0,y );
+	cairo_rectangle( cr,0,y, width, height );
+	cairo_fill( cr );
+	cairo_restore( cr );
+	//gdk_draw_pixbuf(gui->pixmap, NULL, StaffDirectivesPixbuf,  0,0, 0,y, width, height, GDK_RGB_DITHER_NONE,0,0/*staff edit*/);
       }
       if(staff->voice_directives) {
 
 	guint width = gdk_pixbuf_get_width( GDK_PIXBUF(StaffDirectivesPixbuf));
 	guint height = gdk_pixbuf_get_height( GDK_PIXBUF(StaffDirectivesPixbuf));
-	gdk_draw_pixbuf(gui->pixmap, NULL, StaffDirectivesPixbuf,  0,0, 0,y + STAFF_HEIGHT/2, width, height, GDK_RGB_DITHER_NONE,0,0/*staff edit*/);
+	cairo_save( cr );
+	gdk_cairo_set_source_pixbuf( cr, GDK_PIXBUF(StaffDirectivesPixbuf), 0,y+STAFF_HEIGHT/2 );
+	cairo_rectangle( cr,0,y+STAFF_HEIGHT/2, width, height );
+	cairo_fill( cr );
+	cairo_restore( cr );
+	//gdk_draw_pixbuf(gui->pixmap, NULL, StaffDirectivesPixbuf,  0,0, 0,y + STAFF_HEIGHT/2, width, height, GDK_RGB_DITHER_NONE,0,0/*staff edit*/);
       }
     
     
@@ -782,9 +964,18 @@ draw_score (GtkWidget * widget, DenemoGUI * gui)
       /* draw background of clef, keysig, timesig */
       gint key = gui->si->maxkeywidth;
       gint cmajor = key?0:5;//allow some area for keysig in C-major
-      gdk_draw_rectangle (gui->pixmap, graygc, TRUE, LEFT_MARGIN,y,KEY_MARGIN-LEFT_MARGIN - cmajor,STAFF_HEIGHT);/*clef edit*/
-      gdk_draw_rectangle (gui->pixmap, lightbluegc, TRUE, KEY_MARGIN-cmajor,y,key+2*cmajor,STAFF_HEIGHT);/*keysig edit*/
-      gdk_draw_rectangle (gui->pixmap, graygc, TRUE, KEY_MARGIN+key+cmajor,y,SPACE_FOR_TIME-cmajor,STAFF_HEIGHT);/*timesig edit*/
+      cairo_save(cr);
+
+      cairo_set_source_rgb( cr, 0.7, 0.7, 0.7 );
+      cairo_rectangle (cr, LEFT_MARGIN,y,KEY_MARGIN-LEFT_MARGIN - cmajor,STAFF_HEIGHT);/*clef edit*/
+      cairo_rectangle (cr, KEY_MARGIN+key+cmajor,y,SPACE_FOR_TIME-cmajor,STAFF_HEIGHT);/*timesig edit*/
+      cairo_fill(cr);
+
+      cairo_set_source_rgb( cr, 0.7, 0.7, 1 );
+      cairo_rectangle (cr, KEY_MARGIN-cmajor,y,key+2*cmajor,STAFF_HEIGHT);/*keysig edit*/
+      cairo_fill(cr);
+
+      cairo_restore(cr);
     }
     if(si->currentstaffnum==itp.staffnum) {
 
@@ -797,55 +988,54 @@ draw_score (GtkWidget * widget, DenemoGUI * gui)
       reset_lyrics(staff, count);
     }
     
-    //allocate wronglength[] large enough for si->rightmeasurenum - si->leftmeasurenum entries then put it in itp
-    // and set it in draw_measure
-
-    gboolean *wronglength = (gboolean *)g_malloc((sizeof(gboolean))*(2+si->rightmeasurenum - si->leftmeasurenum));
-    itp.wronglengths = wronglength;
-    repeat = draw_staff (staff, y, gui, &itp);
-
-#if 0
-    //IN FACT itp.highy is only set by one measure, it is reset to zero in the measure loop
-    if(-itp.highy>highy && -itp.highy<MAXEXTRASPACE) //FIXME this should be done before draw_staff returns
-      /*g_print("setting space above %d staff %d\n", -itp.highy, itp.staffnum),*/staff->space_above = -itp.highy, repeat=TRUE;
-    if(itp.lowy>lowy && itp.lowy<MAXEXTRASPACE)
-      staff->space_below = itp.lowy, repeat=TRUE;
-#endif
-    gint measures_in_staff = g_list_length(staff->measures);
-    /* Now draw the barlines between the measures, across all the staffs */
-    mwidthiterator = g_list_nth (si->measurewidths, si->leftmeasurenum - 1);
-    for (x = KEY_MARGIN + si->maxkeywidth + 
-	   SPACE_FOR_TIME - HALF_BARLINE_SPACE,
-	   i = si->leftmeasurenum; mwidthiterator && 
-	   (i <= si->rightmeasurenum); mwidthiterator = mwidthiterator->next, 
-	   i++)
-      {
-
-	gint top = y + STAFF_HEIGHT; 
-
-	x += GPOINTER_TO_INT (mwidthiterator->data) + SPACE_FOR_BARLINE;
-	if(i==measures_in_staff)
-	  mwidthiterator = g_list_last(si->measurewidths);//do not draw barlines for short staffs
-	if (!mwidthiterator->next) /* Last measure - draw light line 3 pixels early */
-	  x -= 3;
-	gdk_draw_line (gui->pixmap, wronglength[i-si->leftmeasurenum]?graygc:blackgc, x, top, x, y);
-	
-	if (!mwidthiterator->next)
-	  {
-	    /* we've reached the end of the score and should
-	     * draw the heavy part of double-barline at regular position */
-	    x += 3;
-	    gdk_draw_rectangle (gui->pixmap, wronglength[i-si->leftmeasurenum]?graygc:blackgc, TRUE, x,
-				y, 4,
-				STAFF_HEIGHT+1);
-	  }
-	   
-      }
+  
+    itp.measurenum = si->leftmeasurenum;
+    itp.line_end = FALSE;
+    itp.left = &gui->lefts[0];
+    itp.right = &gui->rights[0];
 
 
-    g_free(wronglength);
-    if ( itp.staffnum < si->bottom_staff
-	 &&    ((DenemoStaff *) curstaff->next->data)->voicenumber !=2)
+    repeat = draw_staff (cr, curstaff, y, gui, &itp);
+
+    draw_playback_markers(cr, &itp, y, line_height);
+    
+    gint system_num;
+    system_num = 1;
+    // g_print("Drawn staffnum %d, at %d %s.\n", itp.staffnum,  y, itp.line_end?" another line":"End");
+
+    if (itp.staffnum==si->top_staff)
+      print_system_separator (cr, line_height*system_num++);
+
+
+    // This block prints out continuations of the staff just printed
+    // we are going to have to store an x-count for each continuation so that mousing.c will know
+    // the correct value. LEFT_MARGIN only works for all same value notes...
+    {
+    int yy;
+    yy = y + line_height;
+    itp.left++;
+    itp.right++;
+
+
+    while(((itp.left-gui->lefts)<DENEMO_MAX_SYSTEMS-1) && itp.line_end && (yy<(gui->scorearea->allocation.height/gui->si->zoom))) {
+      if (itp.staffnum==si->top_staff)
+	print_system_separator (cr, line_height*system_num++);
+
+      if(draw_staff (cr, curstaff, yy, gui, &itp))
+	repeat = TRUE;
+      draw_playback_markers(cr, &itp, yy, line_height);
+
+      // g_print("Drawn successively staffnum %d, at %d %s. Aloc %d,%d yy now %d line height %d\n", itp.staffnum,  yy, itp.line_end?" another line":"End", gui->scorearea->allocation.width, gui->scorearea->allocation.height, yy, line_height);
+     
+      yy += line_height;
+      itp.left++;
+      itp.right++;
+    }
+    }//end of block printing continuations
+    *itp.left=0;//To signal end of valid systems
+
+    if ( (!curstaff->next)
+	 ||    ((DenemoStaff *) curstaff->next->data)->voicenumber !=2)
       {
 	if (itp.verse) {
 	  y += LYRICS_HEIGHT;
@@ -853,16 +1043,13 @@ draw_score (GtkWidget * widget, DenemoGUI * gui)
 	y +=
 	  (si->staffspace + staff->space_below);
       }
-    //itp.haslyrics = FALSE;
     curstaff = curstaff->next;
   }// for all the staffs
 
 
- 
-  /* Draw the selection rectangle */
-  if (si->markstaffnum)
-    draw_selection (gui->pixmap, bluegc, itp.markx1, itp.marky1, itp.markx2,
-		    itp.marky2);
+  if(itp.last_midi)
+    si->rightmost_time = get_midi_off_time(itp.last_midi);
+
   return repeat;
 
   /* And we're done */
