@@ -15,13 +15,9 @@
  *
  */
 void
-draw_stem_directive (GdkPixmap * pixmap, GdkGC * gc, GdkFont * font,
+draw_stem_directive (cairo_t *cr,
 		     gint xx, gint y, DenemoObject * theobj)
 {
-  PangoContext *context =
-    gdk_pango_context_get_for_screen (gdk_drawable_get_screen (pixmap));;
-  PangoLayout *layout = pango_layout_new (context);
-  PangoFontDescription *desc = pango_font_description_from_string (FONT);
 
   gchar *text = NULL;
 
@@ -37,8 +33,6 @@ draw_stem_directive (GdkPixmap * pixmap, GdkGC * gc, GdkFont * font,
       text = _("stem\ndown");
       break;
     }
-  pango_layout_set_text (layout, text, -1);
-  pango_layout_set_font_description (layout, desc);
-  gdk_draw_layout (pixmap, gc, xx, y - 4, layout);
-  pango_font_description_free (desc);
+
+  drawnormaltext_cr( cr, text, xx, y - 4);
 }
