@@ -1468,6 +1468,10 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 		  if (chordval.notes)
 		    {
 
+		      gint tmp_volume = cur_volume;
+		      if (curobj->isinvisible)
+			cur_volume = 0;
+
 	    /**************************
 	     * prepare for note output
 	     **************************/
@@ -1495,18 +1499,13 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 		      for (curtone = chordval.notes; curtone;
 			   curtone = curtone->next)
 			{
-			  gint tmp = cur_volume;
-			  if (curobj->isinvisible)
-			    cur_volume = 0;
-			  else if (chordval.has_dynamic){
+			  if (chordval.has_dynamic){
 			      g_debug("\nThis chord has a dynamic marking attatched\n");
 			      GList *dynamic = g_list_first(chordval.dynamics);	
 			      cur_volume =
 		              string_to_vol (((GString *) dynamic->data)->str,
 				   cur_volume);
 			  }
-			  else
-			    cur_volume = tmp;
 
 			  mid_c_offset =
 			    ((note *) curtone->data)->mid_c_offset;
@@ -1527,7 +1526,7 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 				     notenumber,
 				     tmpstaccato,
 				     tmpstaccatissimo, chordval.is_tied);
-
+			  
 			}
 		      /* End chord read loop */
 #if slurdebug
@@ -1573,6 +1572,7 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 			      smf_track_add_event_delta_pulses(track, event, mididelta);
 			      //g_print("Note on %x %x %x\n", MIDI_NOTE_ON | midi_channel, n, (mute_volume ? 0:cur_volume/*FIXME as above, mix*/));
 			      //g_print("event on %f\n", event->time_seconds);
+			      event->user_pointer = curobj;
 			      curobj->midi_events = g_list_append(curobj->midi_events, event);
 #if DEBUG
 			      g_print("'%d len %d'", event->event_number, event->midi_buffer_length);
@@ -1584,7 +1584,9 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 			      event = smf_event_new_from_bytes ( MIDI_NOTE_OFF | midi_channel, n, 0);
 			      //g_print("{%d}", event->event_number);
 			      smf_track_add_event_delta_pulses(track, event, mididelta);
+			      event->user_pointer = curobj;
 			      curobj->midi_events = g_list_append(curobj->midi_events, event);
+			      // g_print("event off lur kill %f\n", event->time_seconds);
 			    }
 			}
 		      /* end of first chord output loop */
@@ -1629,11 +1631,13 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 			      /* write note off */
 			      event = smf_event_new_from_bytes ( MIDI_NOTE_OFF | midi_channel, n, 60);
 			      smf_track_add_event_delta_pulses(track, event, mididelta);
+			      event->user_pointer = curobj;
 			      curobj->midi_events = g_list_append(curobj->midi_events, event);
+			      //print("event off %f\n", event->time_seconds);
 			    }
 			}
 		      /* end of second chord output loop */
-
+		      cur_volume = tmp_volume;
 		    }//end of for notes in chord. Note that rests have no MIDI representation, of course.
 		  width = 0;
 #if slurdebug
@@ -1672,10 +1676,10 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 		      fprintf (stderr, "timesig change to %d:%d\n",
 			       timesigupper, timesiglower);
 		    }
-		  printf ("\nchange to timesigupper = %i\n", timesigupper);
-		  printf ("\nchange to timesiglower = %i\n", timesiglower);
+
 		  event = midi_timesig (timesigupper, timesiglower);
 		  smf_track_add_event_delta_pulses(track, event, 0);
+		  event->user_pointer = curobj;
 		  curobj->midi_events = g_list_append(curobj->midi_events, event);
 		  break;
 
@@ -1753,6 +1757,7 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 		  event = midi_keysig ((((keysig *) curobj->object)->number),
 			       curstaffstruct->keysig.isminor);
 		  smf_track_add_event_delta_pulses(track, event, 0);
+		  event->user_pointer = curobj;
 		  curobj->midi_events = g_list_append(curobj->midi_events, event);
 		  break;
 
@@ -1802,7 +1807,9 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 		  }
 		  break;
 		default:
+#if DEBUG
 		  fprintf (stderr, "midi ignoring type %d\n", curobj->type);
+#endif
 		  break;
 		}
 	    }
@@ -1885,8 +1892,17 @@ exportmidi (gchar * thefilename, DenemoScore * si, gint start, gint end)
 
   si->smfsync = si->changecount;
 
+
+
   if(start || end)
     return exportmidi(NULL, si, 0, 0);// recurse if we have not generated all the MIDI for si
+  else {
+    if(si->start_time<0.0)
+      si->start_time = 0.0;
+    if(si->end_time<0.0)
+      si->end_time = smf_get_length_seconds(smf);
+    g_print("Start time %f end time %f\n", si->start_time, si->end_time);
+  }
   return smf_get_length_seconds(smf);
 }
 
