@@ -18,6 +18,7 @@
  */
 
 #include <stdlib.h>
+#include <stdint.h>
 #include <math.h>
 #include <string.h>
 #include <assert.h>
@@ -25,6 +26,11 @@
 #include "importmidi.h"
 #include "staffops.h"
 #include "smf.h"
+#include "view.h"
+#include "utils.h"
+#include "file.h"
+#include "commandfuncs.h"
+#include "processstaffname.h"
 
 #define TEXT			0x01
 #define COPYRIGHT		0X02
@@ -64,7 +70,6 @@ typedef struct nstack
 
 typedef struct midicallback
 {
-	DenemoGUI *gui;
 	nstack *currentnote; /* current note being processed */
 	GList *notestack; /* polyphonic voices notes that need reprocessing */
 	gint leftover; /* note/rest value that is leftover across the measure */
@@ -85,7 +90,7 @@ typedef struct harmonic
 	gint enshift;
 }harmonic;
 
-struct harmonic enharmonic(gint input, gint key);
+static struct harmonic enharmonic(gint input, gint key);
 static int readtrack(midicallback *mididata);
 void dotimesig(gint numerator, gint denominator, midicallback *mididata);
 void dokeysig(gint key, gint isminor, midicallback *mididata);
@@ -97,6 +102,7 @@ void donoteoff(midicallback *mididata);
 struct notetype ConvertLength(gint duration, midicallback *mididata);
 void ProcessNoteStack(midicallback *mididata);
 void process_list(midicallback *mididata);
+static gint Get_Smf_Note_OFF (gint pitch, gint timeon, gint delta_time, midicallback *mididata);
 
 static void
 note_from_int(char *buf, int note_number)
@@ -141,40 +147,6 @@ cmd_load(midicallback *mididata, char *file_name)
 
 #define BUFFER_SIZE 1024
 
-static int
-show_event(smf_event_t *event)
-{
-	int off = 0, i;
-	char *decoded, *type;
-
-	if (smf_event_is_metadata(event))
-		type = "Metadata";
-	else
-		type = "Event";
-	
-	decoded = smf_event_decode(event);
-
-	if (decoded == NULL) {
-		decoded = malloc(BUFFER_SIZE);
-		if (decoded == NULL) {
-			g_critical("show_event: malloc failed.");
-			return -1;
-		}
-
-		off += snprintf(decoded + off, BUFFER_SIZE - off, "Unknown event:");
-
-		for (i = 0; i < event->midi_buffer_length && i < 5; i++)
-			off += snprintf(decoded + off, BUFFER_SIZE - off, " 0x%x", event->midi_buffer[i]);
-	}
-
-	g_message("%d: %s: %s, %f seconds, %d pulses, %d delta pulses", event->event_number, type, decoded,
-		event->time_seconds, event->time_pulses, event->delta_time_pulses);
-
-	free(decoded);
-
-	return 0;
-}
-
 void
 decode_metadata(const smf_event_t *event, midicallback *mididata)
 {
@@ -331,9 +303,15 @@ decode_midi_event(const smf_event_t *event, midicallback *mididata)
 			note_from_int(note, event->midi_buffer[1]);
 			g_debug("\nNote On channel %d note %s velocity %d\n", 
 					channel, note, event->midi_buffer[2]);
-			mididata->delta_time = event->delta_time_pulses;
-			mididata->event_number = event->event_number;
-			donoteon(mididata,  event->midi_buffer[1],  event->midi_buffer[2], event->time_pulses);
+
+                        if (0 == event->midi_buffer[2]) { // actually note off
+			    donoteoff (mididata);
+                        } // actually note off
+                        else { // really note on
+			    mididata->delta_time = event->delta_time_pulses;
+			    mididata->event_number = event->event_number;
+			    donoteon(mididata,  event->midi_buffer[1],  event->midi_buffer[2], event->time_pulses);
+                        } // really note on
 			break;
 
 		case AFTERTOUCH:
@@ -398,10 +376,6 @@ readtrack(midicallback *mididata)
 		ProcessNoteStack(mididata);
 	  mididata->selected_track = smf_get_track_by_number(mididata->smf, mididata->track);
 	  while ((event = smf_track_get_next_event(mididata->selected_track)) != NULL) {
-#ifdef DEBUG
-		/* print midi event */
-		show_event(event);
-#endif
 		/* Do something with the event */
 		process_midi(event, mididata);
 	  }
@@ -412,25 +386,78 @@ readtrack(midicallback *mididata)
 	return 0;
 }
 
-/* TODO consolidate this into one function */
-void
-new_dnm_object(midicallback *mididata, nstack *currentnote, notetype length, gboolean is_note){
-	DenemoObject *mudela_obj_new;
-	mudela_obj_new = dnm_newchord (length.notetype, length.numofdots, length.tied);
-	if (is_note){
-	  harmonic enote = enharmonic ((int) mididata->currentnote->pitch,mididata->key);
-	  dnm_addtone (mudela_obj_new, enote.pitch, enote.enshift, mididata->gui->si->cursorclef);
-	  while (mididata->currentnote->chordnotes){
-		  struct harmonic enote = enharmonic ((int) 
-				 mididata->currentnote->chordnotes->data,mididata->key);
-	          dnm_addtone (mudela_obj_new, enote.pitch, enote.enshift, mididata->gui->si->cursorclef);		
-		  mididata->currentnote->chordnotes = g_list_remove_link(mididata->currentnote->chordnotes, 
-				  g_list_first (mididata->currentnote->chordnotes));
-	  }
-	}
-	object_insert (mididata->gui, mudela_obj_new);	
-}
+static void
+insert_note_into_score(midicallback *mididata, notetype length)
+{
+  DenemoGUI *gui = Denemo.gui;
+  gint i;
 
+  /* 0-8 accepted bellow */
+  gchar *proc = g_strdup_printf("(d-Insert%d)", length.notetype);
+  call_out_to_guile(proc);
+  g_free(proc);
+  
+  /* get correct note name */
+  harmonic enote = enharmonic ((int) mididata->currentnote->pitch,mididata->key);
+  gchar *name =  mid_c_offsettolily (enote.pitch, enote.enshift);
+  
+  /* Rename note to the correct note */
+  gchar *accidental = g_strdup_printf("(d-ChangeChordNotes \"%s\")", name);
+  call_out_to_guile(accidental);
+  g_free(accidental);
+
+  /* Add dots */
+  for (i=0;i<length.numofdots;i++)
+    add_dot_key (gui);
+
+  /* insert tie */
+  if (length.tied)
+    call_out_to_guile("(d-ToggleTie)");
+}	
+
+static void
+insert_rest_into_score(midicallback *mididata, notetype length)
+{
+  DenemoGUI *gui = Denemo.gui;
+  gint i;
+
+  switch (length.notetype)
+  {
+     case 0:
+       insert_rest_0key (gui);
+       break;
+     case 1:
+       insert_rest_1key (gui);
+       break;
+     case 2:
+       insert_rest_2key (gui);
+       break;
+     case 3:
+       insert_rest_3key (gui);
+       break;
+     case 4:
+       insert_rest_4key (gui);
+       break;
+     case 5:
+       insert_rest_5key (gui);
+       break;
+     case 6:
+       insert_rest_6key (gui);
+       break;
+     default:
+       break;
+  }
+  displayhelper (gui);
+
+  /* add dots */
+  for (i=0;i<length.numofdots;i++)
+    add_dot_key (gui);
+  
+  /* Insert tie */
+  if (length.tied)
+    call_out_to_guile("(d-ToggleTie)");
+
+}
 /**
  * Insert time signature into current staff 
  *
@@ -438,8 +465,9 @@ new_dnm_object(midicallback *mididata, nstack *currentnote, notetype length, gbo
 void
 dotimesig (gint numerator, gint denominator, midicallback *mididata)
 {
+  DenemoGUI *gui = Denemo.gui;
   /*only does initial TS */
-  DenemoStaff *curstaffstruct = (DenemoStaff *) mididata->gui->si->currentstaff->data;
+  DenemoStaff *curstaffstruct = (DenemoStaff *) gui->si->currentstaff->data;
 
   curstaffstruct->timesig.time1 = numerator;
   curstaffstruct->timesig.time2 = denominator;
@@ -454,14 +482,14 @@ dotimesig (gint numerator, gint denominator, midicallback *mididata)
 void
 dokeysig (gint isminor, gint key, midicallback *mididata)
 {
-
+  DenemoGUI *gui = Denemo.gui;
   if (key > 7)
     key = key - 256;		/*get flat key num, see keysigdialog.cpp */
 #ifdef DEBUG
   g_print("\nkey = %d\n", key); 
 #endif
   mididata->key = key;
-  DenemoStaff *curstaffstruct = (DenemoStaff *) mididata->gui->si->currentstaff->data;
+  DenemoStaff *curstaffstruct = (DenemoStaff *) gui->si->currentstaff->data;
   curstaffstruct->keysig.number = key;
   curstaffstruct->keysig.isminor = isminor;
   dnm_setinitialkeysig (curstaffstruct, key, isminor);
@@ -470,22 +498,25 @@ dokeysig (gint isminor, gint key, midicallback *mididata)
 void
 dotempo (gint tempo, midicallback *mididata)
 { 
-  mididata->gui->si->tempo = (gint) (6.0e7 / (double) tempo);
+  DenemoGUI *gui = Denemo.gui;
+  gui->si->tempo = (gint) (6.0e7 / (double) tempo);
 }
 
 void
 dotrackname (gchar *name, midicallback *mididata)
 {
-  DenemoStaff *curstaffstruct = (DenemoStaff *) mididata->gui->si->currentstaff->data;
+  DenemoGUI *gui = Denemo.gui;
+  DenemoStaff *curstaffstruct = (DenemoStaff *) gui->si->currentstaff->data;
   
   curstaffstruct->denemo_name->str = g_strdup(name);
-  dnm_set_lily_name (curstaffstruct->denemo_name, curstaffstruct->lily_name);
+  //set_lily_name (curstaffstruct->denemo_name, curstaffstruct->lily_name);
 }
 
 void
 doinstrname (gchar* name,  midicallback *mididata)
 {
-  DenemoStaff *curstaffstruct = (DenemoStaff *) mididata->gui->si->currentstaff->data;
+  DenemoGUI *gui = Denemo.gui;
+  DenemoStaff *curstaffstruct = (DenemoStaff *) gui->si->currentstaff->data;
 
   curstaffstruct->midi_instrument->str = g_strdup(name);
 }
@@ -499,6 +530,7 @@ stack (gint pitch, gint timeon, gint delta_time, gint duration, gint tracknum)
   mystack->on_delta_time = delta_time;
   mystack->duration = duration;
   mystack->tracknum = tracknum;
+  mystack->chordnotes = NULL;
   return mystack;
 }
 
@@ -541,7 +573,7 @@ ChordToneCheck(midicallback *mididata, gint pitch, gint timeon, gint delta_time,
      (currentnote->duration == duration)){
 	  g_print("\n***Same Duration and timeon***\n");
   	  /* append note to GList */
-	  currentnote->chordnotes = g_list_append(currentnote->chordnotes, (gpointer)pitch);
+	  currentnote->chordnotes = g_list_append(currentnote->chordnotes, (gpointer)(intptr_t)pitch);
 	  return TRUE;
   }
   else if ((currentnote->timeon <= timeon) && (timeon <= ((int) currentnote->timeon + (int) currentnote->duration))){
@@ -555,23 +587,41 @@ ChordToneCheck(midicallback *mididata, gint pitch, gint timeon, gint delta_time,
   return FALSE;
 }
 
-gint
+/** 
+ * extremely simple quantizer that rounds 
+ * to the closest granule size
+ */
+static int
+round2granule(int tick)
+{
+  gint smallestgrain = 48;
+  gdouble div = ((gdouble) tick / (gdouble) smallestgrain);
+  return smallestgrain * (gint) round(div);
+}
+
+static gint
 Get_Smf_Note_OFF (gint pitch, gint timeon, gint delta_time, midicallback *mididata){
   gint event_number = mididata->event_number;
   smf_event_t *event;
+  gint starttime;
   gint duration;
   gint tracknum = mididata->track;
   gboolean chordtone = FALSE;
 
   while ((event = smf_track_get_event_by_number(mididata->selected_track, event_number)) != NULL){
-    if (((event->midi_buffer[0] & SYS_EXCLUSIVE_MESSAGE1) == NOTE_OFF) 
-		    & (event->midi_buffer[1] == (int) pitch)){
-	duration = event->time_pulses - timeon; 
+    if (   (((event->midi_buffer[0] & SYS_EXCLUSIVE_MESSAGE1) == NOTE_OFF) 
+		    && (event->midi_buffer[1] == (int) pitch))
+        || (((event->midi_buffer[0] & SYS_EXCLUSIVE_MESSAGE1) == NOTE_ON) 
+		    && (event->midi_buffer[1] == (int) pitch)
+		    && (0 == event->midi_buffer[2]))
+       ){
+	duration = round2granule(event->time_pulses - timeon); 
+	starttime = round2granule(timeon); //simple quantize timeon
 	if (mididata->currentnote != NULL) 
-	   chordtone = ChordToneCheck(mididata, pitch, timeon, delta_time, duration);
+	   chordtone = ChordToneCheck(mididata, pitch, starttime, delta_time, duration);
 	if (!chordtone){
       	  /* store note in mididata->currentnote */
-	  mididata->currentnote = stack(pitch, timeon, delta_time, duration, tracknum);
+	  mididata->currentnote = stack(pitch, starttime, delta_time, duration, tracknum);
 	}
 	g_print("\nFound corresponding note off to pitch %d timeon = %d duration = %d\n", (gint) pitch, timeon, (gint) duration);
     	break;
@@ -581,7 +631,7 @@ Get_Smf_Note_OFF (gint pitch, gint timeon, gint delta_time, midicallback *midida
   return 0;
 }
 
-harmonic
+static harmonic
 enharmonic (gint input, gint key)
 {
   harmonic local;
@@ -683,7 +733,7 @@ ConvertNoteType2ticks(midicallback *mididata, notetype *gnotetype){
 
   ticks = dsq >> notetype;
   while (i++ < numofdots) 
-    ticks += dsq >> notetype + 1;
+    ticks += dsq >> (notetype + 1);
 
   return ticks;
 }
@@ -698,7 +748,7 @@ notetype ConvertLength(gint duration, midicallback *mididata){
   gint tied = 0;
   gint leftover = 0;
   gint dsq = (4 * PPQN);
-
+  g_debug("\nDuration = %d ticks\n", duration);
   
   while ((dsq >> notetype) > duration)
 	  notetype++;
@@ -720,20 +770,15 @@ notetype ConvertLength(gint duration, midicallback *mididata){
  * check to see if a new staff needs to be added
  */
 void StaffCheck(midicallback *mididata){
-  DenemoScore *si = mididata->gui->si;
+  DenemoGUI *gui = Denemo.gui;	
   gint track = (int) mididata->currentnote->tracknum;
-  gint currentstaffnum = si->currentstaffnum;
+  gint currentstaffnum = gui->si->currentstaffnum;
 
   if (track > currentstaffnum) /*if not first track add track */
     {
-      si->currentstaffnum++;
-      si->currentstaff = g_list_first (si->thescore); /* set the first track to be copied */
-      newstaff (mididata->gui, ADDFROMLOAD, DENEMO_NONE); /* add track */
-      si->currentstaff = g_list_last (si->thescore); /* make last staff the current */
-      mididata->lastoff = 0;
-      si->cursor_x = 0;
-      mididata->bartime = 0;
-      si->currentmeasurenum = 1;
+     call_out_to_guile("(d-AddAfter)");
+     mididata->lastoff = 0;
+     mididata->bartime = 0;
     }
 }
 
@@ -742,53 +787,53 @@ void StaffCheck(midicallback *mididata){
  * to see if we need to add a new measure 
  */
 void MeasureCheck(midicallback *mididata){
+  DenemoGUI *gui = Denemo.gui;
   if (mididata->bartime >= mididata->barlength)	
   {			/* mididata->bartime >= barlenth will be true if there are rests  or notes
 			   going over end of measures. */
-    if (!mididata->gui->si->currentmeasure->next)
-      /* Add a measure and make it currentmeasure */
-      mididata->gui->si->currentmeasure =
-      dnm_addmeasures (mididata->gui->si, mididata->gui->si->currentmeasurenum, 1, 1);
+    if (!gui->si->currentmeasure->next)
+      call_out_to_guile("(d-AddMeasure)");
     else
-      mididata->gui->si->currentmeasure = mididata->gui->si->currentmeasure->next;
-    /* Now the stuff that needs to be done for each case */
-    mididata->gui->si->currentmeasurenum++;
-    mididata->gui->si->currentobject = NULL;
-    mididata->gui->si->cursor_x = 0;
-    memcpy (mididata->gui->si->cursoraccs, mididata->gui->si->nextmeasureaccs, SEVENGINTS);
-    memcpy (mididata->gui->si->curmeasureaccs, mididata->gui->si->nextmeasureaccs, SEVENGINTS);
-    mididata->gui->si->curmeasureclef = mididata->gui->si->cursorclef;
+      call_out_to_guile("(d-MeasureRight)");
     mididata->bartime = 0;
   }
 }
 
 void RestCheck(midicallback *mididata){
   gint rest = 0;
+  gint ticks;
   gint PPQN = mididata->smf->ppqn;
   gint starttime = (int) mididata->currentnote->timeon;
   gint duration = (int) mididata->currentnote->duration;
   gint on_delta_time = (int) mididata->currentnote->on_delta_time;
   gint dsq = (4 * mididata->smf->ppqn);
 
+  if (duration == 0)
+    return;
   if (starttime > mididata->lastoff){
     rest = starttime - mididata->lastoff;
+    g_debug("\nRest = %d\n",rest);
     if (mididata->bartime + on_delta_time  >= (mididata->barlength)){
+      g_debug("\nmididata->bartime + on_delta_time  >= mididata->barlength\n");
       while(mididata->barlength - mididata->bartime){
         rest = mididata->barlength - mididata->bartime;
         struct notetype length = ConvertLength(rest, mididata);
-        new_dnm_object(mididata, mididata->currentnote, length, FALSE);
-        mididata->bartime += ConvertNoteType2ticks(mididata, &length);
-        mididata->lastoff += ConvertNoteType2ticks(mididata, &length);
+        insert_rest_into_score(mididata, length);
+	ticks = ConvertNoteType2ticks(mididata, &length);
+        mididata->bartime += ticks;
+        mididata->lastoff += ticks;
       } 
       process_list(mididata);		          
       rest = 0;/* I am not sure if this is the best choice here*/
     }
-    if (rest){
+    while (rest){
         struct notetype length = ConvertLength(rest, mididata);
-	new_dnm_object(mididata, mididata->currentnote, length, FALSE);
-	mididata->bartime = mididata->bartime + ConvertNoteType2ticks(mididata, &length);
-	mididata->lastoff += ConvertNoteType2ticks(mididata, &length);
-	rest = 0;
+	insert_rest_into_score(mididata, length);
+	ticks = ConvertNoteType2ticks(mididata, &length);
+	mididata->bartime += ticks;
+	mididata->lastoff += ticks;
+	rest -= ticks;
+	g_debug("\nbartime = %d, lastoff = %d\n", mididata->bartime, mididata->lastoff);
     }
   }
 }
@@ -813,24 +858,27 @@ void TiedNoteCheck(midicallback *mididata){
 }
 
 void ProcessNote(midicallback *mididata) {
-	DenemoScore *si = mididata->gui->si;
+	DenemoGUI *gui = Denemo.gui;
+	DenemoScore *si = gui->si;
 	gint starttime = (int) mididata->currentnote->timeon;
 	gint lastoff = mididata->lastoff;
 	gint duration = (int) mididata->currentnote->duration;
 	gint pitch = (int) mididata->currentnote->pitch;
-	        
+	if (duration == 0)
+  	  return;
+        g_debug("\nProcessing pitch %d\n",pitch);	        
 	/*if a noteon event happends just after anouther note finishes just add the next note */
 	if (starttime == mididata->lastoff) {
 		notetype length = ConvertLength(duration, mididata);
 		length.tied = mididata->leftover;
-		new_dnm_object (mididata, mididata->currentnote, length, TRUE);
+		insert_note_into_score(mididata, length);
 		mididata->lastoff = starttime + duration;
 		mididata->bartime += duration;
 	}
 	if (mididata->leftover){
 		MeasureCheck(mididata);
 		notetype tied_length = ConvertLength(mididata->leftover, mididata);
-		new_dnm_object (mididata, mididata->currentnote, tied_length, TRUE);
+		insert_note_into_score(mididata, tied_length);
 		mididata->lastoff += mididata->leftover;
 		mididata->bartime += mididata->leftover;
 		mididata->leftover = 0;
@@ -877,16 +925,16 @@ gint
 importMidi (gchar *filename, DenemoGUI *gui)
 {
   midicallback *mididata = (midicallback *)g_malloc0(sizeof(midicallback));
+  mididata->notestack = NULL;
   mididata->selected_track = NULL;
   mididata->smf = NULL;
-  mididata->gui = gui;
   mididata->bartime = 0;
   mididata->lastoff = 0;
   mididata->track = 1;
   gint ret = 0;	// (-1 on failure)
 
   /* delete old data in the score */
-  dnm_deletescore (NULL, gui);
+  deletescore (NULL, gui);
 
   /* load the file */
   ret = cmd_load(mididata, filename);
