@@ -12,6 +12,7 @@
 #include "pitchentry.h"
 #include "smf.h"
 #include "device_manager.h"
+#include "moveviewport.h"
 #define NOTE_OFF                0x80
 #define NOTE_ON                 0x90
 #define SYS_EXCLUSIVE_MESSAGE1  0xF0
@@ -88,22 +89,12 @@ jackmidi_default_port_name(){
   return OUTPUT_PORT_NAME;
 }
 
-	double 
-get_time(void)
-{
-  double seconds;
-  int ret;
-  struct timeval tv;
-
-  ret = gettimeofday(&tv, NULL);
-
-  if (ret) 
-    perror("gettimeofday");
-	
-  seconds = tv.tv_sec + tv.tv_usec / 1000000.0;
-  return seconds;
+gboolean
+jackmidi_server_running(){
+  return jack_server_running;
 }
 
+
 static gboolean
 warning_async(gpointer s)
 {
@@ -120,14 +111,6 @@ warn_from_jack_thread_context(const char *str)
 #endif
 }
 
-/* this will be become obsolete when we have a playback bar*/ 
-static gint 
-move_on(DenemoGUI *gui){
-  if(timeout_id==0)
-    return FALSE;
-  set_currentmeasurenum (gui, gui->si->currentmeasurenum+1);
-  return TRUE;
-}
 
 gint 
 jack_kill_timer(void){
@@ -222,35 +205,70 @@ send_midi_event(jack_nframes_t nframes, gint client_number){
   }
 }
 #undef MDC
-static gboolean 
-jackmidi_read_smf_events(){
 
-  smf_event_t *event = Denemo.gui->si->smf?smf_peek_next_event(Denemo.gui->si->smf):NULL;
-  DevicePort *DP; 
+static gboolean finish_play(gchar *callback) {
+  if(callback && *callback)
+    call_out_to_guile (callback);
+  return FALSE;
+}
+
+static gdouble 		last_draw_time;
+static gdouble 		pause_time = -1.0;
+static gboolean jackmidi_play_smf_event(gchar *callback)
+{
+  DevicePort *DP;
+  DenemoScore *si = Denemo.gui->si;
+  smf_event_t *event = si->smf?smf_peek_next_event(si->smf):NULL;
+  
+  if (!jack_server_running)
+    return FALSE;
   if (!MD[0].jack_client)
     return FALSE;
-
   if (!playing_piece)
-    return FALSE;
+    return finish_play(callback);
 
-  if (event == NULL || event->time_seconds>end_time){
+  if (event == NULL || (event->time_seconds > si->end_time)){  
+    si->playingnow = NULL;
     playing_piece = FALSE;
-    return FALSE;
+    pause_time = -1.0;
+    toggle_playbutton();
+    return  finish_play(callback);
   }
-  else
+  else 
     playing_piece = TRUE;
-
-  /* Skip over metadata events. */
+  
+   /* Skip over metadata events. */
   if (smf_event_is_metadata(event)) {
-    event = smf_get_next_event(Denemo.gui->si->smf);
-    return TRUE;
-  }
-  if ((get_time() - start_player) >= event->time_seconds){
-    DP = event->track->user_pointer;
-    if(DP) {
+    event = smf_get_next_event(si->smf);
+    return TRUE; 
+  } 
+  // g_print("rightmost %f event %f\n", si->rightmost_time, event->time_seconds);
+  if(si->rightmost_time>0.0 && event->time_seconds>si->rightmost_time)
+     center_viewport();
+  gdouble thetime = get_time() - si->start_player;
+  pause_time = thetime;
+  //g_print("thetime %f\n", thetime);
+  thetime -= si->tempo_change_time - si->start_player;
+  thetime *= si->master_tempo;
+  thetime +=  si->tempo_change_time - si->start_player;
+  //g_print("transformed to %f\n", thetime);
+  if (thetime > event->time_seconds){
+     event = smf_get_next_event(si->smf);
+     si->playingnow = event->user_pointer;
+     DP = event->track->user_pointer;
+     //g_print("current object %p %x\n", event->user_pointer,((event->midi_buffer[0] & SYS_EXCLUSIVE_MESSAGE1)) );
+     if(((event->midi_buffer[0] & SYS_EXCLUSIVE_MESSAGE1)==NOTE_ON) &&
+	event->time_seconds - last_draw_time>Denemo.prefs.display_refresh) {
+       //       g_print("drawing because %f %f\n", event->time_seconds, last_draw_time);
+       last_draw_time = event->time_seconds;
+       
+       gtk_widget_queue_draw (Denemo.gui->scorearea);
+     }
+    gint chan = (event->midi_buffer[0] & 0x0f);
+    //g_print("message %x %x\n", event->midi_buffer[0] & SYS_EXCLUSIVE_MESSAGE1, PROGRAM_CHANGE);
+    int success;
+    if (DP && (si->end_time - event->time_seconds > 0.01)) //Do not turn notes on too close to the end
       jack_output_midi_event(event->midi_buffer, DP->device_number, DP->port_number);
-    }
-    event = smf_get_next_event(Denemo.gui->si->smf);
   }
   return TRUE;
 }
@@ -492,18 +510,13 @@ int
 init_jack(void){
   int i, err, port_number;
   i = err = port_number = 0;
-  DeviceManager();
+  //DeviceManager();
   if(MD==NULL) {
     jack_server_running = FALSE;
     if(Denemo.prefs.midi_audio_output==Jack)
       g_warning("No devices in preferences, edit->preferences->MIDI add devices and re-start");
     return -1;}
-  if(MD[0].jack_client) {
-    if(input_port==NULL)
-      input_port = jack_port_register(MD[0].jack_client, INPUT_PORT_NAME, JACK_DEFAULT_MIDI_TYPE,
-				      JackPortIsInput, 0);
-    return input_port == NULL;
-  }
+
   for (i=0;MD[i].client_name;i++){
     g_debug("\njack init *** client name == %s \n",MD[i].client_name->str);
     //create_jack_midi_client_from_load(MD[i].client_name->str);
@@ -513,7 +526,7 @@ init_jack(void){
 	g_warning("Could not open JACK client %s, no jack server running?", MD[i].client_name->str );
       return -1;
     }
-    if (jack_set_process_callback(MD[i].jack_client, process_callback, i)){
+    if (jack_set_process_callback(MD[i].jack_client, (JackProcessCallback)process_callback, (void *)i)){
       jack_server_running = FALSE;
       g_warning("Could not register JACK process callback.");
       return -1;
@@ -528,7 +541,10 @@ init_jack(void){
 	MD[i].ports[j].Index = MD[i].ports[j].FillIndex = 0;
 	MD[i].ports[j].BufferEmpty = TRUE;   
       }
-      
+      if (i==0 && MD[0].jack_client && input_port==NULL)
+        input_port = jack_port_register(MD[0].jack_client, INPUT_PORT_NAME, JACK_DEFAULT_MIDI_TYPE,
+				      JackPortIsInput, 0);
+  
       /* activate client */
       jack_activate(MD[i].jack_client);
     }
@@ -536,71 +552,55 @@ init_jack(void){
   return err;
 }
 
-void
-jack_midi_playback_start(){
+
+
+void jack_midi_play(gchar *callback)
+{
   DenemoGUI *gui = Denemo.gui;
   if (!jack_server_running)
     return;
-  start_player = get_time();
-  playing_piece = TRUE;
-
-  if (!MD[0].jack_client) //TODO check if this is correct
-    return;
-
-
-  /* set tranport on/off */
-  use_transport = (gboolean)Denemo.prefs.jacktransport; 
-  /* set transport start_stopped */
-  start_stopped = (gboolean) Denemo.prefs.jacktransport_start_stopped; 
-
   if((gui->si->smf==NULL) || (gui->si->smfsync!=gui->si->changecount))
-    exportmidi (NULL, gui->si, 1, 0/* means to end */);
-	  
+    generate_midi();
   if (Denemo.gui->si->smf == NULL) {
     g_critical("Loading SMF failed.");
-  } else {
-    smf_rewind(Denemo.gui->si->smf);
-    g_idle_add(jackmidi_read_smf_events, NULL);
+    return;
   }
-  
-  playback_duration = smf_get_length_seconds(gui->si->smf);
-  
-  DenemoObject *curobj;
-  start_time = 0.0;
-  curobj = get_mark_object();
-  if(curobj==NULL && gui->si->currentobject)
-    curobj = gui->si->currentobject->data;
-  if(curobj && curobj->midi_events) {
-    smf_event_t *event = curobj->midi_events->data;
-    start_time = event->time_seconds;
-    g_debug("\nsetting start %f\n", start_time);
-  }
-  end_time = playback_duration;
-  curobj = NULL;
-  curobj =  get_point_object();
-  if(curobj && curobj->midi_events)/*is this ever true?*/ { 
-    smf_event_t *event = g_list_last(curobj->midi_events)->data;
-    end_time = event->time_seconds;
-    g_debug("\nsetting end %f\n", end_time);	   
-    //could investigate to see if event is NoteOn and g_warning("Setting stop time to a NoteOn event!");
-  } 
+  static GString *callback_string;
 
-  if(start_time>end_time) {
-    gdouble temp = start_time;
-    start_time = end_time;
-    end_time = temp;
-  }
-  playback_duration = end_time - start_time;
-  g_debug("\nstart %f for %f seconds\n",start_time, playback_duration);
+  if(callback_string==NULL)
+    callback_string=g_string_new("");
+  if(callback)
+    g_string_assign(callback_string, callback);
+  else
+    g_string_assign(callback_string,"(display \"Stopped Playing\")");
+  
 
-  if(gui->si->end==0) {//0 means not set, we move the cursor on unless the specific range was specified
-    DenemoStaff *staff = (DenemoStaff *) gui->si->currentstaff->data;
-    //FIXME add a delay before starting the timer.
-    timeout_id = g_timeout_add ( 4*((double)staff->timesig.time1/(double)staff->timesig.time2)/(gui->si->tempo/(60.0*1000.0)), (GSourceFunc)move_on, gui);
-    // g_print("Setting end time to %f %u\n", duration*1000, (guint)(duration*1000));
-    kill_id = g_timeout_add ((guint)(playback_duration*1000), (GSourceFunc)jack_kill_timer, NULL);
+  if(playing_piece) {
+    // pause
+    toggle_playbutton();
+    playing_piece = FALSE;
+    (void)finish_play(callback);
+    return;
   }
-  return;
+  if(gui->si->end_time<0.0)
+    gui->si->end_time = smf_get_length_seconds(Denemo.gui->si->smf);
+  
+  if(gui->si->start_time>gui->si->end_time)
+    gui->si->start_time =  0.0;
+  if(gui->si->start_time<0.0)
+    gui->si->start_time = 0.0;
+  if( (pause_time>0.0) && (pause_time<gui->si->end_time))
+    gui->si->start_player = get_time() - pause_time;
+  else
+    pause_time = -1.0, gui->si->start_player = get_time() - gui->si->start_time;
+  playing_piece = TRUE;
+  toggle_playbutton();
+  gui->si->tempo_change_time = gui->si->start_player;
+  
+  smf_rewind(Denemo.gui->si->smf);
+  last_draw_time = 0.0;
+  g_idle_add(jackmidi_play_smf_event, callback_string->str);
+  smf_seek_to_seconds(gui->si->smf, pause_time>0.0? pause_time:gui->si->start_time);
 }
 
 void
@@ -608,14 +608,44 @@ jack_midi_playback_stop ()
 {
    //if(jack_client)
      //jack_transport_stop(jack_client);
-  playing_piece = FALSE;    
+  if(playing_piece)
+    toggle_playbutton();
+  Denemo.gui->si->playingnow = NULL;
+  playing_piece = FALSE;
+  pause_time = -1.0;
 }
+
+void
+jack_midi_panic()
+{
+  gint i,j,c;
+  gint client_number, port_number;
+  jack_midi_playback_stop ();
+  unsigned char buffer[3];
+
+  //flush buffers, stop callback?
+#define MD Denemo.prefs.midi_device
+  for (i=0;MD[i].client_name;i++)
+    for(j=0;MD[i].ports[j].port_name;j++) 
+      for (c=0;c<16;c++){
+        buffer[0] = MIDI_CONTROLLER | c;
+        buffer[1] = MIDI_ALL_SOUND_OFF;
+        buffer[2] = 0x00;  
+        jack_output_midi_event(buffer, i, j);
+      }
+  
+  Denemo.gui->si->end_time = pause_time = -1.0;
+  Denemo.gui->si->start_time =  0.0;
+  displayhelper(Denemo.gui);
+}
+
 #else //If not _HAVE_JACK_
 void jack_playpitch(){}
 void jack_output_midi_event(){}
 int jack_kill_timer(){}
 void jack_midi_playback_stop (){}
 void jack_midi_playback_start (){}
+void jack_midi_play(){}
 void remove_last_jack_midi_port (){}
 void create_jack_midi_port (){}
 void remove_jack_midi_client (){}
@@ -627,6 +657,8 @@ jackmidi_default_client_name(){}
 jackmidi_default_port_name(){}
 int maxnumber_of_ports(){ return 0;}
 int maxnumber_of_clients(){ return 0;}
-stop_jack(){}
+int jackmidi_server_running(){ return 0;}
+void stop_jack(){}
+void jack_midi_panic(){}
 #endif 
 
