@@ -8,10 +8,11 @@
 #define KBDINTERFACE_H
 
 void
-configure_keyboard_dialog (GtkAction *action ,DenemoGUI *gui);
+configure_keyboard_dialog (GtkAction *action, gpointer param);
 
 void
-configure_keyboard_dialog_OLD (GtkAction *action, DenemoGUI *gui);
+configure_keyboard_dialog_init_idx (GtkAction * action, DenemoGUI * gui,
+				    gint command_idx);
 
 
 #endif
