@@ -12,6 +12,7 @@
 #include "utils.h"
 #include "tupletops.h"
 #include "view.h"
+#include "commandfuncs.h"
 
 
 
@@ -264,7 +265,7 @@ go_to_G_key (DenemoGUI * gui)
 
 
 /**
- * Move cursor by amount
+ * Move cursor by amount or in EDIT mode change the note by one octave
  */
 static void
 octave_shift_key (DenemoGUI * gui, gint amount)
@@ -282,6 +283,7 @@ octave_shift_key (DenemoGUI * gui, gint amount)
 	    gui->si->cursor_y = copy.mid_c_offset + amount;
 	    insert_chordnote(gui->si);
 	    changeenshift(gui->si->currentobject->data, gui->si->cursor_y, copy.enshift);
+	    score_status(gui, TRUE);
 	  }
 	}
       }
@@ -1031,6 +1033,15 @@ newclefg8 (DenemoGUI * gui)
   object_insert (gui, dnm_newclefobj (DENEMO_G_8_CLEF));
 }
 
+/**
+ * Wrapper function to create new bass_8 clef and insert into the score
+ */
+void
+newcleff8 (DenemoGUI * gui)
+{
+  object_insert (gui, dnm_newclefobj (DENEMO_F_8_CLEF));
+}
+
 /**
  * Wrapper function to create new tenor clef and insert into the score
  */
@@ -1673,6 +1684,13 @@ setclefg8 (DenemoGUI * gui)
   if (curstaff)
     dnm_setinitialclef (gui->si, curstaff, DENEMO_G_8_CLEF);
 }
+void
+setcleff8 (DenemoGUI * gui)
+{
+  DenemoStaff *curstaff = (DenemoStaff *) gui->si->currentstaff->data;
+  if (curstaff)
+    dnm_setinitialclef (gui->si, curstaff, DENEMO_F_8_CLEF);
+}
 
 void
 setclefalto (DenemoGUI * gui)
