@@ -50,6 +50,8 @@ void insert_chord_3key (DenemoGUI *gui);
 void insert_chord_4key (DenemoGUI *gui);
 void insert_chord_5key (DenemoGUI *gui);
 void insert_chord_6key (DenemoGUI *gui);
+void insert_chord_7key (DenemoGUI *gui);
+void insert_chord_8key (DenemoGUI *gui);
 void insert_blankchord_0key (DenemoGUI *gui);
 void insert_blankchord_1key (DenemoGUI *gui);
 void insert_blankchord_2key (DenemoGUI *gui);
@@ -57,6 +59,8 @@ void insert_blankchord_3key (DenemoGUI *gui);
 void insert_blankchord_4key (DenemoGUI *gui);
 void insert_blankchord_5key (DenemoGUI *gui);
 void insert_blankchord_6key (DenemoGUI *gui);
+void insert_blankchord_7key (DenemoGUI *gui);
+void insert_blankchord_8key (DenemoGUI *gui);
 void insert_rest_0key (DenemoGUI *gui);
 void insert_rest_1key (DenemoGUI *gui);
 void insert_rest_2key (DenemoGUI *gui);
@@ -64,6 +68,8 @@ void insert_rest_3key (DenemoGUI *gui);
 void insert_rest_4key (DenemoGUI *gui);
 void insert_rest_5key (DenemoGUI *gui);
 void insert_rest_6key (DenemoGUI *gui);
+void insert_rest_7key (DenemoGUI *gui);
+void insert_rest_8key (DenemoGUI *gui);
 
 void insert_duplet (DenemoGUI *gui);
 void insert_triplet (DenemoGUI *gui);
