@@ -1303,6 +1303,10 @@ widget_for_directive(DenemoDirective *directive,  void fn()) {
       value = directive->display->str;
 
   if((directive->widget==NULL) ) {
+
+
+  //FIXME at this point you could allow the user to specify a custom button box for his directive - some property of the directive saying which button box it should be in. We could even allow the directive to create a toolitem of a toolbar or menuitem on a menu bar???
+
     if(fn==(void(*)())score_directive_put_graphic ||fn==(void(*)())scoreheader_directive_put_graphic ||fn==(void(*)())paper_directive_put_graphic)  
       box = Denemo.gui->buttonbox;
     else
@@ -1377,7 +1381,7 @@ widget_for_directive(DenemoDirective *directive,  void fn()) {
 
 //As the above (for string and int) but for the graphic name field
 //FIXME this is just storing the graphic name, any bitmap of that name could be placed on the button/menu item as an icon
-#define PUT_GRAPHIC_WIDGET_GRAPHIC(what, name) gpointer \
+#define PUT_GRAPHIC_WIDGET_GRAPHIC(what, name) gboolean \
 what##_directive_put_graphic(gchar *tag, gchar *value) {\
   what *current = get_##what();\
   if(current==NULL) return NULL;\
@@ -1394,7 +1398,7 @@ what##_directive_put_graphic(gchar *tag, gchar *value) {\
     g_string_assign(directive->graphic_name, value);\
   widget_for_directive(directive, (void(*)())what##_directive_put_graphic);\
   g_object_set_data(G_OBJECT(directive->widget), "directives-pointer", &current->name);\
-  return (gpointer)directive;\
+  return (gboolean)directive;\
 }
 
 PUT_GRAPHIC_WIDGET_GRAPHIC( score, directives)
@@ -2132,6 +2136,7 @@ TEXT_EDIT_IF(paper);
 TEXT_EDIT_IF(layout);
 TEXT_EDIT_IF(movementcontrol);
 TEXT_EDIT_IF(standalone);
+return TRUE;
 }
 #undef TEXT_EDIT_IF
 /* allow edit of a directive, either via script or textually if no script exists 
