@@ -40,6 +40,7 @@
 #include "exportxml.h"
 #include "runsilent.h"
 #include "utils.h"
+#include "keyboard.h"
 #ifdef _HAVE_FLUIDSYNTH_
 #include "fluid.h"
 #endif
@@ -125,7 +126,7 @@ register_stock_icon (GtkIconFactory * icon_factory, const gchar * stock_id,
                  error->message);
       g_error_free (error);
       if(first_time_user())
-	warningdialog("some icons will not display properly, but program will run ok.");
+	g_warning("some icons will not display properly, but program will run ok.");
       warned = TRUE;
     }
   if(pixbuf != NULL) {
@@ -454,10 +455,10 @@ main (int argc, char *argv[])
   g_setenv ("GTK_PREFIX", prefix, TRUE);
   g_print ("Setting GTK_PREFIX=%s\n", prefix);
 
-  gchar *fc_path = g_strconcat (prefix, "/etc/fonts", NULL);
+  gchar *fc_path = g_build_filename (prefix, "etc","fonts", NULL);
   g_setenv ("FONTCONFIG_PATH", fc_path, TRUE);
   g_print ("Setting FONTCONFIG_PATH=%s\n", fc_path);
-  gchar *fc_file = g_strconcat (fc_path, "/fonts.conf", NULL);
+  gchar *fc_file = g_build_filename (fc_path, "fonts.conf", NULL);
   g_setenv ("FONTCONFIG_FILE", fc_file, TRUE);
   g_print ("Setting FONTCONFIG_FILE=%s\n", fc_file);
 
@@ -469,6 +470,10 @@ main (int argc, char *argv[])
   path = g_strconcat (path, ";", program_files, "/Adobe/Reader 9.0/Reader", NULL);
   g_setenv ("PATH", path, TRUE);
 
+  gchar *fontpath = g_build_filename (prefix, "share", "fonts", "truetype","denemo", "fetta.ttf", NULL);
+  add_font_directory(fontpath);
+  fontpath = g_build_filename (prefix, "share", "fonts", "truetype","denemo", "denemo.ttf", NULL);
+  add_font_directory(fontpath);
 #endif
   GError *error = NULL;
   /* gtk initialization */
@@ -485,7 +490,8 @@ main (int argc, char *argv[])
 
   register_stock_items ();
   if(first_time_user())
-    infodialog("Nearly every menu item can be right-clicked, for help, setting keyboard shortcuts and more");
+    ;
+    // infodialog("Nearly every menu item can be right-clicked, for help, setting keyboard shortcuts and more"); this does not always appear on top of the main window so leave it out for now.
   //g_print("Calling scm boot guile with %d and %p\n", argc, argv);
   scm_boot_guile (argc, argv, inner_main, NULL);
   
