@@ -11,6 +11,8 @@
 #include "utils.h"
 #include "selectops.h"
 #include "lilydirectives.h"
+#include "lyric.h"
+#include "moveviewport.h"
 #include "mousing.h"
 
 
@@ -42,12 +44,18 @@ offset_from_height (gdouble height, enum clefs clef)
     case DENEMO_G_8_CLEF:
       R (3);
       break;
+    case DENEMO_F_8_CLEF:
+      R (-9);
+      break;
     case DENEMO_TENOR_CLEF:
       R (2);
       break;
     case DENEMO_SOPRANO_CLEF:
       R (8);
       break;
+    default:
+      R (0);
+      break;
     }
 #undef R
   return 0;
@@ -156,29 +164,32 @@ static gint staff_at (gint y, DenemoScore *si) {
  * Gets the position from the clicked position
  *
  */
-void
+static void
 get_placement_from_coordinates (struct placement_info *pi,
-				gdouble x, gdouble y, DenemoScore * si)
+				gdouble x, gdouble y, gint leftmeasurenum, gint rightmeasurenum )
 {
+  DenemoScore *si = Denemo.gui->si;
   GList *mwidthiterator = g_list_nth (si->measurewidths,
-				      si->leftmeasurenum - 1);
+				      leftmeasurenum - 1);
   objnode *obj_iterator;
   gint x_to_explain = (gint) (x);
 
   pi->staff_number = staff_at((gint)y, si);
-/*   g_print("get staff number %d\n",pi->staff_number); */
-  pi->measure_number = si->leftmeasurenum;
+  //g_print("L/R %d %d got staff number %d\n", leftmeasurenum, rightmeasurenum, pi->staff_number); 
+  pi->measure_number = leftmeasurenum;
   x_to_explain -= (KEY_MARGIN + si->maxkeywidth + SPACE_FOR_TIME);
+  //g_print("Explaining %d\n", x_to_explain);
   while (x_to_explain > GPOINTER_TO_INT (mwidthiterator->data)
-	 && pi->measure_number < si->rightmeasurenum)
+	 && pi->measure_number < rightmeasurenum)
     {
       x_to_explain -= (GPOINTER_TO_INT (mwidthiterator->data)
 		       + SPACE_FOR_BARLINE);
       mwidthiterator = mwidthiterator->next;
       pi->measure_number++;
     }
+  //g_print("got to measure %d\n", pi->measure_number);
   pi->nextmeasure = (x_to_explain > GPOINTER_TO_INT (mwidthiterator->data)
-		     && pi->measure_number >= si->rightmeasurenum);
+		     && pi->measure_number >= rightmeasurenum);
     
   pi->the_staff = g_list_nth (si->thescore, pi->staff_number - 1);
   pi->the_measure
@@ -225,6 +236,7 @@ get_placement_from_coordinates (struct placement_info *pi,
 		    pi->cursor_x++;
 		}
 	    }
+	  //g_print("got to cursor x %d\n", pi->cursor_x);
   }
 }
 
@@ -317,9 +329,10 @@ perform_command(gint modnum, mouse_gesture press, gboolean left)
 }  
 
 static gboolean selecting = FALSE;
+static gboolean dragging_separator = FALSE;
 
 
-static change_staff(DenemoScore *si, gint num, GList *staff) {
+static gboolean change_staff(DenemoScore *si, gint num, GList *staff) {
   if(si->currentstaffnum==num)
     return FALSE;
   hide_lyrics();
@@ -329,6 +342,23 @@ static change_staff(DenemoScore *si, gint num, GList *staff) {
   return TRUE;
 }
 
+static void
+transform_coords(double* x, double* y) {
+  DenemoGUI *gui = Denemo.gui;
+  gint line_height = gui->scorearea->allocation.height*gui->si->system_height;
+  gint line_num = ((int)*y)/line_height;
+  *y -= line_num * line_height;
+  *x /= gui->si->zoom;
+  *y /= gui->si->zoom;
+  // *x += ((double)line_num * gui->si->widthtoworkwith / ((int)(1/gui->si->system_height))) - 1.0* (line_num?(double)LEFT_MARGIN:0.0);
+}
+
+
+gint 
+scorearea_leave_event(GtkWidget *widget, GdkEventCrossing *event) {
+   gdk_window_set_cursor(Denemo.window->window, gdk_cursor_new(GDK_LEFT_PTR));//FIXME? does this take time/hog memory
+   return FALSE;//allow other handlers (specifically the pitch entry one)
+}
 
 /**
  * Mouse motion callback 
@@ -338,14 +368,43 @@ gint
 scorearea_motion_notify (GtkWidget * widget, GdkEventButton * event)
 {
   DenemoGUI *gui = Denemo.gui;
+  if(gui==NULL)
+    return FALSE;
+  if(gui->scorearea==NULL)
+    return FALSE;
+  gint line_height = gui->scorearea->allocation.height*gui->si->system_height;
+  if (event->y < 0)
+    event->y = 0.0;
+  gint line_num = ((int)event->y)/line_height;
+  
+#define DENEMO_MINIMUM_SYSTEM_HEIGHT (0.01)
+
+
+  if(dragging_separator) {
+    gui->si->system_height =  event->y/gui->scorearea->allocation.height;
+    if(gui->si->system_height<DENEMO_MINIMUM_SYSTEM_HEIGHT)
+      gui->si->system_height = DENEMO_MINIMUM_SYSTEM_HEIGHT;
+    if(gui->si->system_height>1.0)
+      gui->si->system_height = 1.0;
+    scorearea_configure_event(gui->scorearea, NULL);
+    gtk_widget_queue_draw (gui->scorearea);
+    return TRUE;
+  }
+
+  if(line_height - ((int)event->y - 8)%line_height<12)
+    gdk_window_set_cursor(Denemo.window->window, gdk_cursor_new(GDK_SB_V_DOUBLE_ARROW));
+  else
+    gdk_window_set_cursor(Denemo.window->window, gdk_cursor_new(GDK_LEFT_PTR));//FIXME? does this take time/hog memory
+
+  transform_coords(&event->x, &event->y);
   //  g_print("Marked %d\n", gui->si->markstaffnum);
+  if(gui->lefts[line_num] == 0)
+    return TRUE;
+
 
   if(event->x<LEFT_MARGIN) {
     struct placement_info pi; //FIXME duplicate code
-    if (event->y < 0)
-      get_placement_from_coordinates (&pi, event->x, 0, gui->si);
-    else
-      get_placement_from_coordinates (&pi, event->x, event->y, gui->si);
+    get_placement_from_coordinates (&pi, event->x, event->y, gui->lefts[line_num],gui->rights[line_num]);
     if(pi.staff_number==gui->si->currentstaffnum) {
       gint offset = (gint)get_click_height(gui, event->y);
       if(offset<STAFF_HEIGHT/2) {
@@ -358,9 +417,9 @@ scorearea_motion_notify (GtkWidget * widget, GdkEventButton * event)
     if (lh_down || (selecting && gui->si->markstaffnum)){
       struct placement_info pi; 
       if (event->y < 0)
-	get_placement_from_coordinates (&pi, event->x, 0, gui->si);
+	get_placement_from_coordinates (&pi, event->x, 0, gui->lefts[line_num],gui->rights[line_num]);
       else
-	get_placement_from_coordinates (&pi, event->x, event->y, gui->si);
+	get_placement_from_coordinates (&pi, event->x, event->y, gui->lefts[line_num],gui->rights[line_num]);
       if (pi.the_measure != NULL){ /*don't place cursor in a place that is not there*/
 	change_staff(gui->si, pi.staff_number, pi.the_staff);
 	gui->si->currentmeasurenum = pi.measure_number;
@@ -396,7 +455,21 @@ scorearea_motion_notify (GtkWidget * widget, GdkEventButton * event)
 gint
 scorearea_button_press (GtkWidget * widget, GdkEventButton * event)
 {
-DenemoGUI *gui = Denemo.gui;
+  DenemoGUI *gui = Denemo.gui;
+  //if the cursor is at a system separator start dragging it
+  gint line_height = gui->scorearea->allocation.height*gui->si->system_height;
+  gint line_num = ((int)event->y)/line_height;
+  //g_print("diff %d\n", line_height - ((int)event->y)%line_height);
+  if(dragging_separator == FALSE)
+  if(line_height - ((int)event->y - 8)%line_height<12) {
+    dragging_separator = TRUE;
+    return TRUE;
+  }
+  dragging_separator = FALSE;
+  //g_print("before %f %f\n", event->x, event->y);
+  transform_coords(&event->x, &event->y);
+  //g_print("after %f %f\n", event->x, event->y);
+
   struct placement_info pi;
   gboolean left = (event->button != 3);
   gtk_widget_grab_focus(widget);
@@ -407,10 +480,15 @@ DenemoGUI *gui = Denemo.gui;
     gtk_widget_queue_draw (gui->scorearea);
     return TRUE;
   } 
+
+  if(gui->lefts[line_num] == 0)
+    return TRUE;//On an empty system at the bottom where there is not enough room to draw another staff.
+
+
   if (event->y < 0)
-    get_placement_from_coordinates (&pi, event->x, 0, gui->si);
+    get_placement_from_coordinates (&pi, event->x, 0, gui->lefts[line_num],gui->rights[line_num]);
   else
-    get_placement_from_coordinates (&pi, event->x, event->y, gui->si);
+    get_placement_from_coordinates (&pi, event->x, event->y, gui->lefts[line_num],gui->rights[line_num]);
   change_staff(gui->si, pi.staff_number, pi.the_staff);
 
   if(event->x<LEFT_MARGIN) {
@@ -485,6 +563,12 @@ scorearea_button_release (GtkWidget * widget, GdkEventButton * event)
 {
 DenemoGUI *gui = Denemo.gui;
  gboolean left = (event->button != 3);
+
+  if(dragging_separator) {
+    dragging_separator = FALSE;
+    return TRUE;
+  }
+
  //g_signal_handlers_block_by_func(gui->scorearea, G_CALLBACK (scorearea_motion_notify), gui); 
  if(left)
    lh_down = FALSE;
@@ -497,25 +581,49 @@ DenemoGUI *gui = Denemo.gui;
 
 gint
 scorearea_scroll_event (GtkWidget *widget, GdkEventScroll *event) {
+  DenemoGUI *gui = Denemo.gui;
   switch(event->direction) {
     DenemoScriptParam param;
   case GDK_SCROLL_UP:
-    staffup (&param);
-    if(!param.status)
-      warningmessage("This is the top staff");
+    if(event->state&GDK_CONTROL_MASK) {
+      Denemo.gui->si->zoom *= 1.1;
+      scorearea_configure_event(Denemo.gui->scorearea, NULL);
+      // displayhelper(gui);
+      //update_vscrollbar (gui); these do not seem to work ...
+      //update_hscrollbar (gui);
+    } else
+    if(event->state&GDK_SHIFT_MASK) {
+      scroll_left ();
+
+    } else {
+      movetostaffup (&param);
+      if(!param.status)
+	warningmessage("This is the top staff");
+    }
     break;
   case GDK_SCROLL_DOWN:
-    staffdown (&param);
+    if(event->state&GDK_CONTROL_MASK) {
+      Denemo.gui->si->zoom /= 1.1;
+      if(Denemo.gui->si->zoom <0.01)
+	Denemo.gui->si->zoom = 0.01;
+      scorearea_configure_event(Denemo.gui->scorearea, NULL);
+      //displayhelper(gui);
+    } else
+    if(event->state&GDK_SHIFT_MASK) {
+      scroll_right ();
+    } else {
+      movetostaffdown (&param);
     if(!param.status)
       warningmessage("This is the bottom staff");
+    }
     break;
   case GDK_SCROLL_LEFT:
-    measureleft (&param);
+    movetomeasureleft (&param);
     if(!param.status)
       warningmessage("This is the first measure");
     break;
   case GDK_SCROLL_RIGHT:
-    measureright (&param);
+    movetomeasureright (&param);
     if(!param.status)
       warningmessage("This is the last measure");
     break;
@@ -523,4 +631,5 @@ scorearea_scroll_event (GtkWidget *widget, GdkEventScroll *event) {
 
   }
   displayhelper(Denemo.gui);
+  return FALSE;
 }
