@@ -63,8 +63,8 @@ update_vscrollbar (DenemoGUI * gui)
 }
 
 /**
- * Sets the si->rigthmeasurenum to the largest
- * value in the score
+ * Sets the si->rigthmeasurenum to use
+ * all the space si->widthtoworkwith assumeing si->leftmeasurenum, as determined by the si->measurewidths
  */
 void
 set_rightmeasurenum (DenemoScore * si)
@@ -119,7 +119,7 @@ set_bottom_staff (DenemoGUI * gui)
   /* With that settled, now determine how many additional (primary)
      staves will fit into the window.  */
   staff_number = gui->si->top_staff;
-  space_left = gui->scorearea->allocation.height;
+  space_left = gui->scorearea->allocation.height*gui->si->system_height/gui->si->zoom;
   do
     {
       space_left -= gui->si->staffspace;
@@ -131,7 +131,7 @@ set_bottom_staff (DenemoGUI * gui)
 }
 
 /**
- * Find out if measure is past the current leftmeasurenum
+ * inverse of the below
  *
  */
 void
@@ -151,8 +151,8 @@ isoffleftside (DenemoGUI * gui)
 }
 
 /**
- * Find out if measure is past the current rightmeasurenum
- *
+ * Advance the leftmeasurenum until currentmeasurenum is before rightmeasurenum
+ * then adjust rightmeasurenum to match.
  */
 void
 isoffrightside (DenemoGUI * gui)
@@ -189,6 +189,18 @@ move_viewport_up (DenemoGUI * gui)
 }
 
 
+
+void center_viewport(void) {
+  GtkAdjustment *adj = GTK_ADJUSTMENT(Denemo.gui->hadjustment);
+  gint amount = (Denemo.gui->si->rightmeasurenum-Denemo.gui->si->leftmeasurenum)/2;
+  if(adj->value + amount < adj->upper) {    
+      gtk_adjustment_set_value(adj, adj->value + amount);
+  } else
+    gtk_adjustment_set_value(adj, adj->upper);
+  return TRUE;
+}
+
+
 /**
  * Move viewable part of the score down
  *
@@ -208,6 +220,33 @@ move_viewport_down (DenemoGUI * gui)
 }
 
 
+/**
+ * Sets the si->currentmeasurenum to the given value
+ * if exists, making it the leftmost measure visible, extending selection or not
+ *
+ */
+gboolean
+goto_currentmeasurenum (DenemoGUI * gui, gint dest, gboolean extend_selection)
+{
+  if ((dest > 0) && (dest <= (gint) (g_list_length (gui->si->measurewidths))))
+    {
+      //gui->si->leftmeasurenum = dest;
+      gui->si->currentmeasurenum = dest;
+      if((dest < gui->si->leftmeasurenum) ||
+	 (dest > gui->si->rightmeasurenum))
+	 center_viewport();
+      setcurrents (gui->si);
+      if(extend_selection)
+	calcmarkboundaries (gui->si);
+      set_rightmeasurenum (gui->si);
+      find_leftmost_allcontexts (gui->si);
+      update_hscrollbar (gui);
+      gtk_widget_queue_draw (gui->scorearea);
+      return TRUE;
+    }
+  return FALSE;
+}
+
 /**
  * Sets the si->currentmeasurenum to the given value
  * if exists, making it the leftmost measure visible
@@ -216,14 +255,42 @@ move_viewport_down (DenemoGUI * gui)
 gboolean
 set_currentmeasurenum (DenemoGUI * gui, gint dest)
 {
-  if ((dest > 0) && (dest <= (gint) (g_list_length (gui->si->measurewidths))))
+  return goto_currentmeasurenum (gui, dest, TRUE);
+}
+
+/**
+ * Sets the si->currentmeasurenum to the given value
+ * if exists, making it the leftmost measure visible, leaving selection
+ *
+ */
+gboolean
+moveto_currentmeasurenum (DenemoGUI * gui, gint dest)
+{
+  return goto_currentmeasurenum (gui, dest, FALSE);
+}
+
+
+
+
+/**
+ * Sets the si->currentstaffnum to the given value
+ * if it exists, extending selection if extend_selection
+ *
+ */
+gboolean
+goto_currentstaffnum (DenemoGUI * gui, gint dest, gboolean extend_selection)
+{
+  if ((dest > 0) && (dest <= (gint) (g_list_length (gui->si->thescore))))
     {
-      gui->si->leftmeasurenum = dest;
-      gui->si->currentmeasurenum = dest;
+      gui->si->currentstaffnum = dest;
+      gui->si->currentstaff =
+	g_list_nth (gui->si->thescore, gui->si->currentstaffnum - 1);
+      setcurrentprimarystaff (gui->si);
       setcurrents (gui->si);
-      set_rightmeasurenum (gui->si);
+      if(extend_selection)
+	calcmarkboundaries (gui->si);
       find_leftmost_allcontexts (gui->si);
-      update_hscrollbar (gui);
+      update_vscrollbar (gui);
       gtk_widget_queue_draw (gui->scorearea);
       return TRUE;
     }
@@ -238,19 +305,13 @@ set_currentmeasurenum (DenemoGUI * gui, gint dest)
 gboolean
 set_currentstaffnum (DenemoGUI * gui, gint dest)
 {
-  if ((dest > 0) && (dest <= (gint) (g_list_length (gui->si->thescore))))
-    {
-      gui->si->currentstaffnum = dest;
-      gui->si->currentstaff =
-	g_list_nth (gui->si->thescore, gui->si->currentstaffnum - 1);
-      setcurrentprimarystaff (gui->si);
-      setcurrents (gui->si);
-      find_leftmost_allcontexts (gui->si);
-      update_vscrollbar (gui);
-      gtk_widget_queue_draw (gui->scorearea);
-      return TRUE;
-    }
-  return FALSE;
+  return goto_currentstaffnum(gui, dest, TRUE);
+}
+
+gboolean
+moveto_currentstaffnum (DenemoGUI * gui, gint dest)
+{
+  return goto_currentstaffnum(gui, dest, FALSE);
 }
 
 /**
@@ -274,6 +335,7 @@ vertical_scroll (GtkAdjustment * adjust, DenemoGUI * gui)
 	    g_list_nth (gui->si->thescore, gui->si->bottom_staff - 1);
 	  setcurrentprimarystaff (gui->si);
 	  setcurrents (gui->si);
+calcmarkboundaries (gui->si);
 	}
       else if (gui->si->currentstaffnum < gui->si->top_staff)
 	{
@@ -282,6 +344,7 @@ vertical_scroll (GtkAdjustment * adjust, DenemoGUI * gui)
 	    g_list_nth (gui->si->thescore, gui->si->top_staff - 1);
 	  setcurrentprimarystaff (gui->si);
 	  setcurrents (gui->si);
+calcmarkboundaries (gui->si);
 	}
       gtk_widget_queue_draw (gui->scorearea);
     }
@@ -292,12 +355,12 @@ vertical_scroll (GtkAdjustment * adjust, DenemoGUI * gui)
  * Scroll score horizontally
  *
  */
-void
-horizontal_scroll (GtkAdjustment * adjust, DenemoGUI * gui)
+static void
+h_scroll (gdouble value, DenemoGUI * gui)
 {
   gint dest;
 
-  if ((dest = (gint) (adjust->value + 0.5)) != gui->si->leftmeasurenum)
+  if ((dest = (gint) (value + 0.5)) != gui->si->leftmeasurenum)
     {
       gui->si->leftmeasurenum = dest;
       set_rightmeasurenum (gui->si);
@@ -313,7 +376,22 @@ horizontal_scroll (GtkAdjustment * adjust, DenemoGUI * gui)
 	}
       find_leftmost_allcontexts (gui->si);
       setcurrents (gui->si);
+      //calcmarkboundaries (gui->si);
       gtk_widget_queue_draw (gui->scorearea);
     }
   update_hscrollbar (gui);
 }
+
+void
+horizontal_scroll (GtkAdjustment * adjust, DenemoGUI * gui)
+{
+  h_scroll (adjust->value, gui);
+}
+void scroll_left(void) {
+  if(Denemo.gui->si->leftmeasurenum>1)
+    h_scroll(Denemo.gui->si->leftmeasurenum-1.0, Denemo.gui);
+}
+
+void scroll_right (void) {
+  h_scroll(Denemo.gui->si->leftmeasurenum+1.0, Denemo.gui);
+}
