@@ -31,6 +31,8 @@ isoffleftside (DenemoGUI *gui);
 void
 isoffrightside (DenemoGUI *gui);
 
+void
+center_viewport(void);
 void
 move_viewport_up (DenemoGUI *gui);
 
@@ -40,13 +42,22 @@ move_viewport_down (DenemoGUI *si);
 gboolean
 set_currentmeasurenum (DenemoGUI *si, gint dest);
 
+gboolean
+moveto_currentmeasurenum(DenemoGUI *si, gint dest);
+
 void
 vertical_scroll (GtkAdjustment *adjust, DenemoGUI *si);
 
 void
 horizontal_scroll (GtkAdjustment *adjust, DenemoGUI *si);
 
+void scroll_left(void);
+void scroll_right (void);
+
 gboolean
 set_currentstaffnum(DenemoGUI *si, gint dest);
 
+gboolean
+moveto_currentstaffnum(DenemoGUI *si, gint dest);
+
 #endif
