@@ -94,6 +94,26 @@ Equal Temperament
 261.6	277.2	293.7 	311.1	329.6	349.2 	370.0	392.0	415.3 	440.0	466.2	493.9
 HARPSICHORD TUNING - A COURSE OUTLINE, by G.C. Klop, distributed by The Sunbury Press, P.O. Box 1778, Raleigh, NC 27602.
 #endif
+
+  static temperament Rameau = {
+  "Rameau", 8,3,
+    { 
+      {261.6, {0, 0}},
+      {276.9, {0, 1}},
+      {293.3, {1, 0}},
+      {309.7, {2, -1}},
+      {330.4, {2, 0}},
+      {348.8, {3, 0}},
+      {369.2, {3, 1}},
+      {391.1, {4, 0}},
+      {413.9, {4, 1}},
+      {438.5, {5, 0}},
+      {463.0, {6, -1}},
+      {493.9, {6, 0}}
+  }
+};
+
+
 static temperament Equal = {
   "Equal", 8,3,
     { 
@@ -182,7 +202,7 @@ static temperament WerckmeisterIV = {
 
 static temperament *PR_temperament=&Equal; /* the currently used temperament */
 
-static temperament *temperaments[] = {&Equal, &Meantone, &WerckmeisterIV, &Lehman};
+static temperament *temperaments[] = {&Equal, &Meantone, &WerckmeisterIV, &Lehman, &Rameau};
 
 static void switch_back_to_main_window(void) {
 
@@ -298,7 +318,7 @@ static void sharpen(GtkButton *button, GtkWidget *label) {
 #undef g
   PR_temperament->sharp = PR_temperament->flat;
   PR_temperament->flat = (PR_temperament->flat+7)%12;
-  if(PR_window) {
+  if(1 /*PR_window*/) {
     gchar *names = notenames(PR_temperament);
     gtk_label_set_markup((GtkLabel*)label, names);
     g_free(names);
@@ -326,7 +346,7 @@ if(t.alteration-1<-2)
 #undef t
   PR_temperament->flat = PR_temperament->sharp;
   PR_temperament->sharp = (PR_temperament->sharp+5)%12;
-  if(PR_window) {
+  if(1/*PR_window*/) {
   gchar *names = notenames(PR_temperament);
   gtk_label_set_markup((GtkLabel*)label, names);
   g_free(names);
@@ -392,7 +412,7 @@ static GList *get_tones(GList *tone_store, gint measurenum) {
 * return TRUE if measure has enough tones for all its notes
 */
 
-static gboolean  apply_tones(DenemoScore *si) { 
+gboolean  apply_tones(DenemoScore *si) { 
   gboolean ret=FALSE;
   DenemoStaff* curstaff = ((DenemoStaff*)si->currentstaff->data);
   GList *store;
@@ -621,6 +641,22 @@ static notepitch * determine_note(gdouble pitch, temperament *t, gint* which_oct
   return NULL;
 }
 
+
+//Computes the mid_c_offset and enshift for the MIDI notenum passed in
+void
+notenum2enharmonic (gint notenum, gint *poffset, gint *penshift) {
+
+  *penshift = notenum % 12;  
+  temperament *t = PR_temperament;
+  
+  *poffset = t->notepitches[*penshift].spec.step;
+  *penshift =  t->notepitches[*penshift].spec.alteration;
+  return;
+}
+
+
+
+
 static void display_pitch(double note, DenemoGUI *gui) {
   gint octave;
   double deviation;
@@ -946,6 +982,40 @@ static void  temperament_changed_callback (GtkComboBox *combobox,  GtkListStore
   g_string_assign(Denemo.prefs.temperament, PR_temperament->name);
 }
 
+GtkWidget *get_enharmonic_frame(void) {
+  static GtkWidget *frame;
+  if(frame==NULL) {
+    frame = gtk_frame_new( "Enharmonic selection");
+    g_object_ref(frame);
+    //gtk_container_add (GTK_CONTAINER (main_vbox), frame);
+    GtkWidget *hbox = gtk_hbox_new (FALSE, 1);
+    gtk_container_add (GTK_CONTAINER (frame), hbox);
+    GtkWidget *label = gtk_label_new("");
+    PR_label = label;
+    GtkWidget *button = gtk_button_new_with_label("flatten");
+    gtk_box_pack_start (GTK_BOX (hbox), button,
+			FALSE, TRUE, 0);
+    gtk_action_connect_proxy(gtk_ui_manager_get_action (Denemo.ui_manager, "/MainMenu/InputMenu/FlattenEnharmonicSet"), button);
+    gchar *names = notenames(PR_temperament);
+  
+    gtk_label_set_markup(GTK_LABEL(label),names);
+    g_free(names);
+    gtk_box_pack_start (GTK_BOX (hbox), label,
+			FALSE, TRUE, 0);
+  
+    button = gtk_button_new_with_label("sharpen");
+    gtk_box_pack_start (GTK_BOX (hbox), button,
+			FALSE, TRUE, 0);
+    gtk_action_connect_proxy(gtk_ui_manager_get_action (Denemo.ui_manager, "/MainMenu/InputMenu/SharpenEnharmonicSet"), button);
+  }
+  GtkContainer *cont = gtk_widget_get_parent(frame);
+  if(cont)
+    gtk_container_remove(cont, frame);
+  return frame;
+}
+
+
+
 static void create_pitch_recognition_window(DenemoGUI *gui) {
   GtkWidget *hbox, *hbox2;
   GtkWidget *button;
@@ -1050,27 +1120,14 @@ static void create_pitch_recognition_window(DenemoGUI *gui) {
   }// if audio input allow use of overlay mechanism, MIDI can use MIDIAdvanceOnEdit filter.
 
 
-  frame = gtk_frame_new( "Enharmonic selection");
-  gtk_container_add (GTK_CONTAINER (main_vbox), frame);
-  hbox = gtk_hbox_new (FALSE, 1);
-  gtk_container_add (GTK_CONTAINER (frame), hbox);
-  label = gtk_label_new("");
-  PR_label = label;
-  button = gtk_button_new_with_label("flatten");
-  gtk_box_pack_start (GTK_BOX (hbox), button,
-		      TRUE, TRUE, 0);
-  gtk_action_connect_proxy(gtk_ui_manager_get_action (Denemo.ui_manager, "/MainMenu/InputMenu/FlattenEnharmonicSet"), button);
-  gchar *names = notenames(PR_temperament);
-  
-  gtk_label_set_markup(GTK_LABEL(label),names);
-  g_free(names);
-  gtk_box_pack_start (GTK_BOX (hbox), label,
-		      TRUE, TRUE, 0);
-  
-  button = gtk_button_new_with_label("sharpen");
-  gtk_box_pack_start (GTK_BOX (hbox), button,
-		      TRUE, TRUE, 0);
-  gtk_action_connect_proxy(gtk_ui_manager_get_action (Denemo.ui_manager, "/MainMenu/InputMenu/SharpenEnharmonicSet"), button);
+  frame = get_enharmonic_frame();
+  if(!gtk_widget_get_parent(frame))
+    gtk_container_add (GTK_CONTAINER (main_vbox), frame);
+
+
+
+
+
 
   frame = gtk_frame_new( "Detected note");
   gtk_container_add (GTK_CONTAINER (main_vbox), frame);
