@@ -28,12 +28,15 @@ struct callbackdata
   GtkWidget *lilypath;
   GtkWidget *midi_audio_output;
   GtkWidget *immediateplayback;
+  GtkWidget *startmidiin;
+  GtkWidget *applytoselection;
   GtkWidget *saveparts;
   GtkWidget *autosave;
   GtkWidget *notation_palette;
   GtkWidget *rhythm_palette;
   GtkWidget *object_palette;
   GtkWidget *articulation_palette;
+  GtkWidget *console_pane;
   GtkWidget *visible_directive_buttons;
   GtkWidget *autoupdate;
 
@@ -46,12 +49,17 @@ struct callbackdata
   GtkWidget *password;
   GtkWidget *sequencer;
   GtkWidget *midi_in;
+
+  GtkWidget *zoom;
+  GtkWidget *system_height;
+
 #ifdef _HAVE_JACK_
   GtkWidget *jacktransport;
   GtkWidget *jacktransport_start_stopped;
 #endif
 #ifdef _HAVE_FLUIDSYNTH_
   GtkWidget *fluidsynth_audio_driver;
+  GtkWidget *fluidsynth_midi_driver;
   GtkWidget *fluidsynth_soundfont;
   GtkWidget *fluidsynth_reverb;
   GtkWidget *fluidsynth_chorus;
@@ -59,7 +67,7 @@ struct callbackdata
   GtkWidget *fluidsynth_period_size;
 
 #endif
-  GtkWidget *texteditor;
+  GtkWidget *display_refresh;
   GtkWidget *midiplayer;
   GtkWidget *audioplayer;
   GtkWidget *denemopath;
@@ -70,10 +78,12 @@ struct callbackdata
   GtkWidget *continuous;
 };
 
-struct callbackdata1
+struct audio_callback_data
 {
-  DenemoGUI *gui;
-  GtkListStore *model;
+  GtkWidget *main_vbox;
+  GtkWidget *fs;
+  GtkWidget *DM;
+  GtkWidget *pas;
 };
 
 /**
@@ -106,6 +116,10 @@ set_preferences (struct callbackdata *cbdata)
    prefs->field =\
     gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(cbdata->field));
 
+#define ASSIGNDOUBLE(field) \
+   prefs->field =\
+     gtk_spin_button_get_value (GTK_SPIN_BUTTON(cbdata->field));
+
 #define ASSIGNCOMBO(field) \
   g_string_assign (prefs->field,\
     (gchar *) gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (cbdata->field)->entry)));
@@ -120,14 +134,15 @@ set_preferences (struct callbackdata *cbdata)
   ASSIGNTEXT(imageviewer)
   ASSIGNTEXT(username)
   ASSIGNTEXT(password)
-  ASSIGNTEXT(texteditor)
   ASSIGNTEXT(midiplayer)
   ASSIGNTEXT(audioplayer)
   ASSIGNTEXT(denemopath)
     // ASSIGNTEXT(sequencer)
 #ifndef _HAVE_FLUIDSYNTH_
+#ifndef G_OS_WIN32
   ASSIGNTEXT(midi_in)
 #endif
+#endif
 #ifdef _HAVE_JACK_
     // ASSIGNBOOLEAN(jacktransport)
     //  ASSIGNBOOLEAN(jacktransport_start_stopped)
@@ -135,12 +150,14 @@ set_preferences (struct callbackdata *cbdata)
 #ifdef _HAVE_FLUIDSYNTH_
   /*TODO save combo as int????*/
   ASSIGNCOMBO(fluidsynth_audio_driver);
+  ASSIGNCOMBO(fluidsynth_midi_driver);
   ASSIGNTEXT(fluidsynth_soundfont);
   ASSIGNBOOLEAN(fluidsynth_reverb)
   ASSIGNBOOLEAN(fluidsynth_chorus)
     ASSIGNINT(fluidsynth_sample_rate)
     ASSIGNINT(fluidsynth_period_size)
 #endif
+  ASSIGNDOUBLE(display_refresh)
   ASSIGNTEXT(temperament)
   ASSIGNBOOLEAN(strictshortcuts)
   ASSIGNBOOLEAN(overlays)
@@ -148,13 +165,19 @@ set_preferences (struct callbackdata *cbdata)
   ASSIGNINT(resolution)
   ASSIGNINT(maxhistory)
 
+  ASSIGNINT(zoom)
+  ASSIGNINT(system_height)
+
 
   ASSIGNCOMBO2(midi_audio_output); 
   
   ASSIGNBOOLEAN(immediateplayback)
+  ASSIGNBOOLEAN(startmidiin)
+  ASSIGNBOOLEAN(applytoselection)
   ASSIGNBOOLEAN(autosave)
   ASSIGNINT(autosave_timeout)
   ASSIGNBOOLEAN(articulation_palette)
+  ASSIGNBOOLEAN(console_pane)
   ASSIGNBOOLEAN(visible_directive_buttons)
   ASSIGNBOOLEAN(autoupdate)
   ASSIGNBOOLEAN(notation_palette)
@@ -167,6 +190,31 @@ set_preferences (struct callbackdata *cbdata)
   writeXMLPrefs (prefs);
 }
 
+static void
+midi_audio_tab_update(GtkWidget *box, gpointer data)
+{
+  struct audio_callback_data *cbdata = (struct audio_callback_data *) data;
+  gchar *output = get_midi_audio_pointer((gchar *)gtk_entry_get_text (GTK_ENTRY (box))); 
+  if (output == Fluidsynth){
+    gtk_widget_hide(cbdata->pas);
+    if(cbdata->DM)
+      gtk_widget_hide(cbdata->DM);
+    gtk_widget_show(cbdata->fs);
+  }
+  else if (output == Jack){
+    gtk_widget_hide(cbdata->pas);
+    gtk_widget_hide(cbdata->fs);
+    if(cbdata->DM)
+      gtk_widget_show(cbdata->DM);
+  }
+  else if (output == Portaudio){
+    gtk_widget_hide(cbdata->fs);
+    if(cbdata->DM)
+      gtk_widget_hide(cbdata->DM);
+    gtk_widget_show(cbdata->pas);
+  }
+}
+
 void
 preferences_change (GtkAction *action, gpointer param)
 {
@@ -179,8 +227,9 @@ preferences_change (GtkAction *action, gpointer param)
   GtkWidget *maxhistory;
   GtkWidget *notebook;
   GtkWidget *hbox;
-  GtkWidget *vbox;
-  GtkWidget *entrywidget;
+  GtkWidget *fs;
+  GtkWidget *DM; 
+
   static struct callbackdata cbdata;
   g_assert (gui != NULL);
 
@@ -197,6 +246,7 @@ preferences_change (GtkAction *action, gpointer param)
   notebook = gtk_notebook_new ();
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), notebook, TRUE,
 		      TRUE, 0);
+#define VBOX main_vbox
 
 #define NEWPAGE(thelabel) \
     main_vbox = gtk_vbox_new (FALSE, 1);\
@@ -209,12 +259,12 @@ preferences_change (GtkAction *action, gpointer param)
     gtk_check_button_new_with_label (thelabel); \
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (field),\
 				(gboolean)Denemo.prefs.field);\
-  gtk_box_pack_start (GTK_BOX (main_vbox), field, FALSE, TRUE, 0);\
+  gtk_box_pack_start (GTK_BOX (VBOX), field, FALSE, TRUE, 0);\
   cbdata.field = field;
 
 #define TEXTENTRY(thelabel, field) \
   hbox = gtk_hbox_new (FALSE, 8);\
-  gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, TRUE, 0);\
+  gtk_box_pack_start (GTK_BOX (VBOX), hbox, FALSE, TRUE, 0);\
   label = gtk_label_new (_(thelabel));\
   gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);\
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);\
@@ -247,17 +297,25 @@ preferences_change (GtkAction *action, gpointer param)
   gtk_box_pack_start (GTK_BOX (hbox), field, FALSE, FALSE, 0);\
   cbdata.field = field;
 
-#define INTENTRY_LIMITS(thelabel, field, min, max) \
+
+#define ENTRY_LIMITS(thelabel, field, min, max, step)	\
   hbox = gtk_hbox_new (FALSE, 8);\
-  gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, TRUE, 0);\
+  gtk_box_pack_start (GTK_BOX (VBOX), hbox, FALSE, TRUE, 0);\
   label = gtk_label_new (thelabel);\
   gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);\
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);\
-  GtkWidget *field = gtk_spin_button_new_with_range (min, max, 1.0);\
+  GtkWidget *field = gtk_spin_button_new_with_range (min, max, step);\
   gtk_spin_button_set_value (GTK_SPIN_BUTTON (field), Denemo.prefs.field);\
   gtk_box_pack_start (GTK_BOX (hbox), field, FALSE, FALSE, 0);\
   cbdata.field = field;
 
+
+
+#define INTENTRY_LIMITS(thelabel, field, min, max) ENTRY_LIMITS(thelabel, field, min, max, 0.1)
+
+#define DOUBLEENTRY_LIMITS  ENTRY_LIMITS
+
+
 #define BUTTON(thelabel, field, thecallback, data) \
   hbox = gtk_hbox_new (FALSE, 8);\
   gtk_box_pack_start (GTK_BOX (vbox1), hbox, FALSE, FALSE, 0);\
@@ -285,12 +343,12 @@ preferences_change (GtkAction *action, gpointer param)
 
 #define COMBOBOX(thelable, field, thelist, settext)\
   hbox = gtk_hbox_new (FALSE, 8);\
-  gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, TRUE, 0);\
+  gtk_box_pack_start (GTK_BOX (VBOX), hbox, FALSE, TRUE, 0);\
   label = gtk_label_new (thelable);\
   gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);\
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);\
   hbox = gtk_hbox_new (FALSE, 8);\
-  gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, TRUE, 0);\
+  gtk_box_pack_start (GTK_BOX (VBOX), hbox, FALSE, TRUE, 0);\
   GtkWidget *field = gtk_combo_new ();\
   gtk_combo_set_popdown_strings (GTK_COMBO (field), thelist);\
   gtk_box_pack_start (GTK_BOX (hbox), field, FALSE, FALSE, 0);\
@@ -299,15 +357,63 @@ preferences_change (GtkAction *action, gpointer param)
   gtk_widget_show (field);\
   cbdata.field = field;
 
-  COMBOBOX("Midi/Audio output", midi_audio_output, output_option_list, Denemo.prefs.midi_audio_output) 
-  BOOLEANENTRY("Play back entered notes immediately", immediateplayback);  
- 
+
   BOOLEANENTRY("Display Note/Rest entry toolbar", notation_palette);
   BOOLEANENTRY("Display articulation palette", articulation_palette);
+  BOOLEANENTRY("Display console pane", console_pane);
   BOOLEANENTRY("Display Titles. Controls etc", visible_directive_buttons);
 
   BOOLEANENTRY("Display rhythm pattern toolbar", rhythm_palette);
   BOOLEANENTRY("Display menu of objects toolbar", object_palette);
+  INTENTRY_LIMITS(_("% Zoom"), zoom, 1, 100);
+  INTENTRY_LIMITS(_("% of display height per system"), system_height, 1, 100);
+
+  /*
+   * Pitch Entry Parameters 
+   */
+  NEWPAGE("Pitch Entry");
+  
+  TEXTENTRY("Temperament", temperament)
+  BOOLEANENTRY("Use Overlays", overlays);
+  BOOLEANENTRY("Continuous Entry", continuous);
+
+  /*
+   * Preferences to do with commands
+   */
+  NEWPAGE("Command Behavior");
+
+  //  TEXTENTRY("Strict", strictshortcuts)
+  BOOLEANENTRY("Apply commands to selection if present", applytoselection);  
+  BOOLEANENTRY("Strict Shortcuts", strictshortcuts);
+
+  /*
+   * External (Helper) Programs 
+   */
+  NEWPAGE("Externals");
+ 
+  TEXTENTRY("Path to Lilypond", lilypath)
+  TEXTENTRY("Pdf Viewer", pdfviewer)
+  TEXTENTRY("File/Internet Browser", browser)
+
+  TEXTENTRY("Image Viewer", imageviewer)
+  TEXTENTRY("Midi Player", midiplayer)
+  TEXTENTRY("Audio Player", audioplayer)
+
+  TEXTENTRY("Default Save Path", denemopath)
+  BOOLEANENTRY("Update the command set on startup", autoupdate);
+   /*
+   * Misc Menu 
+   */
+  NEWPAGE("Misc");
+
+  DOUBLEENTRY_LIMITS(_("Playback Display Refresh"), display_refresh, 0.001, 0.5, 0.002);
+
+
+  INTENTRY_LIMITS(_("Excerpt Resolution"), resolution, 72, 600);
+
+  INTENTRY(_("Max recent files"), maxhistory)
+  TEXTENTRY("User Name", username)
+  PASSWORDENTRY("Password for Denemo.org", password)
 
 
   hbox = gtk_hbox_new (FALSE, 8);
@@ -330,59 +436,33 @@ preferences_change (GtkAction *action, gpointer param)
 
 
   BOOLEANENTRY("Autosave Parts", saveparts);
-  //TEXTENTRY("Sequencer Device", sequencer)
-#ifndef _HAVE_FLUIDSYNTH_
-  TEXTENTRY("Midi Input Device", midi_in)
-#endif
 
-  /*
-   * Pitch Entry Parameters 
-   */
-  NEWPAGE("Pitch Entry");
-
-  TEXTENTRY("Temperament", temperament)
-  BOOLEANENTRY("Use Overlays", overlays);
-  BOOLEANENTRY("Continuous Entry", continuous);
-
-  /*
-   * Shortcut control 
-   */
-  NEWPAGE("Shortcuts");
-
-  //  TEXTENTRY("Strict", strictshortcuts)
-  BOOLEANENTRY("Strict Shortcuts", strictshortcuts);
-
-  /*
-   * External (Helper) Programs 
-   */
-  NEWPAGE("Externals");
+  static struct audio_callback_data audio_cbdata;
+  NEWPAGE("Audio/MIDI")
  
-  TEXTENTRY("Path to Lilypond", lilypath)
-  TEXTENTRY("Pdf Viewer", pdfviewer)
-  TEXTENTRY("File/Internet Browser", browser)
+  {
+    hbox = gtk_hbox_new (FALSE, 8);
+    gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, TRUE, 0);
+    label = gtk_label_new ("");	
+    gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+    gtk_label_set_markup(GTK_LABEL (label), _("<span background=\"red\">Warning: changes only have effect after quitting and re-starting Denemo</span>"));
+    gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
+    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+  }
+  BOOLEANENTRY("Play back entered notes immediately", immediateplayback);  
+  BOOLEANENTRY("Auto-start midi in", startmidiin);
 
-  TEXTENTRY("Image Viewer", imageviewer)
-  TEXTENTRY("Text Editor", texteditor)
-  TEXTENTRY("Midi Player", midiplayer)
-  TEXTENTRY("Audio Player", audioplayer)
-
-  TEXTENTRY("Default Save Path", denemopath)
-  BOOLEANENTRY("Update the command set on startup", autoupdate);
-   /*
-   * Misc Menu 
-   */
-  NEWPAGE("Misc");
+  COMBOBOX("Midi/Audio output", midi_audio_output, output_option_list, Denemo.prefs.midi_audio_output)
+  g_signal_connect(G_OBJECT(GTK_COMBO(midi_audio_output)->entry), "changed",
+  G_CALLBACK( GTK_SIGNAL_FUNC(midi_audio_tab_update) ), &audio_cbdata);
  
-  INTENTRY_LIMITS(_("Excerpt Resolution"), resolution, 72, 600);
-
-  INTENTRY(_("Max recent files"), maxhistory)
-  TEXTENTRY("User Name", username)
-  PASSWORDENTRY("Password for Denemo.org", password)
   /*
    * Fluidsynth Menu
    */
 #ifdef _HAVE_FLUIDSYNTH_
-  NEWPAGE("Fluidsynth");
+#undef VBOX
+#define VBOX fs
+
   /* Start/Restart Button */
   //BUTTON("Start/Restart FLUIDSYNTH", fluid_restart, fluidsynth_start_restart, NULL)
 
@@ -391,21 +471,30 @@ preferences_change (GtkAction *action, gpointer param)
    *defaults are:
    *jack (Linux), dsound (Windows), sndman (MacOS9), coreaudio (Mac OS X), dart (OS/2) 
    */
+  fs = gtk_vbox_new (FALSE, 8);
+  gtk_box_pack_start (GTK_BOX (main_vbox), fs, FALSE, TRUE, 0);
+  //gtk_widget_show(fs);
 #ifdef G_OS_WIN32
-  gchar *driver_options[5] = {"portaudio", "jack"};
+  gchar *driver_options[5] = {"portaudio", "jack"}; //Isn't there more options?
+  gchar *midi_driver_options[1] = {"portaudio"}; //Is this correct?
 #else
   gchar *driver_options[5] = {"alsa", "jack", "oss", "pulseaudio", "portaudio"};
+  gchar *midi_driver_options[2] = {"alsa_seq", "oss"};
 #endif
   GList *driver_option_list = NULL;
+  GList *midi_driver_option_list = NULL;
   gint i;
   for (i=0;i<G_N_ELEMENTS(driver_options);i++)
     driver_option_list = g_list_append (driver_option_list, driver_options[i]);
-
+  for (i=0;i<G_N_ELEMENTS(midi_driver_options);i++)
+    midi_driver_option_list = g_list_append (midi_driver_option_list, midi_driver_options[i]);
+ 
   COMBOBOX("Audio Driver", fluidsynth_audio_driver, driver_option_list, Denemo.prefs.fluidsynth_audio_driver->str)
+  COMBOBOX("Midi Driver", fluidsynth_midi_driver, midi_driver_option_list, Denemo.prefs.fluidsynth_midi_driver->str)	  
   TEXTENTRY("Soundfont", fluidsynth_soundfont)	
   
   hbox = gtk_hbox_new (FALSE, 8);
-  gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (VBOX), hbox, FALSE, TRUE, 0);
   GtkWidget *button = gtk_button_new_with_label (_("Choose Soundfont"));
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
@@ -416,41 +505,51 @@ preferences_change (GtkAction *action, gpointer param)
   BOOLEANENTRY("Enable Chorus on soundfont", fluidsynth_chorus)
     INTENTRY_LIMITS(_("Sample Rate"), fluidsynth_sample_rate, 0, 96000);
    INTENTRY_LIMITS(_("Period Size"), fluidsynth_period_size, 0, 2048);
-
+#undef VBOX
 #endif
-  /**
-   * Device Manager
-   */
+
 #ifdef _HAVE_JACK_
-  NEWPAGE("MIDI Device Manager");
-  
-  GtkWidget *mhbox = gtk_hbox_new(FALSE, 5);
-  gtk_box_pack_start (GTK_BOX (main_vbox), mhbox, FALSE, FALSE, 0);
-
-  GtkWidget *vbox1 = gtk_vbox_new(FALSE, 5);
-  gtk_box_pack_start (GTK_BOX (mhbox), vbox1, FALSE, FALSE, 0);
-  
-  //None of this is working right now anyway  
-  //BOOLEANENTRY("Enable Jack Transport", jacktransport);
-  //BOOLEANENTRY("Jack Transport starts stopped", jacktransport_start_stopped);
-  //BOOLEANENTRY("Enable Jack at startup", jack_at_startup);
-
-  //BUTTON("Start/Restart Jack Client", jack_restart, jack_start_restart, NULL);
-  BUTTON("Add Device", midi_add_device, device_manager_create_device, NULL); 
-  BUTTON("Remove Device", midi_remove_device, device_manager_remove_device, NULL); 
-  BUTTON("Add Port", midi_device_add_port, device_manager_create_port, NULL);
-  BUTTON("Remove Port", midi_device_remove_port, device_manager_remove_port, NULL);
-  
-  GtkWidget *vbox2 = gtk_vbox_new(FALSE, 5);
-  gtk_box_pack_start (GTK_BOX (mhbox), vbox2, FALSE, FALSE, 0);
-  
-  GtkWidget *view = DeviceManager();
-  g_print("the view %p is %d\n", view, GTK_IS_WIDGET(view));
-  gtk_box_pack_start (GTK_BOX (vbox2), view, FALSE, FALSE, 0);
-  
-  gtk_widget_show (view);
-  device_manager_refresh_model();
+  DM = DeviceManager();
+  gtk_box_pack_start (GTK_BOX (main_vbox), DM, FALSE, TRUE, 0);
+  //gtk_widget_show(DM);
+#else
+  DM = NULL;
 #endif
+#define VBOX pas
+  GtkWidget *pas;
+  pas = gtk_vbox_new (FALSE, 8);
+  gtk_box_pack_start (GTK_BOX (main_vbox), pas, FALSE, TRUE, 0);
+  //TEXTENTRY("Sequencer Device", sequencer)
+#ifndef G_OS_WIN32
+  TEXTENTRY("Midi Input Device", midi_in) 
+#endif
+#undef VBOX
+
+  gtk_widget_show_all (dialog);
+
+  if (Denemo.prefs.midi_audio_output == Fluidsynth){
+    gtk_widget_hide(pas);
+    if(DM)
+      gtk_widget_hide(DM);
+    gtk_widget_show(fs);
+  }
+  else if (Denemo.prefs.midi_audio_output == Jack){
+    gtk_widget_hide(pas);
+    gtk_widget_hide(fs);
+    if(DM)
+      gtk_widget_show(DM);
+  }
+  else if (Denemo.prefs.midi_audio_output == Portaudio){
+    gtk_widget_hide(fs);
+    if(DM)
+      gtk_widget_hide(DM);
+    gtk_widget_show(pas);
+  }
+
+  audio_cbdata.main_vbox = main_vbox;
+  audio_cbdata.fs = fs;
+  audio_cbdata.DM = DM;
+  audio_cbdata.pas = pas;
 
 #define SETCALLBACKDATA(field) \
   cbdata.field = field;
@@ -462,13 +561,13 @@ preferences_change (GtkAction *action, gpointer param)
   
   
   //gtk_widget_grab_focus (lilypath);
-  gtk_widget_show_all (dialog);
+  
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
     {
       set_preferences (&cbdata);
     }
 #ifdef _HAVE_JACK_
-  gtk_container_remove(GTK_CONTAINER(vbox2), view);
+  //gtk_container_remove(GTK_CONTAINER(vbox2), view);
 #endif
   gtk_widget_destroy (dialog);
 }
