@@ -83,22 +83,22 @@ initprefs ()
   ret->midi_audio_output = Portaudio;
 //#endif
 
-#ifdef _HAVE_FLUIDSYNTH_ 
-  ret->midi_audio_output = Fluidsynth;	  
-#endif
 #ifdef _HAVE_JACK_
   ret->midi_audio_output = Jack;
 #endif
 
+#ifdef _HAVE_FLUIDSYNTH_ 
+  ret->midi_audio_output = Fluidsynth;	  
+#endif
+
 #ifdef G_OS_WIN32
-  ret->browser = g_string_new ("firefox");
-  ret->midiplayer = g_string_new ("wmplayer");
-  ret->audioplayer = g_string_new ("wmplayer");
-  ret->lilypath = g_string_new ("lilypond-windows");
-  ret->pdfviewer = g_string_new ("acrord32");
-  ret->imageviewer = g_string_new ("mspaint");
-  ret->texteditor = g_string_new ("wordpad");
-  ret->midiplayer = g_string_new("wmplayer");
+  ret->browser = g_string_new ("");//use file association
+  ret->midiplayer = g_string_new ("");
+  ret->audioplayer = g_string_new ("");
+  ret->lilypath = g_string_new ("lilypond-windows");//We don't assume the file assoc works - we are installing this anyway to a known place
+  ret->pdfviewer = g_string_new ("");
+  ret->imageviewer = g_string_new ("");
+  ret->midiplayer = g_string_new("");
 #else /* !G_OS_WIN32 */
   ret->browser = g_string_new ("firefox");
   ret->midiplayer = g_string_new ("playmidi");
@@ -106,7 +106,6 @@ initprefs ()
   ret->lilypath = g_string_new ("lilypond");
   ret->pdfviewer = g_string_new ("xpdf");
   ret->imageviewer = g_string_new ("eog");
-  ret->texteditor = g_string_new ("xedit");
 #endif /* !G_OS_WIN32 */
   ret->sequencer = g_string_new ("/dev/sequencer");
   ret->midi_in = g_string_new ("/dev/midi");
@@ -116,6 +115,7 @@ initprefs ()
   ret->temperament = g_string_new("Equal");
   ret->strictshortcuts = FALSE;
   ret->resolution = 300;
+  ret->display_refresh = 0.01;
   ret->overlays = FALSE;
   ret->continuous = TRUE;
 #ifdef _HAVE_JACK_
@@ -128,14 +128,21 @@ initprefs ()
   	for some reason the default for linux is jack */
 #ifdef G_OS_WIN32
   ret->fluidsynth_audio_driver = g_string_new ("portaudio");
+  ret->fluidsynth_midi_driver = g_string_new ("");
 #else
-  ret->fluidsynth_audio_driver = g_string_new ((char *)fluidsynth_get_default_audio_driver());
+  ret->fluidsynth_audio_driver = g_string_new ("alsa");
+  ret->fluidsynth_midi_driver = g_string_new ("alsa_seq");
 #endif
   gchar *soundfontpath = g_build_filename (get_data_dir (), "soundfonts",
 		                                           "A320U.sf2", NULL);
   ret->fluidsynth_soundfont = g_string_new(soundfontpath);
-  ret->fluidsynth_sample_rate = 0;//do not set
 
+
+#ifdef G_OS_WIN32
+  ret->fluidsynth_sample_rate = 22050;//the worst case slow machine
+#else
+  ret->fluidsynth_sample_rate = 0;//do not set
+#endif
 #endif
   ret->saveparts = FALSE;
   ret->rtcs = TRUE;
@@ -147,12 +154,16 @@ initprefs ()
   ret->maxhistory = 20;
   ret->notation_palette = TRUE;
   ret->articulation_palette = FALSE;
+  ret->console_pane = TRUE;
   ret->visible_directive_buttons = TRUE;
   ret->autoupdate = FALSE;
   ret->rhythm_palette = TRUE;
   ret->object_palette = TRUE;
   ret->history = g_queue_new ();
-
+  ret->zoom = 100;
+  ret->system_height = 100;
+  ret->applytoselection = TRUE;
+  
   /* Read values from systemwide preferences file */
 
   readxmlprefsFile (systemwide);
@@ -325,11 +336,41 @@ parseConfig (xmlDocPtr doc, xmlNodePtr cur, DenemoPrefs * prefs)
 	  xmlChar *tmp = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);\
 	  if(tmp)\
 	    {\
-              define_scheme_variable("DenemoPref_" #field, tmp, NULL);\
+              define_scheme_int_variable("DenemoPref_" #field, atoi(tmp), NULL); \
 	      prefs->field = atoi ((gchar *) tmp);\
 	      xmlFree (tmp);\
 	    }\
-	}\
+	}
+
+#define READDOUBLEXMLENTRY(field) \
+      else if (0 ==\
+	       xmlStrcmp (cur->name, (const xmlChar *) #field))\
+	{\
+	  xmlChar *tmp = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);\
+	  if(tmp)\
+	    {\
+              define_scheme_int_variable("DenemoPref_" #field, atof(tmp), NULL); \
+	      prefs->field = atof ((gchar *) tmp);\
+	      xmlFree (tmp);\
+	    }\
+	}
+
+
+#define READBOOLXMLENTRY(field) \
+      else if (0 ==\
+	       xmlStrcmp (cur->name, (const xmlChar *) #field))\
+	{\
+	  xmlChar *tmp = xmlNodeListGetString (doc, cur->xmlChildrenNode, 1);\
+	  if(tmp)\
+	    {\
+              define_scheme_bool_variable("DenemoPref_" #field, atoi(tmp), NULL); \
+	      prefs->field = atoi ((gchar *) tmp);\
+	      xmlFree (tmp);\
+	    }\
+	}
+
+
+
 
       if (0 == xmlStrcmp (cur->name, (const xmlChar *) "lilypath"))
 	{
@@ -362,7 +403,7 @@ parseConfig (xmlDocPtr doc, xmlNodePtr cur, DenemoPrefs * prefs)
 	  if(tmp)
 	    {
 	      gchar *curname = g_strdup_printf("DenemoPref_%s", cur->name);
-	      define_scheme_variable(curname, tmp, NULL); 
+	      define_scheme_int_variable(curname, atoi(tmp), NULL); 
 	      g_free(curname);
 	      prefs->autosave_timeout = atoi ((gchar *) tmp);
 	      if(prefs->autosave_timeout <1) prefs->autosave_timeout = 1;
@@ -377,7 +418,7 @@ parseConfig (xmlDocPtr doc, xmlNodePtr cur, DenemoPrefs * prefs)
 	  if(tmp)
 	    {
 	      gchar *curname = g_strdup_printf("DenemoPref_%s", cur->name);
-	      define_scheme_variable(curname, tmp, NULL); 
+	      define_scheme_int_variable(curname, atoi(tmp), NULL); 
 	      g_free(curname);
 	      prefs->maxhistory = atoi ((gchar *) tmp);
 	      if(prefs->maxhistory <1) prefs->maxhistory = 1;
@@ -387,46 +428,53 @@ parseConfig (xmlDocPtr doc, xmlNodePtr cur, DenemoPrefs * prefs)
       READXMLENTRY(csoundcommand)
       READXMLENTRY(csoundorcfile)
 	
-      READINTXMLENTRY(rtcs)
-      READINTXMLENTRY(autosave)
+      READBOOLXMLENTRY(rtcs)
+      READBOOLXMLENTRY(autosave)
       
       READXMLENTRY(pdfviewer)
       READXMLENTRY(imageviewer)    
       READXMLENTRY(username)    
       READXMLENTRY(password)           
-      READXMLENTRY(texteditor)            
       READXMLENTRY(denemopath)          
       READXMLENTRY(temperament)
       READXMLENTRY(midi_in)
       READXMLENTRY(sequencer)
       READXMLENTRY2(midi_audio_output)
       
-      READINTXMLENTRY(createclones)
-      READINTXMLENTRY(immediateplayback) 
+      READBOOLXMLENTRY(createclones)
+      READBOOLXMLENTRY(immediateplayback) 
+      READBOOLXMLENTRY(applytoselection) 
+      READBOOLXMLENTRY(startmidiin) 
+      READINTXMLENTRY(mode) 
   
-      READINTXMLENTRY(strictshortcuts)
+      READBOOLXMLENTRY(strictshortcuts)
       READINTXMLENTRY(resolution)
-      READINTXMLENTRY(overlays)
-      READINTXMLENTRY(continuous)
-      READINTXMLENTRY(jacktransport)
-      READINTXMLENTRY(jacktransport_start_stopped)
-      READINTXMLENTRY(lilyentrystyle)
-      READINTXMLENTRY(notation_palette)
-      READINTXMLENTRY(articulation_palette)
-      READINTXMLENTRY(visible_directive_buttons)
-      READINTXMLENTRY(rhythm_palette) 
-      READINTXMLENTRY(object_palette)
-      READINTXMLENTRY(autoupdate) 
+      READDOUBLEXMLENTRY(display_refresh)
+      READBOOLXMLENTRY(overlays)
+      READBOOLXMLENTRY(continuous)
+      READBOOLXMLENTRY(jacktransport)
+      READBOOLXMLENTRY(jacktransport_start_stopped)
+      READBOOLXMLENTRY(lilyentrystyle)
+      READBOOLXMLENTRY(notation_palette)
+      READBOOLXMLENTRY(articulation_palette)
+      READBOOLXMLENTRY(console_pane)
+      READBOOLXMLENTRY(visible_directive_buttons)
+      READBOOLXMLENTRY(rhythm_palette) 
+      READBOOLXMLENTRY(object_palette)
+      READBOOLXMLENTRY(autoupdate) 
      
       READXMLENTRY(fluidsynth_audio_driver)
+      READXMLENTRY(fluidsynth_midi_driver)
       READXMLENTRY(fluidsynth_soundfont)
-      READINTXMLENTRY(fluidsynth_reverb)
-      READINTXMLENTRY(fluidsynth_chorus)
+      READBOOLXMLENTRY(fluidsynth_reverb)
+      READBOOLXMLENTRY(fluidsynth_chorus)
       READINTXMLENTRY(fluidsynth_sample_rate)
       READINTXMLENTRY(fluidsynth_period_size)
+      READINTXMLENTRY(zoom)
+      READINTXMLENTRY(system_height)
 
 
-      READINTXMLENTRY(saveparts)
+      READBOOLXMLENTRY(saveparts)
       
       cur = cur->next;
     }
@@ -595,6 +643,24 @@ newXMLIntChild (xmlNodePtr parent, const xmlChar * name, gint content)
   return xmlNewChild (parent, NULL, name, (xmlChar *) sIntBuf);
 }
 
+/**
+ * Output an double child as a child of the given node.
+ *
+ * @param parent - pointer to child nodes parent
+ * @param name -  name for the child node
+ * @param content - data to add to the child node
+ */
+static xmlNodePtr
+newXMLDoubleChild (xmlNodePtr parent, const xmlChar * name, gdouble content)
+{
+  static GString* str;
+  if(str==NULL)
+    str = g_string_new("");
+  g_string_printf(str, "%f", content);
+  return xmlNewChild (parent, NULL, name, (xmlChar *) str->str);
+}
+
+
 /**
  * Write the denemorc file
  *
@@ -632,7 +698,6 @@ writeXMLPrefs (DenemoPrefs * prefs)
   xmlDocPtr doc;
   xmlNodePtr parent, child;
   static GString *localrc = NULL;
-
   if (!localrc)
     {
       localrc = g_string_new (locatedotdenemo ());
@@ -675,50 +740,81 @@ writeXMLPrefs (DenemoPrefs * prefs)
   WRITEXMLENTRY(imageviewer)
   WRITEXMLENTRY(username)
   WRITEXMLENTRY(password)
-  WRITEXMLENTRY(texteditor) 
   WRITEXMLENTRY(denemopath)
   WRITEXMLENTRY(temperament)
   WRITEXMLENTRY(midi_in)
   WRITEXMLENTRY(sequencer)
 
 #define WRITEINTXMLENTRY(field){ \
-    gchar *def = g_strdup("Holds the value of the user's " #field " preference");\
-    gchar *value = g_strdup_printf("%d", prefs->field);\
+    gchar *def = g_strdup("Holds the interger value of the user's " #field " preference");\
+    gint value = prefs->field;\
     gchar *curname = g_strdup_printf("DenemoPref_%s", #field);\
-    define_scheme_variable(curname, value, def);\
+    define_scheme_int_variable(curname, value, def);\
     g_free(curname);\
-    g_free(value);\
     g_free(def);\
   newXMLIntChild (child, (xmlChar *) #field,\
 		  prefs->field);}
 
-  WRITEINTXMLENTRY(autosave)
+#define WRITEDOUBLEXMLENTRY(field){ \
+    gchar *def = g_strdup("Holds the interger value of the user's " #field " preference");\
+    gdouble value = prefs->field;\
+    gchar *curname = g_strdup_printf("DenemoPref_%s", #field);\
+    define_scheme_double_variable(curname, value, def);\
+    g_free(curname);\
+    g_free(def);\
+  newXMLDoubleChild (child, (xmlChar *) #field,\
+		  prefs->field);}
+
+
+
+
+
+#define WRITEBOOLXMLENTRY(field){ \
+    gchar *def = g_strdup("Holds #t or #f, the user's " #field " preference");\
+    gboolean value = prefs->field;\
+    gchar *curname = g_strdup_printf("DenemoPref_%s", #field);\
+    define_scheme_bool_variable(curname, value, def);\
+    g_free(curname);\
+    g_free(def);\
+  newXMLIntChild (child, (xmlChar *) #field,\
+		  prefs->field);}
+
+  WRITEBOOLXMLENTRY(autosave)
   WRITEINTXMLENTRY(autosave_timeout)
   WRITEINTXMLENTRY(maxhistory)
-  WRITEINTXMLENTRY(saveparts)
-  WRITEINTXMLENTRY(createclones)
-  WRITEINTXMLENTRY(lilyentrystyle)
-  WRITEINTXMLENTRY(immediateplayback)
-  WRITEINTXMLENTRY(strictshortcuts)
+  WRITEBOOLXMLENTRY(saveparts)
+  WRITEBOOLXMLENTRY(createclones)
+  WRITEBOOLXMLENTRY(lilyentrystyle)
+  WRITEBOOLXMLENTRY(immediateplayback)
+  WRITEBOOLXMLENTRY(applytoselection)
+  WRITEBOOLXMLENTRY(startmidiin)
+  WRITEINTXMLENTRY(mode)
+  WRITEBOOLXMLENTRY(strictshortcuts)
   WRITEINTXMLENTRY(resolution)
-  WRITEINTXMLENTRY(overlays)
-  WRITEINTXMLENTRY(continuous)
-  WRITEINTXMLENTRY(jacktransport)
-  WRITEINTXMLENTRY(jacktransport_start_stopped)
+  WRITEDOUBLEXMLENTRY(display_refresh)
+
+  WRITEBOOLXMLENTRY(overlays)
+  WRITEBOOLXMLENTRY(continuous)
+  WRITEBOOLXMLENTRY(jacktransport)
+  WRITEBOOLXMLENTRY(jacktransport_start_stopped)
   WRITEINTXMLENTRY(rtcs)
-  WRITEINTXMLENTRY(notation_palette)
-  WRITEINTXMLENTRY(articulation_palette)
-  WRITEINTXMLENTRY(visible_directive_buttons)
-  WRITEINTXMLENTRY(autoupdate)
-  WRITEINTXMLENTRY(rhythm_palette)
-  WRITEINTXMLENTRY(object_palette)
+  WRITEBOOLXMLENTRY(notation_palette)
+  WRITEBOOLXMLENTRY(articulation_palette)
+  WRITEBOOLXMLENTRY(console_pane)
+  WRITEBOOLXMLENTRY(visible_directive_buttons)
+  WRITEBOOLXMLENTRY(autoupdate)
+  WRITEBOOLXMLENTRY(rhythm_palette)
+  WRITEBOOLXMLENTRY(object_palette)
   WRITEXMLENTRY2(midi_audio_output)
   WRITEXMLENTRY(fluidsynth_audio_driver)
+  WRITEXMLENTRY(fluidsynth_midi_driver)
   WRITEXMLENTRY(fluidsynth_soundfont)
-  WRITEINTXMLENTRY(fluidsynth_reverb)
-  WRITEINTXMLENTRY(fluidsynth_chorus)
+  WRITEBOOLXMLENTRY(fluidsynth_reverb)
+  WRITEBOOLXMLENTRY(fluidsynth_chorus)
   WRITEINTXMLENTRY(fluidsynth_sample_rate)
   WRITEINTXMLENTRY(fluidsynth_period_size)
+  WRITEINTXMLENTRY(zoom)
+  WRITEINTXMLENTRY(system_height)
 
   WRITEXMLENTRY(browser) 
   
