@@ -336,23 +336,33 @@ open_viewer(GPid pid, gint status, gchar *filename, gboolean is_png){
 
     arguments = pdf;  
   }
-
-  g_spawn_async (locatedotdenemo (),		/* dir */
-		 arguments, NULL,	/* env */
-		 G_SPAWN_SEARCH_PATH, /* search in path for executable */
-		 NULL,	/* child setup func */
-		 NULL,		/* user data */		
-		 NULL, /* FIXME &pid see g_spawn_close_pid(&pid) */
-		 &err);
-  
-  if (err != NULL)
-    {
+  if((!is_png && (Denemo.prefs.pdfviewer->len==0))||
+     (is_png && (Denemo.prefs.imageviewer->len==0))) {
+    gboolean ok =  run_file_association(printfile);
+    if(!ok) {
+      err = g_error_new(G_FILE_ERROR, -1, "Could not run file assoc for %s", is_png?".png":".pdf");
+      g_warning("Could not run the file association for a %s file\n", is_png?".png":".pdf");
+    }
+  }
+  else {
+    g_spawn_async (locatedotdenemo (),		/* dir */
+		   arguments, NULL,	/* env */
+		   G_SPAWN_SEARCH_PATH, /* search in path for executable */
+		   NULL,	/* child setup func */
+		   NULL,		/* user data */		
+		   NULL, /* FIXME &pid see g_spawn_close_pid(&pid) */
+		   &err);
+  }
+  if (err != NULL) {
+    if(Denemo.prefs.pdfviewer->len) {
       g_warning ("Failed to find %s", Denemo.prefs.pdfviewer->str);
       warningdialog("Cannot display: Check Edit->Preferences->externals\nfor your PDF viewer");
-      g_warning ("%s", err->message);
-      err = NULL;
-      if(err) g_error_free (err);
-    }
+    } else 
+      warningdialog(err->message);
+    g_warning ("%s", err->message);
+    if(err) g_error_free (err);
+    err = NULL;
+  }
   g_free(printfile);
 }
 
@@ -1115,7 +1125,7 @@ void refresh_print_view (gboolean preview_only) {
 	         &printpreview_errors,		/* stderr */
 		 &error);
   g_free(lilyfile);
-  g_child_watch_add (printviewpid, (GChildWatchFunc)printview_finished  /*  GChildWatchFunc function */, preview_only);
+  g_child_watch_add (printviewpid, (GChildWatchFunc)printview_finished  /*  GChildWatchFunc function */, (gpointer)preview_only);
 }
 
 /* callback to print whole of score */
@@ -1152,7 +1162,7 @@ popup_print_preview_menu(void) {
   GtkWidget *item = gtk_menu_item_new_with_label("Refresh Print Preview");
 
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-  g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(refresh_print_view), TRUE);
+  g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(refresh_print_view), (gpointer)TRUE);
   item = gtk_menu_item_new_with_label("Drag to desired offset");
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
   g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(start_drag), &offsetting);
