@@ -3,6 +3,10 @@ gchar *text;
 SCM scheme_CursorLeft(SCM optional);
 install_scm_function ("d-CursorLeft", scheme_CursorLeft);
 g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "CursorLeft")), "scm", (gpointer)1);
+/*MoveCursorLeft movecursorleft*/
+SCM scheme_MoveCursorLeft(SCM optional);
+install_scm_function ("d-MoveCursorLeft", scheme_MoveCursorLeft);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "MoveCursorLeft")), "scm", (gpointer)1);
 /*CursorDown cursordown*/
 SCM scheme_CursorDown(SCM optional);
 install_scm_function ("d-CursorDown", scheme_CursorDown);
@@ -15,10 +19,22 @@ g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "CursorUp")), "scm", (gpoi
 SCM scheme_CursorRight(SCM optional);
 install_scm_function ("d-CursorRight", scheme_CursorRight);
 g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "CursorRight")), "scm", (gpointer)1);
+/*MoveCursorRight movecursorright*/
+SCM scheme_MoveCursorRight(SCM optional);
+install_scm_function ("d-MoveCursorRight", scheme_MoveCursorRight);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "MoveCursorRight")), "scm", (gpointer)1);
 /*GoToMark goto_mark*/
 SCM scheme_GoToMark(SCM optional);
 install_scm_function ("d-GoToMark", scheme_GoToMark);
 g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "GoToMark")), "scm", (gpointer)1);
+/*SwapPointAndMark swap_point_and_mark*/
+SCM scheme_SwapPointAndMark(SCM optional);
+install_scm_function ("d-SwapPointAndMark", scheme_SwapPointAndMark);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "SwapPointAndMark")), "scm", (gpointer)1);
+/*GoToSelectionStart goto_selection_start*/
+SCM scheme_GoToSelectionStart(SCM optional);
+install_scm_function ("d-GoToSelectionStart", scheme_GoToSelectionStart);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "GoToSelectionStart")), "scm", (gpointer)1);
 /*PushPosition PushPosition*/
 SCM scheme_PushPosition(SCM optional);
 install_scm_function ("d-PushPosition", scheme_PushPosition);
@@ -39,6 +55,14 @@ g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "StaffUp")), "scm", (gpoin
 SCM scheme_StaffDown(SCM optional);
 install_scm_function ("d-StaffDown", scheme_StaffDown);
 g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "StaffDown")), "scm", (gpointer)1);
+/*MoveToStaffUp movetostaffup*/
+SCM scheme_MoveToStaffUp(SCM optional);
+install_scm_function ("d-MoveToStaffUp", scheme_MoveToStaffUp);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "MoveToStaffUp")), "scm", (gpointer)1);
+/*MoveToStaffDown movetostaffdown*/
+SCM scheme_MoveToStaffDown(SCM optional);
+install_scm_function ("d-MoveToStaffDown", scheme_MoveToStaffDown);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "MoveToStaffDown")), "scm", (gpointer)1);
 /*MeasureLeft measureleft*/
 SCM scheme_MeasureLeft(SCM optional);
 install_scm_function ("d-MeasureLeft", scheme_MeasureLeft);
@@ -47,6 +71,14 @@ g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "MeasureLeft")), "scm", (g
 SCM scheme_MeasureRight(SCM optional);
 install_scm_function ("d-MeasureRight", scheme_MeasureRight);
 g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "MeasureRight")), "scm", (gpointer)1);
+/*MoveToMeasureLeft movetomeasureleft*/
+SCM scheme_MoveToMeasureLeft(SCM optional);
+install_scm_function ("d-MoveToMeasureLeft", scheme_MoveToMeasureLeft);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "MoveToMeasureLeft")), "scm", (gpointer)1);
+/*MoveToMeasureRight movetomeasureright*/
+SCM scheme_MoveToMeasureRight(SCM optional);
+install_scm_function ("d-MoveToMeasureRight", scheme_MoveToMeasureRight);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "MoveToMeasureRight")), "scm", (gpointer)1);
 /*A go_to_A_key*/
 SCM scheme_A(SCM optional);
 install_scm_function ("d-A", scheme_A);
@@ -1011,6 +1043,14 @@ g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "VoiceUp")), "scm", (gpoin
 SCM scheme_VoiceDown(SCM optional);
 install_scm_function ("d-VoiceDown", scheme_VoiceDown);
 g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "VoiceDown")), "scm", (gpointer)1);
+/*MoveToVoiceUp movetovoiceup*/
+SCM scheme_MoveToVoiceUp(SCM optional);
+install_scm_function ("d-MoveToVoiceUp", scheme_MoveToVoiceUp);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "MoveToVoiceUp")), "scm", (gpointer)1);
+/*MoveToVoiceDown movetovoicedown*/
+SCM scheme_MoveToVoiceDown(SCM optional);
+install_scm_function ("d-MoveToVoiceDown", scheme_MoveToVoiceDown);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "MoveToVoiceDown")), "scm", (gpointer)1);
 /*AddBefore newstaffbefore*/
 SCM scheme_AddBefore(SCM optional);
 install_scm_function ("d-AddBefore", scheme_AddBefore);
@@ -1175,6 +1215,14 @@ g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "GoToBeginning")), "scm",
 SCM scheme_GoToEnd(SCM optional);
 install_scm_function ("d-GoToEnd", scheme_GoToEnd);
 g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "GoToEnd")), "scm", (gpointer)1);
+/*MoveToBeginning movetostart*/
+SCM scheme_MoveToBeginning(SCM optional);
+install_scm_function ("d-MoveToBeginning", scheme_MoveToBeginning);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "MoveToBeginning")), "scm", (gpointer)1);
+/*MoveToEnd movetoend*/
+SCM scheme_MoveToEnd(SCM optional);
+install_scm_function ("d-MoveToEnd", scheme_MoveToEnd);
+g_object_set_data(G_OBJECT(action_of_name(Denemo.map, "MoveToEnd")), "scm", (gpointer)1);
 /*NextMovement next_movement*/
 SCM scheme_NextMovement(SCM optional);
 install_scm_function ("d-NextMovement", scheme_NextMovement);
