@@ -35,11 +35,40 @@ static void
 undo (DenemoGUI * gui);
 static void
 redo (DenemoGUI * gui);
-static GList *copybuffer = NULL;
+static GList *copybuffer = NULL; // this is a list one for each staff of lists of objects
 
 static gint staffsinbuffer = 0;
 static gint measurebreaksinbuffer = 0;
 
+static GList *clipboards = NULL;
+
+static GList *clone_obj_list(GList *g) {
+  GList *ret=NULL;
+  do {
+    ret = g_list_append(ret, dnm_clone_object(g->data));
+  } while((g=g->next));
+  return ret;
+}
+void push_clipboard (void) {
+  GList *thecopy = NULL;
+  GList *g;
+  for(g=copybuffer;g;g=g->next) {
+    thecopy = g_list_append(thecopy, clone_obj_list(g->data));
+  }
+  clipboards = g_list_prepend(clipboards, thecopy);
+}
+
+gboolean pop_clipboard(void) {
+
+  GList *thecopy = NULL;
+  if(clipboards==NULL)
+    return FALSE;
+  thecopy = clipboards->data;
+  clipboards = g_list_remove(clipboards, thecopy);
+  clearbuffer();
+  copybuffer = thecopy;
+}
+
 
 /**
  *  sets current object to the given cursor position
@@ -601,30 +630,32 @@ unset_mark (DenemoGUI * gui)
 
 
 gboolean in_selection(DenemoScore *si) {
-  if(si->currentstaffnum >= si->firststaffmarked &&
-     si->currentstaffnum <= si->laststaffmarked) {
-    if(si->currentmeasurenum == si->firstmeasuremarked) {
-      if (si->currentmeasurenum == si->lastmeasuremarked) {
-	if((si->cursor_x >= si->firstobjmarked) &&
-	   (si->cursor_x <= si->lastobjmarked))
-	  return TRUE;
-	else return FALSE;
+  if(si->markstaffnum) {
+    if(si->currentstaffnum >= si->firststaffmarked &&
+       si->currentstaffnum <= si->laststaffmarked) {
+      if(si->currentmeasurenum == si->firstmeasuremarked) {
+	if (si->currentmeasurenum == si->lastmeasuremarked) {
+	  if((si->cursor_x >= si->firstobjmarked) &&
+	     (si->cursor_x <= si->lastobjmarked))
+	    return TRUE;
+	  else return FALSE;
+	}
+	if (si->currentmeasurenum < si->lastmeasuremarked){
+	  if(si->cursor_x >= si->firstobjmarked)
+	    return TRUE;
+	  return FALSE;
+	}
       }
-      if (si->currentmeasurenum < si->lastmeasuremarked){
-	if(si->cursor_x >= si->firstobjmarked)
+      if(si->currentmeasurenum > si->firstmeasuremarked) {
+	if (si->currentmeasurenum == si->lastmeasuremarked) {
+	  if((si->cursor_x <= si->lastobjmarked))
+	    return TRUE;
+	  else return FALSE;
+	}
+	if (si->currentmeasurenum < si->lastmeasuremarked)
 	  return TRUE;
-	return FALSE;
       }
     }
-    if(si->currentmeasurenum > si->firstmeasuremarked) {
-      if (si->currentmeasurenum == si->lastmeasuremarked) {
-	if((si->cursor_x <= si->lastobjmarked))
-	  return TRUE;
-	else return FALSE;
-      }
-      if (si->currentmeasurenum < si->lastmeasuremarked)
-	return TRUE;
-    }
   }
   return FALSE;  
 }
@@ -655,19 +686,6 @@ void restore_selection(DenemoScore *si) {
 }
 
 
-void
-goto_nearestnote (GtkAction *action, gpointer param) {
-#if 0
-  if(gui->si->currentobject) {
-    objnode *thenotenode =  nearestnote (gui->si->currentobject->data, gui->si->cursor_y);
-    if(thenotenode) {
-      note *thenote =  *((note *) thenotenode->data);
-      //thenote->mid_c_offset!!!!!!!!!!!
-    }
-
-  }
-#endif
-}
 
 /**
  * goto_mark
@@ -688,10 +706,39 @@ goto_mark (GtkAction *action, DenemoScriptParam *param)
     while(si->cursor_x < si->markcursor_x)
       cursorright(param);
     restore_selection(si);
-    displayhelper(Denemo.gui);
+    if(!action)
+      displayhelper(Denemo.gui);
   } 
 }
 
+/**
+ * goto_selection_start
+ * move cursor the first object in the selection without changing the selection
+ *
+ * 
+ */
+void
+goto_selection_start (GtkAction *action, DenemoScriptParam *param)
+{
+  DenemoScore *si = Denemo.gui->si;
+  if(!action)
+    ((DenemoScriptParam *)param)->status = si->markstaffnum;
+  if(si->markstaffnum){
+    gint first = si->firstobjmarked;
+    save_selection(si);
+    set_currentmeasurenum (Denemo.gui, si->firstmeasuremarked);
+    set_currentstaffnum (Denemo.gui,si->firststaffmarked);
+    while(si->cursor_x < first)
+      cursorright(param);
+    restore_selection(si);
+    if(!action)
+      displayhelper(Denemo.gui);
+  } 
+}
+
+
+
+
 static GSList *positions=NULL;
 DenemoPosition *pop_position(void) {
   DenemoPosition *pos;
@@ -877,6 +924,24 @@ calcmarkboundaries (DenemoScore * si)
 			  si->currentmeasurenum, si->cursor_x, NORMAL_SELECT);
 }
 
+void
+swap_point_and_mark(GtkAction *action, gpointer param) {
+  DenemoScore * si = Denemo.gui->si;
+  gint temp =  si->currentstaffnum;
+  si->currentstaffnum = si->markstaffnum;
+  si->markstaffnum = temp;
+
+  temp =  si->currentmeasurenum;
+  si->currentmeasurenum = si->markmeasurenum;
+  si->markmeasurenum = temp;
+
+  temp =  si->cursor_x;
+  si->cursor_x = si->markcursor_x;
+  si->markcursor_x = temp;
+  setcurrentobject (si, si->cursor_x);
+  calcmarkboundaries (si);
+  displayhelper(Denemo.gui);
+}
 /**
  * undowrapper
  * Wrapper function for the undo command
@@ -934,6 +999,7 @@ undo (DenemoGUI * gui)
       gui->si->currentstaffnum = undo->staffnum;
       gui->si->currentmeasurenum = undo->measurenum;
       setcurrents (gui->si);
+calcmarkboundaries (gui->si);
       gui->si->cursor_x = undo->position;
       if (undo->action == ACTION_INSERT)
 	{
@@ -959,6 +1025,7 @@ undo (DenemoGUI * gui)
 	  g_debug ("Cursor Position %d\n", gui->si->cursor_x);
 
 	  setcurrents (gui->si);
+calcmarkboundaries (gui->si);
 	}
       else if (undo->action == ACTION_CHANGE)
 	{
@@ -1000,6 +1067,7 @@ redo (DenemoGUI * gui)
 	  si->currentmeasurenum = redo->measurenum;
 	  si->cursor_x = redo->position;
 	  setcurrents (si);
+calcmarkboundaries (si);
 
 	  g_debug ("Position after set_currents %d\n", gui->si->cursor_x);
 
@@ -1016,6 +1084,7 @@ redo (DenemoGUI * gui)
 	  si->currentmeasurenum = redo->measurenum;
 	  si->cursor_x = redo->position;
 	  setcurrents (si);
+calcmarkboundaries (si);
 	  object_insert (gui, (DenemoObject *) redo->object);
 
 	  redo->action = ACTION_DELETE;
