@@ -60,6 +60,7 @@ void redowrapper(GtkAction *action, gpointer param);
 void update_undo_info(DenemoScore *si, unre_data *undo);
 void update_redo_info(DenemoScore *si, unre_data *redo);
 void goto_mark (GtkAction *action,  DenemoScriptParam *param);
+void goto_selection_start (GtkAction *action,  DenemoScriptParam *param);
 
 DenemoPosition *pop_position(void);
 void push_position(void);
@@ -70,4 +71,8 @@ DenemoObject *get_point_object(void);
 void save_selection(DenemoScore *si);
 void restore_selection(DenemoScore *si);
 gboolean in_selection(DenemoScore *si);
+
+void
+swap_point_and_mark(GtkAction *action, gpointer param);
+
 #endif
