@@ -24,6 +24,28 @@
 #endif
 
 
+
+void add_font_directory(gchar *fontpath) {
+#ifdef G_OS_WIN32
+  AddFontResource(fontpath);
+#else
+  ;
+#endif
+}
+
+
+
+gboolean run_file_association(gchar *filename) {
+#ifdef G_OS_WIN32
+  g_print("Running ShellExecute %s\n", filename);
+  CoInitializeEx(NULL, COINIT_APARTMENTTHREADED | COINIT_DISABLE_OLE1DDE);
+  return ShellExecute(NULL, NULL, filename, NULL, NULL, 0) > 32/* value above 32 indicating success */;
+#else
+  g_warning("No file assoc code - set pref in externals tab of prefs dialog");
+  return 0;
+#endif
+
+}
 /**
    Popups up the menu named.
  */
@@ -95,14 +117,81 @@ infodialog (gchar * msg)
  * 
  * @return none
  */
+
+
 void
-drawbitmapinverse (GdkPixmap * pixmap, GdkGC * gc, GdkBitmap * mask, gint x,
+drawbitmapinverse_cr (cairo_t * cr, GdkBitmap * mask, gint x,
 		   gint y, gint width, gint height)
 {
-  gdk_gc_set_clip_mask (gc, mask);
-  gdk_gc_set_clip_origin (gc, x, y);
-  gdk_draw_rectangle (pixmap, gc, TRUE, x, y, width, height);
-  gdk_gc_set_clip_mask (gc, NULL);	/* Removes clip mask */
+  cairo_save(cr);
+  gdk_cairo_set_source_pixmap( cr, mask, x,y );//??? bitmap???? asks torbenh
+  cairo_rectangle( cr, x,y, width, height );
+  cairo_fill( cr );
+  cairo_restore( cr );
+}
+void
+drawfetachar (GdkPixmap * pixmap, GdkGC * gc, gunichar uc, gint x, gint y)
+{
+  int len;
+  char utf_string[8];
+  PangoContext *context =
+    gdk_pango_context_get_for_screen (gdk_drawable_get_screen (pixmap));
+  PangoLayout *layout = pango_layout_new (context);
+  PangoFontDescription *desc = pango_font_description_from_string ( "feta26 35px" );
+  pango_context_load_font( context, desc );
+
+  len = g_unichar_to_utf8( uc, utf_string );
+
+  pango_layout_set_font_description (layout, desc);
+  pango_layout_set_text (layout,
+			 utf_string,
+			 len);
+  PangoLayoutLine *line = pango_layout_get_line_readonly( layout, 0 );
+
+  gdk_draw_layout_line (pixmap, gc, x, y, line);
+}
+
+void
+drawfetachar_cr (cairo_t * cr, gunichar uc, double x, double y)
+{
+  int len;
+  char utf_string[8];
+  len = g_unichar_to_utf8( uc, utf_string );
+  utf_string[len] = '\0';
+  cairo_select_font_face( cr, "feta26", CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL );
+  cairo_set_font_size( cr, 35.0 );
+
+
+  cairo_move_to( cr, x,y );
+  cairo_show_text( cr, utf_string );
+}
+
+void drawtext_cr (cairo_t *cr, const char *text, double x, double y, double size)
+{
+  cairo_select_font_face( cr, "Sans", CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_BOLD );
+  cairo_set_font_size( cr, size );
+
+  cairo_move_to( cr, x,y );
+  cairo_show_text( cr, text );
+
+}
+void drawnormaltext_cr (cairo_t *cr, const char *text, double x, double y)
+{
+  drawtext_cr(cr, text, x, y, 9.0);
+
+}
+void drawlargetext_cr (cairo_t *cr, const char *text, double x, double y)
+{
+ drawtext_cr(cr, text, x, y, 18.0);
+}
+void
+setcairocolor (cairo_t * cr, GdkGC * gc)
+{
+  GdkGCValues vals;
+  GdkColor col;
+  gdk_gc_get_values( gc, &vals );
+  gdk_colormap_query_color( gdk_colormap_get_system(), vals.foreground.pixel, &col );
+  gdk_cairo_set_source_color( cr, &col );
 }
 
 /**
@@ -343,6 +432,9 @@ calculateheight (gint mid_c_offset, gint dclef)
     case DENEMO_BASS_CLEF:
       return -LINE_SPACE - HALF_LINE_SPACE * mid_c_offset;
       break;
+    case DENEMO_F_8_CLEF:
+      return -5*LINE_SPACE - HALF_LINE_SPACE * (mid_c_offset - 1);
+      break;
     case DENEMO_TENOR_CLEF:
       return LINE_SPACE - HALF_LINE_SPACE * mid_c_offset;
       break;
