@@ -29,12 +29,6 @@
 #define WHOLE_NUMTICKS 1536
 #define FONT "Sans 9"
 #define TIMESIGFONT "Sans 24"
-/* Wrapper macro for creating bitmaps */
-
-#define bitmaphelper(widget, name) \
-   gdk_bitmap_create_from_data (NULL, \
-                                (gchar *) name##_bits, \
-                                name##_width, name##_height)
 
 /* Adds a callback that processes the "activate" signal coming from
  * a widget */
@@ -53,6 +47,28 @@ void
 drawbitmapinverse (GdkPixmap *pixmap, GdkGC *gc, GdkBitmap *mask,
 		   gint x, gint y, gint width, gint height);
 
+void
+drawbitmapinverse_cr (cairo_t * cr, GdkBitmap * mask, gint x,
+		   gint y, gint width, gint height);
+
+void
+drawfetachar (GdkPixmap * pixmap, GdkGC * gc, gunichar uc, gint x, gint y);
+
+void
+drawfetachar_cr (cairo_t * cr, gunichar uc, double x, double y);
+
+void
+setcairocolor (cairo_t * cr, GdkGC * gc);
+
+void 
+drawnormaltext_cr (cairo_t *cr, const char *text, double x, double y);
+
+void 
+drawlargetext_cr (cairo_t *cr, const char *text, double x, double y);
+
+void 
+drawtext_cr (cairo_t *cr, const char *text, double x, double y, double size);
+
 /* Gives space after a note or rest */
 
 gint
@@ -254,4 +270,7 @@ void console_output(gchar *text);
 
 gint get_override(GList *g);
 
+void add_font_directory(gchar *fontpath);
+
+gboolean run_file_association(gchar *filenam);
 #endif /* UTILS_H */
